;
;   EVENTMGR.ASM
;
;   Event Manager and mouse interface
;
;   Copyright (c) 1992 SofTech Microsystems
;

VERSION  T320
IDEAL                                        ; Invoke IDEAL mode

Include  "EVENTMGR.INC"                      ; Include structure defs

Public   EventMgr__                          ; Make event mgr routines public
Public   MouseMgr__                          ; Make mouse routines public

Extrn    __ParNI:Far                         ; Clipper Extend System crap
Extrn    __RetL:Far
Extrn    __RetNI:Far
Extrn    __XGrab:Far
Extrn    __XFree:Far

Extrn    _ArrayNew:Far                       ; Clipper array creator
Extrn    __cAtPut:Far                        ; Modifies array elements
Extrn    __PutLN:Far                         ; Accesses eval stack

Extrn    __Eval:Word                         ; Eval stack pointer
Extrn    __TOS:Word                          ; Top of eval stack

Group    DGROUP    XIFB,XIF,XIFE             ; Initialization segments
Group    DGROUP    XCFB,XCF,XCFE             ; Termination segments
Group    DGROUP    EventMgrData              ; Static data segment

Segment  EventMgrData         Word      Public    "DATA"

Label    MouseFlag            Byte
DB       0                                   ; Indicates mouse availability

Label    Visible              Byte
DB       0

Label    EventJumpTable       Word           ; Beginning of event jump table
DW       Offset GetEvent
DW       Offset PostEvent
DW       Offset SetEventQueueSize
DW       Offset GetEventQueueSize
DW       Offset PendingEventCount
DW       Offset ClearEventQueue

Label    MouseJumpTable       Word           ; Beginning of mouse jump table
DW       Offset IsMouse
DW       Offset ShowMouseCursor
DW       Offset HideMouseCursor
DW       Offset GetMousePosition
DW       Offset SetMousePosition
DW       Offset SetMouseLimits
DW       Offset SetMouseCursor
DW       Offset MouseVisible

Label    Int9Handler         DWord           ; This contains a pointer to
Int9Ofs  DW        0                         ; the old keyboard interrupt
Int9Seg  DW        0                         ; handler

EQSize   DW        16                        ; Current queue size
EQPend   DW        0                         ; # of pending events
EQSpot   DW        0                         ; Offset within queue buffer

QBuffer  Event     16 Dup( ? )               ; This is the default queue

Label    EventQueue          DWord           ; This contains a pointer to
EQOfs    DW        Offset QBuffer            ; the current event queue
EQSeg    DW        Seg QBuffer

Ends     EventMgrData



Segment  EventMgrCode         Word      Public    "CODE"
         Assume    CS:EventMgrCode,DS:DGROUP

Proc     EventManagerInit    Far

         Mov       AX,3533h                  ; DOS service -- get int vector
         Int       21h                       ; Call DOS
         Mov       AX,ES                     ; Move segment to AX
         Or        AX,BX                     ; Check for null pointer
         JZ        @@Kbd                     ; If so, mouse not present
         Cmp       [Byte Ptr ES:BX],0CFh     ; Does vector point to IRET?
         JE        @@Kbd                     ; If so, mouse not present

         Xor       AX,AX                     ; Mouse call -- reset
         Int       33h                       ; Call mouse driver
         Or        AX,AX                     ; Did reset fail?
         JZ        @@Kbd                     ; If so, cannot use mouse
         Inc       [MouseFlag]               ; All checks OK, set mouse flag

         Mov       AX,12                     ; Mouse call -- install handler
         Mov       CX,30                     ; Set service mask
         Mov       DX,Offset EventMgrCode:MouseHandler
         Push      CS                        ; Move CS . . .
         Pop       ES                        ; . . . to ES
         Int       33h                       ; Call mouse driver

@@Kbd:   Mov       AX,3509h                  ; DOS service -- get int vector
         Int       21h                       ; Call DOS
         Mov       [Int9Ofs],BX              ; Store offset of old vector
         Mov       [Int9Seg],ES              ; Store segment of old vector

         Push      DS                        ; Save DS
         Push      CS                        ; Move CS . . .
         Pop       DS                        ; . . . to DS
         Mov       AX,2509h                  ; DOS service -- set int vector
         Mov       DX,Offset KeyboardHandler ; Load offset of handler
         Int       21h                       ; Call DOS
         Pop       DS                        ; Restore DS
         RetF
Endp     EventManagerInit



Proc     EventManagerDone

         Cmp       [MouseFlag],1
         JB        @@Kbd
         Mov       AX,33
         Int       33h

@@Kbd:   Push      DS
         LDS       DX,[Int9Handler]
         Mov       AX,2509h
         Int       21h
         Pop       DS
         RetF

Endp     EventManagerDone



Proc     KeyboardHandler     Far

         Push      AX                        ; Save AX
         Push      BX                        ; Save BX
         Push      DX                        ; Save DX
         Push      ES                        ; Save ES
         Push      DS                        ; Save DS

         Mov       AX,DGROUP
         Mov       DS,AX
         Mov       AX,40h                    ; Set AX to BIOS data area
         Mov       ES,AX                     ; Point ES to BIOS data area
         Push      [Word Ptr ES:1Ah]         ; Put head pointer on stack
         Push      [Word Ptr ES:1Ch]         ; Put tail pointer on stack

         PushF                               ; Simulate interrupt
         Call      [Int9Handler]

         Pop       BX                        ; Get old tail back
         Pop       AX                        ; Get old head back
         Cmp       AX,[Word Ptr ES:1Ah]      ; Did head change?
         JNE       @@GetKey                  ; If so, retrieve keystroke
         Cmp       BX,[Word Ptr ES:1Ch]      ; Did tail change?
         JE        @@Exit                    ; If not, exit

@@GetKey:Mov       DX,[EQSize]               ; Load size of event queue
         Cmp       DX,[EQPend]               ; Is queue full?
         JE        @@Exit                    ; If so, bail out

         Push      [Word Ptr ES:BX]          ; Put keystroke on stack
         LES       BX,[EventQueue]           ; Load pointer to queue
         Add       BX,[EQSpot]               ; Set pointer to next event
         Mov       [Byte Ptr ES:BX + 0],0    ; Store event ID
         Pop       [Word Ptr ES:BX + 1]      ; Store keystroke

         Inc       [EQPend]                  ; Increment event counter
         Add       [EQSpot],EventSize        ; Increment buffer spot

@@Exit:  Pop       DS
         Pop       ES
         Pop       DX
         Pop       BX
         Pop       AX
         IRet
Endp     KeyboardHandler



Proc     MouseHandler        Far

         Push      BX                        ; Save BX
         Push      ES                        ; Save ES
         Push      DS                        ; Save DS

         Mov       BX,DGROUP
         Mov       DS,BX
         Mov       BX,[EQSize]               ; Load size of event queue
         Cmp       BX,[EQPend]               ; Is queue full?
         JE        @@Exit                    ; If so, bail out

         Mov       BX,CX                     ; Copy column into BX
         Mov       CL,3                      ; Set shift count
         SHR       BX,CL                     ; Divide column by 8
         SHR       DX,CL                     ; Divide row by 8
         Mov       CX,BX                     ; Move column back into CX
         LES       BX,[EventQueue]           ; Load pointer to queue
         Add       BX,[EQSpot]               ; Set pointer to next event
         Mov       [Byte Ptr ES:BX + 0],AL   ; Store button ID
         Mov       [Byte Ptr ES:BX + 1],DL   ; Store row
         Mov       [Byte Ptr ES:BX + 2],CL   ; Store column

         Inc       [EQPend]                  ; Increment event counter
         Add       [EQSpot],EventSize        ; Increment buffer spot

@@Exit:  Pop       DS
         Pop       ES                        ; Restore ES
         Pop       BX                        ; Restore BX
         RetF
Endp     MouseHandler



Proc     EventMgr__          Far
         Assume    DS:DGROUP

         Mov       AX,1                      ; Specify first param
         Call      GetInt                    ; Get param

         SHL       AX,1                      ; Multiply by 2
         Mov       BX,Offset DGROUP:EventJumpTable
         Add       BX,AX                     ; Adjust offset
         Jmp       [Word Ptr BX]             ; Get address
Endp     EventMgr__



Proc     GetEvent  Far

         Cmp       [EQPend],0                ; Is queue empty?
         JNE       @@Array                   ; If not, continue
         RetF                                ; Return NIL

@@Array: Mov       AX,EventSize              ; Specify array size
         Push      AX                        ; Put on stack
         Call      _ArrayNew                 ; Create Clipper array

         Mov       CX,EventSize              ; Specify loop counter
         LES       BX,[EventQueue]           ; Load pointer to event queue
         Add       BX,CX                     ; Point to last byte in event

@@Top:   Dec       BX                        ; Adjust offset
         Push      BX                        ; Save BX
         Push      CX                        ; Save CX
         Push      ES                        ; Save ES
         Mov       AL,[Byte Ptr ES:BX]       ; Get byte
         Mov       AH,0                      ; Clear high byte
         CWD                                 ; Convert to DWORD
         Push      DX                        ; Put high word on stack
         Push      AX                        ; Put low word on stack
         Call      __PutLN                   ; Create entry on eval stack
         Add       SP,4                      ; Realign stack
         Push      [__TOS]                   ; Source VREF pointer on stack
         Push      CX                        ; Element # on stack
         Push      [__Eval]                  ; Target VREF pointer on stack
         Call      __cAtPut                  ; Store array element
         Add       SP,6                      ; Realign stack
         Sub       [__TOS],14                ; Remove entry from eval stack
         Pop       ES                        ; Restore ES
         Pop       CX                        ; Restore CX
         Pop       BX                        ; Restore BX
         Loop      @@Top                     ; Do next byte

         Dec       [EQPend]                  ; Decrement pending event counter
         Sub       [EQSpot],EventSize        ; Adjust offset pointer
         JNZ       @@Move                    ; If none left, don't relocate
         RetF

@@Move:  Push      DS                        ; Save DS
         Push      SI                        ; Save SI
         Push      DI                        ; Save DI

         LES       DI,[EventQueue]           ; Load target pointer
         LDS       SI,[EventQueue]           ; Load source pointer
         Add       SI,EventSize              ; Discard top event
         Mov       CX,[EQPend]               ; Load pending event counter
         SHL       CX,1                      ; Multiply by 2 . . .
         Add       CX,[EQPend]               ; . . . oops, make it three
         Rep       Movsb                     ; Relocate events

         Pop       DI                        ; Restore DI
         Pop       SI                        ; Restore SI
         Pop       DS                        ; Restore DS
         RetF
Endp     GetEvent



Proc     PostEvent Far

         Mov       AX,[EQSize]               ; Get queue size
         Sub       AX,[EQPend]               ; Get pending entries
         JZ        @@Exit                    ; If queue full, exit

         Mov       CX,3                      ; Set array size
@@Top:   Push      CX                        ; Select element #
         Mov       AX,2                      ; Select param #2
         Call      GetInt                    ; Get param
         Pop       CX                        ; Recover element counter
         Push      AX                        ; Save param value for later
         Loop      @@Top                     ; Do next param

         LES       BX,[EventQueue]           ; Load event queue pointer
         Add       BX,[EQSpot]               ; Adjust offset
         Pop       [Word Ptr ES:BX]          ; Get array values from stack
         Pop       [Word Ptr ES:BX + 1]
         Pop       AX
         Mov       [Byte Ptr ES:BX + 2],AL

         Inc       [EQPend]                  ; Increment pending event counter
         Add       [EQSpot],EventSize        ; Adjust queue offset
         Mov       AX,1                      ; Set return value to .T.

@@Exit:  Push      AX                        ; Put return value on stack
         Call      __RetL                    ; Return it to caller
         Pop       AX
         RetF
Endp     PostEvent



Proc     SetEventQueueSize   Far

         Mov       AX,2                      ; Specify second param
         Call      GetInt                    ; Get param
         Mov       [EQSize],AX               ; Store new size

@@ChkOld:Cmp       [EQSeg],Seg QBuffer       ; Is default buffer in use?
         JNE       @@Free                    ; If not, need to deallocate
         Cmp       [EQOfs],Offset QBuffer    ; Is default buffer in use?
         JE        @@New                     ; If so, no need to deallocate

@@Free:  Push      [EQSeg]                   ; Put segment on stack
         Push      [EQOfs]                   ; Put offset on stack
         Call      __XFree                   ; Free the memory
         Add       SP,4                      ; Realign stack

@@New:   Cmp       [EQSize],16               ; Is requested new size <= 16?
         JA        @@Grab                    ; If so, allocate new memory
         Mov       [EQSeg],Seg QBuffer       ; Set segment to default
         Mov       [EQOfs],Offset QBuffer    ; Set offset to default
         Mov       [EQSize],16               ; Make sure size is 16
         Jmp       ClearEventQueue           ; Clear the queue

@@Grab:  Push      [EQSize]                  ; Put size on stack
         Call      __XGrab                   ; Allocate memory
         Add       SP,2                      ; Realign stack
         Mov       [EQSeg],DX                ; Store new segment
         Mov       [EQOfs],AX                ; Store new offset
         Jmp       ClearEventQueue           ; Clear the queue

Endp     SetEventQueueSize



Proc     GetEventQueueSize   Far

         Push      [EQSize]                  ; Put size on stack
         Jmp       ReturnInteger             ; Return it to Clipper app
Endp     GetEventQueueSize



Proc     PendingEventCount   Far

         Push      [EQPend]                  ; Put event count on stack
         Jmp       ReturnInteger             ; Return it to Clipper app
Endp     PendingEventCount



Proc     ClearEventQueue     Far

         Mov       [EQPend],0
         Mov       [EQSpot],0
         RetF
Endp     ClearEventQueue



Proc     MouseMgr__          Far
         Assume    DS:DGROUP

         Cmp       [MouseFlag],1             ; Is mouse available?
         JE        @@Okay
         RetF

@@Okay:  Mov       AX,1                      ; Specify first param
         Call      GetInt                    ; Get param

         SHL       AX,1                      ; Multiply by 2
         Mov       BX,Offset DGROUP:MouseJumpTable
         Add       BX,AX                     ; Adjust offset
         Jmp       [Word Ptr BX]             ; Get address
Endp     MouseMgr__



Proc     IsMouse   Far

         Mov       AL,[MouseFlag]            ; Load mouse flag
         CBW                                 ; Convert to word
         Push      AX                        ; Put logical on stack
         Call      __RetL                    ; Return value to caller
         Pop       AX
         RetF
Endp     IsMouse



Proc     ShowMouseCursor     Far

         Mov       AX,1                      ; Mouse call -- show cursor
         Mov       [Visible],AL
         Int       33h                       ; Call mouse driver
         RetF

Endp     ShowMouseCursor



Proc     HideMouseCursor     Far

         Mov       [Visible],0
         Mov       AX,2                      ; Mouse call -- hide cursor
         Int       33h                       ; Call mouse driver
         RetF

Endp     HideMouseCursor



Proc     GetMousePosition    Far

         Mov       AX,2                      ; Specify array size
         Push      AX                        ; Put size on stack
         Call      _ArrayNew                 ; Create Clipper array

         Mov       AX,3                      ; Mouse call -- get position
         Int       33h                       ; Call mouse driver

         Mov       BX,CX                     ; Move column to BX
         Mov       CL,3                      ; Set shift count
         SHR       BX,CL                     ; Divide by 8
         SHR       DX,CL                     ; Divide by 8
         Xor       CX,CX                     ; Clear CX
         Push      CX                        ; Put zero on stack
         Push      BX                        ; Put column on stack
         Push      CX                        ; Put zero on stack
         Push      DX                        ; Put row on stack
         Call      __PutLn                   ; Create eval stack entry
         Pop       AX
         Pop       AX
         Call      __PutLn                   ; Create eval stack entry
         Pop       AX
         Pop       AX

         Mov       AX,2                      ; Set element #
         Push      CS
         Call      Near @@Store

@@Store: Push      [__TOS]                   ; Source VREF pointer on stack
         Push      AX                        ; Element # on stack
         Push      [__Eval]                  ; Target VREF pointer on stack
         Call      __cAtPut                  ; Store array element
         Add       SP,6                      ; Realign stack
         Sub       [__TOS],14                ; Remove eval stack entry
         Mov       AX,1                      ; Set element # to 1
         RetF
Endp     GetMousePosition



Proc     SetMousePosition    Far

         Mov       AX,3                      ; Specify param #
         Call      GetInt                    ; Get param
         Push      AX                        ; Save for later

         Mov       AX,2                      ; Specify param #
         Call      GetInt                    ; Get param
         Mov       DX,AX                     ; Load row
         Pop       BX                        ; Load column
         
         Mov       CL,3                      ; Set shift count
         SHL       DX,CL                     ; Multiply by 8
         SHL       BX,CL                     ; Multiply by 8
         Mov       CX,BX                     ; Reload column

         Mov       AX,4                      ; Mouse call -- position cursor
         Int       33h                       ; Call mouse driver
         RetF

Endp     SetMousePosition



Proc     SetMouseLimits      Far

         Mov       AX,3                      ; Specify left column
         Call      GetInt                    ; Get param
         Mov       CL,3                      ; Set shift count
         SHL       AX,CL                     ; Multiply by 8
         Push      AX                        ; Save for later
         Mov       AX,5                      ; Specify right column
         Call      GetInt                    ; Get param
         Mov       CL,3                      ; Set shift count
         SHL       AX,CL                     ; Multiply by 8

         Mov       DX,AX                     ; Load DX with right column
         Pop       CX                        ; Load CX with left column
         Mov       AX,7                      ; Mouse call -- horizontal limit
         Int       33h                       ; Call mouse driver

         Mov       AX,2                      ; Specify top row
         Call      GetInt                    ; Get param
         Mov       CL,3                      ; Set shift count
         SHL       AX,CL                     ; Multiply by 8
         Push      AX                        ; Save for later
         Mov       AX,4                      ; Specify bottom row
         Call      GetInt                    ; Get param
         Mov       CL,3                      ; Set shift count
         SHL       AX,CL                     ; Multiply by 8

         Mov       DX,AX                     ; Load DX with top row
         Pop       CX                        ; Load CX with bottom row
         Mov       AX,8                      ; Mouse call -- vertical limit
         Int       33h                       ; Call mouse driver
         RetF
Endp     SetMouseLimits



Proc     SetMouseCursor      Far

         Mov       AX,2                      ; Specify second param
         Call      GetInt                    ; Retrieve it
         Push      AX                        ; Save for later
         Mov       AX,3                      ; Specify third param
         Call      GetInt                    ; Retrieve it
         Mov       DX,AX                     ; Load DX with cursor mask
         Pop       CX                        ; Load DX with screen mask

         Xor       BX,BX                     ; Select software cursor
         Mov       AX,10                     ; Mouse call -- set cursor
         Int       33h                       ; Call mouse driver
         RetF
Endp     SetMouseCursor



Proc     MouseVisible        Far

         Mov       AL,[Visible]
         CBW
         Push      AX
         Call      __RetL
         Pop       AX
         RetF
Endp     MouseVisible



Proc     ReturnInteger       Far

         Call      __RetNI
         Pop       AX
         RetF
Endp     ReturnInteger



Proc     GetInt    Near

         Push      CX
         Push      AX
         Call      __ParNI
         Add       SP,4
         RetN

Endp     GetInt
Ends     EventMgrCode



Segment  XIFB      Word      Public    "DATA"
Ends     XIFB



Segment  XIF       Word      Public    "DATA"

DD       Far Ptr EventManagerInit

Ends     XIF



Segment  XIFE      Word      Public    "DATA"
Ends     XIFE



Segment  XCFB      Word     Public     "DATA"
Ends     XCFB



Segment  XCF       Word      Public    "DATA"

DD       Far Ptr EventManagerDone

Ends     XCF



Segment  XCFE      Word      Public    "DATA"
Ends     XCFE
End

