*:*********************************************************************
*:
*:        Program: HELP.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988, Greg Lief
*:  Last modified: 02/06/89 -- added Help index
*:                 03/03/90 -- added HELPCODE (see HELPDEV.PRG notes)
*:
*:  Creates/displays user-defined help screens
*:
*:  Procs & Fncts: HELP
*:               : SHOWHELP()
*:
*:          Calls: CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:               : EDITFUNC()    (function in $EDITFUN.PRG)
*:               : SHADOWBOX()   (function in $SHADOWBO.PRG)
*:               : ERR_MSG()     (function in $ERRMSG.PRG)
*:
*:           Uses: HELP.DBF
*:        Indexes: HELP.NTX
*:
*:*********************************************************************
EXTERNAL edithelp
PARAM mproc,mline,mvar   && built-in parameters sent upon pressing F1
IF PCOUNT() = 0          && this must be used as a hot key
   RETURN
ENDIF
SET KEY 28 TO showindex           && prevent recursion
PRIVATE wk_area,old_color,buffer,mcurstop,mcursbot,oldins,mid,oldrow,oldcol
wk_area  =SELECT()
old_color=SETCOLOR()
oldrow   = ROW()
oldcol   = COL()
buffer   = SAVESCREEN(0, 0, 24, 79)
mcurstop = Curs_Top()          && get the current cursor starting scan line
mcursbot = Curs_Bot()          && get the current cursor ending scan line
NewCurs(32,0)                  && ditch the cursor without SET CURSOR OFF
SELECT 0
IF ! FILE('help.dbf') .OR. ! FILE('help.dbt')
   Err_msg('Help file missing')
ELSE
   USE help
   IF ! FILE('help.ntx')     && must rebuild help index file
      IF FLOCK()
         waiton("Indexing Database")
         INDEX ON theproc + var TO help
         UNLOCK
      ELSE
         Waiton("Database Is being reindexed... please wait")
         DO WHILE ! FLOCK()
         ENDDO
         UNLOCK
      ENDIF
      waitoff()
   ELSE
      SET INDEX TO help
   ENDIF
   ** seek on HELPCODE first if such a variable is defined
   SEEK IF(TYPE('helpcode') = 'C', strpad(helpcode, 20), ;
        strpad(mproc,10) + strpad(mvar, 10))
   IF FOUND()
      ShowHelp()
   ELSE
      ShowIndex()
   ENDIF
ENDIF
SETCOLOR(old_color)
RESTORE SCREEN FROM buffer
USE
SELECT(wk_area)
NewCurs(mcurstop, mcursbot)  && restore previous cursor
@ oldrow,oldcol SAY ''
SET KEY 28 TO HELP
RETURN
*-------- end of procedure HELP (i need somebody...)


FUNCTION ShowHelp
SETCOLOR(IF(ISCOLOR(), (Color_N2S(boxcolor)), 'W/N'))
Shadowbox(toprow, lt_col, botrow, rt_col, boxno)
mid = lt_col + (INT(rt_col-lt_col)/2)   && calculate midpoint
SETCOLOR(IF(ISCOLOR(), Color_N2S(ftcolor), 'W/N'))
@ botrow, mid-INT(LEN(TRIM(footer))/2) SAY TRIM(footer)
SETCOLOR(IF(ISCOLOR(), Color_N2S(titcolor), 'I'))
@ toprow, mid-INT(LEN(TRIM(title))/2) SAY TRIM(title)
SETCOLOR(IF(ISCOLOR(), Color_N2S(txtcolor), '+W/N'))
MEMOEDIT(TEXT, toprow+1, lt_col+1, botrow-1, rt_col-1, .F., 'EditHelp')
RETURN('')
*-------- end of procedure ShowHelp (not just anybody...)


FUNCTION ShowIndex
PARAM a,b,c
PRIVATE marker, buffer2, help_index[LASTREC()], oldcolor
*** if we got here from a hot-key, restore the original screen first!
IF PCOUNT() = 3
   SET KEY 28 TO
   buffer2 = SAVESCREEN(0, 0, 24, 79)
   RESTSCREEN(0, 0, 24, 79, buffer)    && original screen
ENDIF
marker = RECNO()
GO TOP
FOR ctr = 1 TO LASTREC()
   help_index[ctr] = title + STR(RECNO(),4)
   SKIP
NEXT
ASORT(help_index)
oldcolor = SETCOLOR('+GR/N')
Shadowbox(7, 24, 17, 55, 2, 'Help Index')
Center(17,' Select topic or Esc to abort ')
indexscrn = SAVESCREEN(7, 24, 18, 57)
DO WHILE .T.
   ele = ACHOICE(8, 25, 16, 54, help_index)
   NewCurs(32,0)  && I hate cleaning up after ACHOICE()
   IF ele = 0
      EXIT
   ELSE
      GO VAL(SUBSTR(help_index[ele],31,4))
      RESTSCREEN(0, 0, 24, 79, buffer)    && original screen
      ShowHelp()
      RESTSCREEN(0, 0, 24, 79, buffer)    && original screen
      SETCOLOR('+GR/N')
      RESTSCREEN(7, 24, 18, 57, indexscrn)
   ENDIF
ENDDO
GO marker
*** restore screen and hot key if we got here from a hot key
IF TYPE('buffer2') = 'C'
   RESTSCREEN(0, 0, 24, 79, buffer2)
   SET KEY 28 TO showindex
ENDIF
SETCOLOR(oldcolor)
RETURN []
*-------- end of procedure ShowIndex


*: end of file HELP.PRG
