*:*********************************************************************
*:
*:        Program: HELPDEV.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1989, Greg Lief
*:  Last modified: 03/03/90 - made color variables global so that if
*:                            you want all of your help screens to
*:                            adhere to the same color scheme, you
*:                            only need to mess with the color scheme
*:                            on the first help screen.
*:                          - added HELPCODE support.  This is useful
*:                            when you want to have different help
*:                            screens apply for the same PROC/VAR
*:                            combination.  For example, if you use
*:                            one function to do all of your READs,
*:                            HELPCODE is for you!
*:
*:  Procs & Fncts: HELPDEV
*:               : APPEARANCE
*:               : WHATCOLOR()
*:
*:          Calls: CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:               : EDITFUNC()    (function in $EDITFUN.PRG)
*:               : YES_NO()      (function in $YES_NO.PRG)
*:               : ERR_MSG()     (function in $ERRMSG.PRG)
*:
*:           Uses: TEMPHELP.DBF
*:               : HELP.DBF
*:
*:        Indexes: HELP.NTX
*:
*:     Creates/displays user-defined help screens
*:
*:*********************************************************************

*   Note: this is identical to HELP.prg except for that it allows
*   creation and customization of help screens.  When development
*   is complete, call HELP.prg instead of this program.

EXTERNAL edithelp
PARAM mproc,mline,mvar   && built-in parameters sent upon pressing F1
IF PCOUNT() = 0          && this must be called from a hot key
   RETURN
ENDIF
SET KEY 28 TO            && prevent recursion
PRIVATE wk_area, old_color, buffer, oldins, mid, fixwindow, mtext, ;
        oldrow, oldcol, mcurstop, mcursbot, cursbottom
wk_area=SELECT()
old_color=SETCOLOR()
oldrow = ROW()
oldcol = COL()
buffer = SAVESCREEN(0,0,24,79)
mcurstop = Curs_Top()          && get the current cursor starting scan line
mcursbot = Curs_Bot()          && get the current cursor ending scan line
cursbottom = IF(ISCOLOR(), 7, 13)   && establish bottom cursor scan line
NewCurs(32,0)                  && ditch the cursor without SET CURSOR OFF
SELECT 0
IF ! FILE('help.dbf')
   Waiton('Initializing help database... please wait')
   PRIVATE fname_str, ftype_str, fleng_str
   fname_str = 'THEPROC VAR     TEXT    TOPROW  BOTROW  LT_COL  '
   fname_str = fname_str + 'RT_COL  BOXCOLORTITCOLORTXTCOLORBOXNO   '
   fname_str = fname_str + 'TITLE   FTCOLOR FOOTER  '
   ftype_str = 'CCMNNNNNNNNCNC'
   fleng_str = '1010100202020203030301300330'
   CREATE temphelp
   FOR X = 1 TO 14
      APPE BLANK
      REPLACE field_name with SUBSTR(fname_str, (X-1)*8+1, 8)
      REPLACE field_type with SUBSTR(ftype_str, X, 1)
      REPLACE field_len with VAL(SUBSTR(fleng_str, (X-1)*2+1, 2))
   NEXT
   RELEASE ALL like f?????STR
   CREATE HELP FROM temphelp
   USE HELP EXCLUSIVE
   INDEX on theproc+var TO HELP
   ERASE temphelp.dbf
   Waitoff()
ELSE
   USE help
ENDIF
IF ! FILE('help.ntx')     && must rebuild help index file
   IF FLOCK()
      waiton("Indexing Database")
      INDEX ON theproc + var TO help
      UNLOCK
   ELSE
      Waiton("Database Is being reindexed... please wait")
      DO WHILE ! FLOCK()
      ENDDO
      UNLOCK
   ENDIF
   waitoff()
ELSE
   SET INDEX TO help
ENDIF

** seek on HELPCODE first if such a variable is defined
SEEK IF(TYPE('helpcode') = 'C', strpad(helpcode, 20), ;
     strpad(mproc,10) + strpad(mvar, 10))
IF FOUND()
   SETCOLOR(Color_N2S(boxcolor))
   Shadowbox(toprow, lt_col, botrow, rt_col, boxno)
   mid = lt_col + (INT(rt_col-lt_col)/2)   && calculate midpoint
   SETCOLOR(Color_N2S(titcolor))
   @ toprow, mid-INT(LEN(TRIM(title))/2) SAY TRIM(title)
   SETCOLOR(Color_N2S(ftcolor))
   @ botrow, mid-INT(LEN(TRIM(footer))/2) SAY TRIM(footer)
   SETCOLOR(Color_N2S(txtcolor))
   fixwindow = .T.   && flag to permit editing of help window in edithelp
   MEMOEDIT(TEXT, toprow+1, lt_col+1, botrow-1, rt_col-1, .F., 'edithelp')
ELSE
   SETCOLOR('+W/N')
   @0,0,20,79 BOX 'Ŀ '
   @0,31 SAY '<< HELP SCREEN >>'
   @10,28 SAY 'No help text defined...'
   @12,18 SAY 'Press F2 to create, any other key to return'
   INKEY(0)
   IF LASTKEY()=-1
      mtext=''
      oldins = READINSERT(.T.)    && toggle Insert mode on, save status
      @20,22 SAY ' Press Ctrl-W to save, Esc to quit '
      NewCurs(0, cursbottom)      && cursor on
      mtext = MEMOEDIT(mtext, 1, 1, 19, 78, .T.)
      IF mtext != SPACE(40)
         APPEND BLANK
         IF ! NETERR()
            REPLACE theproc WITH IF(TYPE('helpcode') = 'C', ;
                                    SUBSTR(helpcode, 1, 10), mproc), ;
                    var WITH IF(TYPE('helpcode') = 'C', ;
                                    SUBSTR(helpcode, 11), mvar),     ;
                    text WITH mtext
            @ 20,22 SAY REPLICATE(CHR(196),35)
            NewCurs(32,0)            && cursor off
            DO appearance WITH 'A'
         ELSE
            Err_msg('Network error, unable to add record at this time')
         ENDIF
      ENDIF
   ENDIF
ENDIF
SETCOLOR(old_color)
RESTSCREEN(0,0,24,79,buffer)
USE
SELECT(wk_area)
NewCurs(mcurstop, mcursbot)         && restore previous cursor
@ oldrow,oldcol SAY ''
SET KEY 28 TO helpdev
RETURN
*--------- end of procedure HelpDev


*-------------------------------------------------------------------------
* appearance: allow user to select box coordinates, box type, title,
* and colors for this help screen
*-------------------------------------------------------------------------
PROCEDURE appearance
PARAM mode   && A = creating help screen, E = editing existing screen
PRIVATE xx, yy, fground, bground, colorstr, palette, boxrow, coordloop, ;
        buffer2, redraw, firstloop, palettetop, boxtypes[6]

FOR xx = 1 TO 5
   boxtypes[xx] = SUBSTR(boxtype(xx), 1, 8)
NEXT
boxtypes[6] = 'no border'

* declare the color and position variables as PUBLIC so if you want to
* have the same color scheme for all help screens, you will only need to
* go through that rigmarole for the first help screen! (thanks to Mark
* Langner)
PUBLIC gftoprow, gflt_col, gfbotrow, gfrt_col, gfboxno, gfboxcolor, ;
       gftitcolor, gfftcolor, gftxtcolor, gftitle, gffooter

* if mode is 'A', we are creating a new screen.  However, we must check
* whether or not the global variables have already been initialized.
* if so, they would be of numeric type.  if not, they would still be of
* logical type.  here we go...
*
gftoprow   = IF(mode='A', IF(TYPE('gftoprow')='L', 0, gftoprow), toprow)
gflt_col   = IF(mode='A', IF(TYPE('gflt_col')='L', 0, gflt_col), lt_col)
gfbotrow   = IF(mode='A', IF(TYPE('gfbotrow')='L', 20, gfbotrow), botrow)
gfrt_col   = IF(mode='A', IF(TYPE('gfrt_col')='L', 79, gfrt_col), rt_col)
gfboxno    = IF(mode='A', IF(TYPE('gfboxno')='L', 2, gfboxno), boxno)
gfboxcolor = IF(mode='A', IF(TYPE('gfboxcolor')='L', 7, gfboxcolor), boxcolor)
gftitcolor = IF(mode='A', IF(TYPE('gftitcolor')='L', 112, gftitcolor), titcolor)
gfftcolor  = IF(mode='A', IF(TYPE('gfftcolor')='L', 112, gfftcolor), ftcolor)
gftxtcolor = IF(mode='A', IF(TYPE('gftxtcolor')='L', 15, gftxtcolor), txtcolor)
gftitle    = IF(mode='A', IF(TYPE('gftitle')='L',strpad('Help Screen', 30), ;
                          gftitle), title)
gffooter = IF(mode='A', IF(TYPE('gffooter')='L',strpad(CHR(24) + CHR(25) + ;
                         '  Alt-P print  Esc exit', 30), gffooter), footer)
* are you still with me? good!

** main loop starts here
DO WHILE .T.

   SETCOLOR(old_color)
   buffer2 = SAVESCREEN(21,11,24,68)
   @ 21,13,24,66 BOX 'Ŀ '
   @ 22,15 SAY 'Use arrow keys to drag top left corner of help box'
   @ 23,27 SAY 'Press Enter when finished'

   ** get top left coordinate for help box
   STORE .T. TO firstloop, coordloop
   DO WHILE coordloop
      SETCOLOR(old_color)
      redraw = .T.
      INKEY(0)
      IF firstloop
         RESTSCREEN(21,11,24,68,buffer2)
         firstloop = .F.
      ENDIF
      DO CASE
         ** move down one row
         CASE (LASTKEY() = 24 .AND. gftoprow < 20 .AND. gftoprow < gfbotrow - 4)
            RESTSCREEN(0,0,24,79,buffer)
            gftoprow = gftoprow + 1
         ** move up one row
         CASE LASTKEY() = 5 .AND. gftoprow > 0
            gftoprow = gftoprow - 1
         ** move right one column
         CASE LASTKEY() = 4 .AND. gflt_col < 70 .AND. gflt_col < gfrt_col - 11
            RESTSCREEN(0,0,24,79,buffer)
            gflt_col = gflt_col + 1
         ** move left one column
         CASE LASTKEY() = 19 .AND. gflt_col > 0
            RESTSCREEN(0,0,24,79,buffer)
            gflt_col = gflt_col - 1
         ** Home - reposition at top left corner
         CASE LASTKEY() = 1
            STORE 0 TO gflt_col, gftoprow
         ** Enter - time to move on
         CASE LASTKEY() = 13
            STORE .F. TO coordloop, redraw
         OTHERWISE
            redraw = .F.
      ENDCASE
      IF redraw
         SETCOLOR(Color_N2S(gfboxcolor))
         @ gftoprow,gflt_col,gfbotrow,gfrt_col BOX Boxtype(gfboxno)
         SETCOLOR(Color_N2S(gftxtcolor))
         MEMOEDIT(mtext, gftoprow+1, gflt_col+1, gfbotrow-1, gfrt_col-1, .F., .F.)
      ENDIF
   ENDDO

   ** get bottom right coordinate of help box
   SETCOLOR(old_color)
   @ 21,11,24,68 BOX 'Ŀ '
   @ 22,13 SAY 'Use arrow keys to drag bottom right corner of help box'
   @ 23,27 SAY 'Press Enter when finished'
   STORE .T. TO coordloop, firstloop
   DO WHILE coordloop
      SETCOLOR(old_color)
      redraw = .T.
      INKEY(0)
      IF firstloop
         RESTSCREEN(21,11,24,68,buffer2)
         firstloop = .F.
      ENDIF
      DO CASE
         ** move down one row
         CASE LASTKEY() = 24 .AND. gfbotrow < 24
            gfbotrow = gfbotrow + 1
         ** move up one row
         CASE LASTKEY() = 5 .AND. gfbotrow > 6 .AND. gfbotrow > gftoprow + 4
            RESTSCREEN(0,0,24,79,buffer)
            gfbotrow = gfbotrow - 1
         ** move right one column
         CASE LASTKEY() = 4 .AND. gfrt_col < 79
            gfrt_col = gfrt_col + 1
         ** move left one column
         CASE LASTKEY() = 19 .AND. gfrt_col > 11 .AND. gfrt_col > gflt_col + 11
            RESTSCREEN(0,0,24,79,buffer)
            gfrt_col = gfrt_col - 1
         ** End - reposition at bottom right corner
         CASE LASTKEY() = 6
            gfrt_col = 79
            gfbotrow = 24
         ** Enter - time to move along
         CASE LASTKEY() = 13
            STORE .F. TO coordloop, redraw
         OTHERWISE
            redraw = .F.
      ENDCASE
      IF redraw
         SETCOLOR(Color_N2S(gfboxcolor))
         @ gftoprow,gflt_col,gfbotrow,gfrt_col BOX Boxtype(gfboxno)
         SETCOLOR(Color_N2S(gftxtcolor))
         MEMOEDIT(mtext, gftoprow+1, gflt_col+1, gfbotrow-1, gfrt_col-1, .F., .F.)
      ENDIF
   ENDDO

   ** drag box around
   SETCOLOR(old_color)
   @ 21,14,24,65 BOX 'Ŀ '
   @ 22,16 SAY 'Use arrow keys to drag box to desired location'
   @ 23,27 SAY 'Press Enter when finished'
   STORE .T. TO firstloop, coordloop
   DO WHILE coordloop
      redraw = .T.
      INKEY(0)
      IF firstloop
         RESTSCREEN(21,11,24,68,buffer2)
         buffer2 = SAVESCREEN(gftoprow,gflt_col,gfbotrow,gfrt_col)
         firstloop = .F.
      ENDIF
      DO CASE
         ** Home or PgUp
         CASE LASTKEY()=1 .OR. LASTKEY() = 18
            gfbotrow = gfbotrow - gftoprow
            gfrt_col = gfrt_col - IF(LASTKEY()=1, gflt_col, 0)
            gftoprow = 0
            gflt_col = IF(LASTKEY()=1, 0, gflt_col)
         ** End or PgDn
         CASE LASTKEY()=6 .OR. LASTKEY()=3
            gftoprow = gftoprow + 24 - gfbotrow
            gflt_col = gflt_col + IF(LASTKEY()=6, 79 - gfrt_col, 0)
            gfbotrow = 24
            gfrt_col = IF(LASTKEY()=6, 79, gfrt_col)
         ** arrow key
         CASE (LASTKEY()=5 .AND. gftoprow>0) .OR. (LASTKEY()=24 .AND. gfbotrow<24) ;
         .OR. (LASTKEY()=19 .AND. gflt_col>0) .OR. (LASTKEY()=4 .AND. gfrt_col<79)
            gftoprow = gftoprow - IF(LASTKEY()=24, -1, IF(LASTKEY()=5, 1, 0))
            gflt_col = gflt_col - IF(LASTKEY()=4, -1, IF(LASTKEY()=19, 1, 0))
            gfbotrow = gfbotrow - IF(LASTKEY()=24, -1, IF(LASTKEY()=5, 1, 0))
            gfrt_col = gfrt_col - IF(LASTKEY()=4, -1, IF(LASTKEY()=19, 1, 0))
         ** Enter - time to move along
         CASE LASTKEY() = 13
            STORE .F. TO coordloop, redraw
         OTHERWISE
            redraw = .F.
      ENDCASE
      IF redraw
         RESTSCREEN(0,0,24,79,buffer)
         RESTSCREEN(gftoprow, gflt_col, gfbotrow, gfrt_col, buffer2)
      ENDIF
   ENDDO

   *** redraw header and footer if we are editing an existing help screen
   IF mode = "E"
      Draw_FtHd()
   ENDIF

   ** select box outline
   SETCOLOR(old_color)
   buffer2 = SAVESCREEN(21, 18, 24, 61)
   @ 21,18,24,60 BOX 'Ŀ '
   @ 22,20 SAY 'Use arrow keys to select help box outline'
   @ 23,27 SAY 'Press Enter when finished'
   buff = ShadowBox(10, 35, 17, 44, 2)
   gfboxno = MAX(ACHOICE(11, 36, 17, 43, boxtypes, '', 'pick_box', gfboxno, gfboxno),1)
   ByeByeBox(buff)
   SETCOLOR(Color_N2S(gfboxcolor))
   @ gftoprow, gflt_col, gfbotrow, gfrt_col BOX SUBSTR(Boxtype(gfboxno),1,8)
   Draw_FtHd()

   RESTSCREEN(21,18,24,61,buffer2)

   * have user input title
   SETCOLOR(old_color)
   buffer2 = SAVESCREEN(21,10,24,69)
   @ 21,10,24,69 BOX 'Ŀ '
   @ 22, 12 SAY 'Enter help screen header:'
   @ 23, 12 SAY 'Enter help screen footer:'
   fake_read(22, 38, @gftitle)
   fake_read(23, 38, @gffooter)
   RESTSCREEN(21,10,24,69,buffer2)
   Draw_FtHd()

   * draw color palette based on where the help window is
   palettetop = IF(gftoprow > 8, 00, 14)
   IF TYPE('palette') != 'C'
      palette = ''
      FOR xx = 0 TO 127
         palette = palette + " " + chr(xx) + chr(4) + chr(xx) + " " + chr(xx)
      NEXT
   ENDIF
   RESTSCREEN(palettetop, 16, palettetop+7, 63, palette)
   SETCOLOR('W/N')
   SCROLL(palettetop + 8, 16, palettetop + 8, 63, 00)

   whatcolor('box', @gfboxcolor)
   whatcolor('title', @gftitcolor)
   whatcolor('footer', @gfftcolor)
   whatcolor('text', @gftxtcolor)

   *  okay, show them the monster they have created and ask for confirmation
   RESTSCREEN(0,0,24,79,buffer)
   SETCOLOR(Color_N2S(gfboxcolor))
   Shadowbox(gftoprow, gflt_col, gfbotrow, gfrt_col, gfboxno)
   Draw_FtHd()
   SETCOLOR(Color_N2S(gftxtcolor))
   MEMOEDIT(mtext, gftoprow+1, gflt_col+1, gfbotrow-1, gfrt_col-1, .F., .F.)
   IF Yes_no('Is this satisfactory')
      exit
   ENDIF
ENDDO
DO WHILE ! RLOCK()
ENDDO
REPLACE toprow with gftoprow, botrow with gfbotrow, lt_col with gflt_col, ;
        rt_col with gfrt_col, title with gftitle, footer with gffooter,   ;
        boxcolor with gfboxcolor, titcolor with gftitcolor, ;
        ftcolor with gfftcolor, txtcolor with gftxtcolor, boxno with gfboxno
UNLOCK
RETURN
*--------- end of procedure Appearance


*-------------------------------------------------------------------------
* WhatColor(): pick a color, any color!
*-------------------------------------------------------------------------
FUNCTION WhatColor
PARAM msg, colortype
PRIVATE mrow, mcol, colorloop
mrow = palettetop + INT(colortype / 16)
mcol = 17 + colortype % 16 * 3
STORE .T. TO colorloop, firstloop
SETCOLOR(old_color)
buffer2 = SAVESCREEN(22,06,24,73)
@ 22,06,24,73 BOX 'Ŀ '
@ 23,08 SAY Strpad('Use arrow keys to change ' + msg + ' color ' + ;
                   '- press Enter when finished', 64)

DO WHILE colorloop
   SETCOLOR('*'+Color_N2S(colortype))
   @ mrow,mcol SAY CHR(4)
   INKEY(0)
   IF firstloop
      RESTSCREEN(22,06,24,73,buffer2)
      firstloop = .F.
   ENDIF
   DO CASE
      ** arrow key
      CASE (LASTKEY()=24 .AND. mrow<palettetop+7) .OR. ;
           (LASTKEY()=5 .AND. mrow > palettetop) .OR.  ;
           (LASTKEY()=4 .AND. mcol < 62) .OR. (LASTKEY()=19 .AND. mcol > 17)
         mrow = mrow + IF(LASTKEY() = 24, 1, IF(LASTKEY()=5, -1, 0))
         colortype = colortype + IF(LASTKEY()=24, 16, IF(LASTKEY()=5, -16, ;
                   IF(LASTKEY()=4, 1, -1)))
         mcol = mcol + IF(LASTKEY()=4, 3, IF(LASTKEY()=19, -3, 0))
      ** Enter
      CASE LASTKEY() = 13
         colorloop = .F.
   ENDCASE
   SETCOLOR(Color_N2S(colortype))
   DO CASE

      CASE msg = 'box'         && adjusting the box color
         @ gftoprow, gflt_col, gfbotrow, gfrt_col BOX SUBSTR(Boxtype(gfboxno),1,8)
         Draw_FtHd()

      CASE msg $ 'title'    && adjusting the title color
         @ gftoprow, gflt_col + (INT(gfrt_col-gflt_col)/2) - ;
                    INT(LEN(TRIM(gftitle))/2) SAY TRIM(gftitle)

      CASE msg = 'footer'    && adjusting the footer color
         @ gfbotrow, gflt_col + (INT(gfrt_col-gflt_col)/2) - ;
                    INT(LEN(TRIM(gffooter))/2) SAY TRIM(gffooter)

      CASE msg = 'text'      && adjusting the text color
         MEMOEDIT(mtext, gftoprow+1, gflt_col+1, gfbotrow-1, gfrt_col-1, .F., .F.)
   ENDCASE
   RESTSCREEN(palettetop, 16, palettetop+7, 63, palette)
ENDDO
RETURN []
*--------- end of function WhatColor()


*!*********************************************************************
*!
*1        Function: FAKE_READ()
*!
*!*********************************************************************
FUNCTION Fake_Read
PARAMETERS mrow, mcol, mvar
PRIVATE mlength, oldcolor, key
oldcolor = SETCOLOR('I')
newcurs(6,7)
mlength = 30
mvar = StrPad(mvar, mlength)
key = 0
DO WHILE key != 13 .AND. key != 27
   @ mrow, mcol SAY Strpad(mvar, mlength)
   @ mrow, mcol + LEN(mvar) SAY ''
   key = INKEY(0)
   DO CASE
      CASE key > 31 .AND. LEN(mvar) < mlength
         mvar = mvar + CHR(key)
      CASE key = 8
         mvar = SUBSTR(mvar, 1, LEN(mvar) - 1)
    ENDCASE
ENDDO
newcurs(32,0)
SETCOLOR(oldcolor)
RETURN ('')
*--------- end of function Fake_Read()


*!*********************************************************************
*!
*!       Function: PICK_BOX()
*!
*!*********************************************************************
FUNCTION pick_box
PARAMETERS stat, curr_elem, rel_row
PRIVATE ret_val, oldrow, oldcol, oldcolor, key, buffer

** save screen coordinates
oldrow = ROW()
oldcol = COL()
key = LASTKEY()

ret_val = 2    && default return value -- continue ACHOICE() selection

DO CASE
   CASE stat = 0            && arrow key - redraw box
      buffer = SAVESCREEN(10, 35, 18, 46)
      oldcolor = SETCOLOR(Color_N2S(gfboxcolor))
      @ gftoprow, gflt_col, gfbotrow, gfrt_col BOX SUBSTR(Boxtype(curr_elem),1,8)
      Draw_FtHd()

      SETCOLOR(oldcolor)
      RESTSCREEN(10, 35, 18, 46, buffer)

   CASE stat = 3            && key exception

      IF key = 27           && Esc to abort
         ret_val = 0
      ELSEIF key = 13       && Enter - made selection
         ret_val = 1
      ENDIF
   
ENDCASE
** restore proper screen coordinates
@ oldrow, oldcol SAY ""
RETURN ret_val
*--------- end of function Pick_Box()


*!*********************************************************************
*!
*!       Function: Draw_FtHd() -- redraw header and footer
*!
*!*********************************************************************
FUNCTION Draw_FtHd
PRIVATE mid, oldcolor
oldcolor = SETCOLOR()
mid = gflt_col + (INT(gfrt_col-gflt_col)/2)   && calculate midpoint
* must redraw title and footer because the box just wiped them out!
SETCOLOR(Color_N2S(gftitcolor))
@ gftoprow, mid-INT(LEN(TRIM(gftitle))/2) SAY TRIM(gftitle)
SETCOLOR(Color_N2S(gfftcolor))
@ gfbotrow, mid-INT(LEN(TRIM(gffooter))/2) SAY TRIM(gffooter)
SETCOLOR(oldcolor)
RETURN []
*--------- end of function Draw_FtHd()


*: EOF: HELPDEV.PRG
