*:*********************************************************************
*:
*:        Program: POPCALC.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988, Greg Lief
*:
*:  Procs & Fncts: MEMKEYS()
*:               : PASTE
*:               : PAPER_TAPE()
*:               : DRAWCALCBX()
*:
*:          Calls: CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:               : SHADOWBOX()   (function in $SHADOWB.PRG)
*:
*:      The dreaded pop-up calculator
*:
*:*********************************************************************

PARAM gfproc,line,var
PRIVATE oldcolor, oldscrn, num1, mem, decplace, op, opstr, tnum, hotkey, ;
    mtop, mleft, keypress, mdecimals, mpic, maincolor, mwincol, gfsavedefs, ;
    moldrow, moldcol, papertape, justcalced, mcurstop, mcursbot

** determine whether this was called via hot-key; if so, disable it
IF PCOUNT() > 0
   hotkey = LASTKEY()
   SET KEY hotkey TO
ELSE
   hotkey = 0
ENDIF

** to make use of calculator paste function, variable PASTE_NO must
** be declared public in calling routine
paste_no = IF(TYPE('paste_no')!='N', 0, paste_no)

num = paste_no
mdecimals = 4
SET DECIMALS TO mdecimals
STORE 0 TO num1, mem, op, decplace
opstr = '+-*/^%'
mpic = '###########.####'      && picture string for displaying numbers
STORE .F. TO papertape, justcalced, gfsavedefs

** NUM  = primary number to be manipulated
** NUM1 = second number when operator has been introduced
** MEM  = stored in memory
** OP   = current operator: 1=add, 2=sub, 3=mult, 4=div, 5=exp
** MDECIMALS= number of decimals to display
** DECPLACE = decimal place counter - if decplace is 0, decimal point
** has not been activated; otherwise indicates # of digits to the right of it

** User-configurable color/screen coordinates: GFSAVEDEFS is a flag that
** is set true (.T.) whenever the user changes either the color or the
** screen position of the calculator window.  Current hot-key to change
** color is Alt-F10, although you can easily change this (see line 236
** below).  Press any arrow key, Home, End, PgUp, PgDn to move window.

** determine screen position and color - check for global variables first
maincolor = IF(TYPE('gfcalccol') = 'U', 'W/RB', gfcalccol)
mwincol   = IF(ISCOLOR(), '+W/N', 'I')
mtop      = IF(TYPE('gfcalctop') = 'U',   5, gfcalctop)
mleft     = IF(TYPE('gfcalcleft') = 'U', 28, gfcalcleft)

** save previous environment
moldrow = ROW()
moldcol = COL()
oldcolor = SETCOLOR(maincolor)     && save previous color settings
oldscrn = SAVESCREEN(mtop, mleft, mtop + 12, mleft + 26)
mcurstop = Curs_Top()          && get current cursor starting scan line
mcursbot = Curs_Bot()          && get current cursor ending scan line

NewCurs(32,0)                  && functional equivalent of SET CURSOR off
drawcalcbx()

** main loop
DO WHILE .T.

   ** display M if there is a number currently in memory
   @ mtop+3, mleft+21 SAY IF(mem != 0, 'M', CHR(32))
   @ mtop+3, mleft+4  SAY num PICTURE mpic
   SETCOLOR(maincolor)

   ** display 1st number if an operator has been introduced
   IF num1 != 0
      @ mtop+1,mleft+4  SAY num1 PICTURE mpic
      @ mtop+1,mleft+21 SAY SUBSTR(opstr,op,1)
   ELSE
      @ mtop+1,mleft+4 SAY SPACE(18)
   ENDIF

   keypress = INKEY(0)

   DO CASE

         ** Esc was pressed to quit
      CASE keypress = 27
         paste_no = num
         exit

         ** numeric key was pressed
      CASE keypress > 47 .AND. keypress < 58
         tnum = VAL(CHR(keypress))             && determine numeric value
         DO CASE
            CASE (num=0 .OR. justcalced) .AND. decplace = 0
               num = tnum
               justcalced = .F.
            CASE num<>0 .AND. decplace=0 .AND. num<10000000000   && integer
               num = num * 10 + tnum

            CASE decplace > 0 .AND. decplace <= mdecimals     && real number
               num = num + (tnum / (10 ^ decplace))
               decplace = decplace + 1

            CASE justcalced .AND. decplace <= mdecimals
               num = (tnum / (10 ^ decplace))
               justcalced = .F.

         ENDCASE

         ** backspace was pressed
      CASE keypress = 8 .AND. (num != 0 .OR. decplace > 0)
         IF decplace = 0              && if number is still an integer
            num = INT(num / 10)       && kill the least significant digit
         ELSE
            decplace = decplace - 1
            IF decplace > 0
               num = LTRIM(STR(num, 16, decplace))
               num = VAL(SUBSTR(num, 1, LEN(num) - 1))
            ELSE
               num = 0
            ENDIF
         ENDIF

         ** decimal point (period) was pressed
      CASE keypress = 46 .AND. decplace = 0
         decplace = 1

         ** operator pressed (43='+' 45='-' 42='*' 47='/' 94='^', 37='%')
         ** with no pending operations
      CASE (keypress = 43 .OR. keypress = 45 .OR. keypress = 42 .OR.     ;
              keypress = 47 .OR. keypress = 94 .OR. keypress = 37) .AND. ;
              op=0 .AND. num1=0
         num1 = num
         STORE 0 TO num, decplace
         op = AT(CHR(keypress), opstr)
         justcalced = .F.
         Paper_tape(STR(num1) + ' ' + SUBSTR(opstr, op, 1))

         ** '=', Return or operator was pressed -- perform calculation
      CASE (keypress = 61 .OR. keypress = 13 .OR. keypress = 43 .OR. keypress = 45 ;
           .OR. keypress = 42 .OR. keypress = 47 .OR. keypress = 94) .AND. op>0
         Paper_tape(STR(num) + ' ' + SUBSTR(opstr, op, 1))
         DO CASE
            CASE op = 1            && addition
               num = num + num1
            CASE op = 2            && subtraction
               num = num1 - num
            CASE op = 3            && multiplication
               num = num * num1
            CASE op = 4 .AND. num != 0   && division (don't allow by zero)
               num = num1 / num
            CASE op = 5            && exponent
               num = num1 ^ num
            CASE op = 6            && percentage
               num = num1 / 100 * num
         ENDCASE
         decplace = 0
         op = AT(CHR(keypress), opstr)
         justcalced = .T.
         IF op = 0
            num1 = 0
            Paper_tape(STR(num) + ' =')
            Paper_tape('')
         ELSE
            num1 = num
            num = 0
         ENDIF

         ** 'C' was pressed to clear number
      CASE keypress = 67 .OR. keypress = 99
         STORE 0 TO num, num1, decplace, op

         ** 'D' was pressed to change decimals
      CASE keypress = 68 .OR. keypress = 100
         SETCOLOR('+' + maincolor)
         @ mtop+1, mleft+2 SAY 'How many decimals?'
         keypress = INKEY(0)
         SCROLL(mtop+1, mleft+2, mtop+1, mleft+20, 0)
         IF keypress > 47 .AND. keypress < 58
            mdecimals = VAL(CHR(keypress))
            mpic = REPLICATE('#', 15-mdecimals) + '.' + IF(mdecimals > 0, ;
                             REPLICATE('#', mdecimals), '')
            SET DECIMALS TO mdecimals
         ENDIF

         ** 'R' was pressed to round number (truncate some decimals)
      CASE keypress = 82 .OR. keypress = 114
         SETCOLOR('+' + maincolor)
         @ mtop+1, mleft+2 SAY 'Decimals to round to?'
         keypress = INKEY(0)
         SCROLL(mtop+1, mleft+2, mtop+1, mleft+23, 0)
         IF (keypress > 47 .AND. keypress < 58)
            keypress = VAL(CHR(keypress))
            IF keypress <= mdecimals
               num = ROUND(num, keypress)
            ENDIF
         ENDIF

         ** 'E' was pressed to clear entry
      CASE keypress = 69 .OR. keypress = 101
         STORE 0 TO num, decplace

         ** 'P' was pressed for paper-tape function
      CASE keypress = 80 .OR. keypress = 112
         papertape = (! papertape)
         @ mtop+3, mleft+23 SAY IF(papertape, 'P', ' ')
         Paper_tape('')

         ** 'M' was pressed to access memory functions
      CASE keypress = 77 .OR. keypress = 109
         SETCOLOR('+*' + maincolor)
         memkeys()
         keypress = INKEY(0)  && we need to get another key before taking action
         DO CASE

               ** memory operators
            CASE keypress = 43       && '+' -- add number to memory
               mem = mem + num
            CASE keypress = 45       && '-' -- subtract number from memory
               mem = mem - num
            CASE keypress = 42       && '*' -- multiple memory by number
               mem = mem * num
            CASE keypress = 47       && '/' -- divide memory by number
               mem = mem / num
            CASE keypress = 82 .OR. keypress = 114   && 'R' -- recall memory
               num = mem
            CASE keypress = 67 .OR. keypress = 99    && 'C' -- clear memory
               mem = 0
         ENDCASE
         SETCOLOR('+' + maincolor)
         memkeys()

      CASE keypress = -39            && Alt-F10: change color
         maincolor = ColorPal(maincolor, 16, IF(mleft > 39, 0, 64))
         ** if color was changed, redraw calculator box
         IF LASTKEY() != 27
            SETCOLOR(maincolor)
            drawcalcbx()
            gfsavedefs = .T.
         ENDIF

      CASE keypress < 32             && user might have pressed arrow key
         IF CheckMove(12, 26)
            gfsavedefs = .T.
         ENDIF

   ENDCASE
   SETCOLOR(mwincol)
ENDDO

*** check for changes to screen coordinates and/or color
IF gfsavedefs
   IF yes_no('Save changes to position/color for this session')
      PUBLIC gfcalcleft, gfcalctop, gfcalccol
      gfcalcleft = mleft
      gfcalctop  = mtop
      gfcalccol  = maincolor
   ENDIF
ENDIF

** restore hot-key
IF hotkey != 0
   SET KEY hotkey TO POPCALC
ENDIF

** restore previous environment
SETCOLOR(oldcolor)
RESTSCREEN(mtop, mleft, mtop+12, mleft+26, oldscrn)
SET DECIMALS TO
@ moldrow,moldcol SAY '' 
NewCurs(mcurstop, mcursbot)         && restore previous cursor
RETURN
*--------------------------------------------------------
FUNCTION memkeys  && display keys in memory function box
@mtop+6, mleft+2 SAY 'R + -'
@mtop+7, mleft+2 SAY 'C * /'
RETURN(.T.)
*--------------------------------------------------------
PROCEDURE paste            && to paste current calculator value
PARAM gfproc,line,var      && internal Clipper parameters
IF TYPE('paste_no')='N'    && first make sure it is declared public
   IF TYPE(var) = 'N'
      ** is a database open??
      if fcount() > 0
         private fnames[fcount()]
         afields(fnames)
         if ascan(fnames, var) > 0
            replace &var with paste_no
         else
            &var. = paste_no
         endif
      else
         &var. = paste_no
      endif
   ELSE
      KEYBOARD LTRIM(STR(paste_no))
   ENDIF
ENDIF
RETURN
*-----------------------------------------------------------
FUNCTION paper_tape   && print last number if papertape mode
PARAM mvar
IF papertape .AND. ISPRINTER()
   SET DEVICE TO PRINTER
   @ prow()+1,1 SAY mvar PICTURE mpic
   SET DEVICE TO SCREEN
ENDIF
RETURN ('')
*-----------------------------------------------------------
FUNCTION drawcalcbx    && draw the calculator box, of course
ShadowBox(mtop, mleft, mtop+11, mleft + 24, 3)
@ mtop+2, mleft+2, mtop+4, mleft+22 BOX boxtype(1)
@ mtop+5,  mleft+3 SAY 'emory'
@ ROW()+1, mleft+2 SAY 'R + - '
@ ROW()+1, mleft+2 SAY 'C * / '
@ ROW()+1, mleft+1 SAY REPLICATE(CHR(196), 7) + CHR(217)
@ ROW()+1, mleft+3 SAY 'lear   rint   C ntry'
@ ROW()+1, mleft+3 SAY 'ound   ecimals'
@ mtop+5,  mleft+10 SAY '+  7  8  9  -'
@ ROW()+1, mleft+10 SAY '*  4  5  6  /'
@ ROW()+1, mleft+10 SAY '%  1  2  3  ^'
@ ROW()+1, mleft+13 SAY '=  0  .'
SETCOLOR(mwincol)
@ mtop+3,  mleft+3 SAY SPACE(19)
@ ROW()+2, mleft+2 SAY 'M'
@ ROW()+4, mleft+2 SAY 'C'
@ ROW(),   COL()+6 SAY 'P'
@ ROW(),   COL()+8 SAY 'E'
@ ROW()+1, mleft+2 SAY 'R'
@ ROW(),   COL()+6 SAY 'D'
@ ROW(),  COL()+10 SAY 'Esc'
RETURN('')

*: EOF: POPCALC.PRG
