*:*********************************************************************
*:
*:        Program: POPDATE.PRG
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:                 with special thanks to:
*:                 - TOM WALDEN for the multiple user time blocks,
*:                   weekly summary, and smashing weekly report
*:                 - BOB SUMMERS for the text search and jump-to-
*:                   current-date ideas
*:
*:      Copyright (c) 1989-90, Greg Lief
*:
*:  Procs & Fncts: VIEWAPPTS()
*:               : APPT_UDF()
*:               : LOCK_APPT()
*:               : APPT_HILIT()
*:               : APPT_UDF()
*:               : APPT_MAINT()
*:               : APPT_RECUR()
*:               : APPT_NOTES()
*:               : GETDATE()
*:               : APPT_CDATE()
*:               : APPT_SHOW()
*:               : GRAB_DATES()
*:               : APPT_DAY()
*:               : APPT_WEEK()
*:               : NOCONFLICT()
*:               : APPTPRINTW()
*:               : APPT_HEADW()
*:               : APPTPRINTD()
*:               : APPT_HEADD()
*:               : APPT_MEET()
*:               : APPT_DOM()
*:
*:          Calls: CALENDBOX()    (function  in $CALENDA.PRG)
*:               : NEXTMONTH()    (function  in $CALENDA.PRG)
*:               : CURS_TOP()     (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()     (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()      (Assembler function in CURSNEW.ASM)
*:
*:           Uses: APPT.DBF
*:
*:                 1  USERID       Character     8
*:                 2  DATE         Date          8
*:                 3  TIME         Character     5
*:                 4  ENDTIME      Character     5
*:                 5  BRIEF        Character    50
*:                 6  COMMENTS     Memo         10
*:
*:        Indexes: APPTUSER.NTX (key: USERID + DTOS(DATE) + TIME)
*:                 APPTDATE.NTX (key: DTOS(DATE) + USERID + TIME)
*:
*:*********************************************************************
PARAMETER gfproc, line, var
PRIVATE oldcolor, oldscrn, oldscrn2, maincolor, mrow, mcol, row, tdate, tmonth,;
   trow, tcol, frow, fcol, lrow, lcol, lastday, newrow, newcol, wk_area, xx,   ;
   hotkey, tempdate, moldrow, moldcol, app_ok, mcurstop, mcursbot, redraw,     ;
   firstloop, start, end, foundone, mtop, mleft, calendscrn, cursbottom,       ;
   mtype, mstring, crow, ccol

* CROW & CCOL are the coordinates of the current (system) date
* MROW & MCOL are the coordinates of the current highlighted date
* FROW & FCOL are the coordinates of the first day of the month
* LROW & LCOL are the coordinates of the last day of the month
* NEWROW & NEWCOL are used to macro substitute name of appropriate set
*    of coordinate variables to use from the three pairs listed above
* lastday is numeric value of last day of month (e.g., 29-31)

** determine screen position and color - check for global variables first
maincolor = IF(TYPE('gfapptcol') != 'C', 'W/N', gfapptcol)
mtop      = IF(TYPE('gfappttop') != 'N',     0, gfappttop)
mleft     = IF(TYPE('gfapptleft')!= 'N',     7, gfapptleft)

* save screen, previous coordinates, cursor size, etc.
moldrow  = ROW()
moldcol  = COL()
oldscrn  = SAVESCREEN( 0,  0, 24, 79)
*** if monochrome monitor, clear the entire screen for clarity
IF ! ISCOLOR()
   CLEAR SCREEN               && instead of CLEAR, which also zaps your GETs
ENDIF
oldscrn2 = SAVESCREEN(15, 02, 24, 79)   && buffer underneath appt window
oldcolor = SETCOLOR(maincolor)
mcurstop = curs_top()         && get the current cursor starting scan line
mcursbot = curs_bot()         && get the current cursor ending scan line
cursbottom = IF(ISCOLOR(), 7, 13)   && establish bottom cursor scan line
NewCurs(32,0)                 && functional equivalent of SET CURSOR OFF
mdate = DATE()
firstloop = .T.   && flag so as to only draw calendar frame one time
foundone = .F.    && flag for whether current date has appointments

wk_area = SELECT()            && save current work area
IF PCOUNT() > 0
   hotkey = LASTKEY()
   SET KEY hotkey TO
ELSE
   hotkey = 0
ENDIF
SET DELETED ON
SET SCOREBOARD OFF

*** open appt.dbf... first determine path, then confirm existence of file
SELE 0
mfile = IF(TYPE('apptdir') != 'U', apptdir + '\', '') + 'appt'
IF ! FILE(mfile + '.dbf')
   Waiton('Initializing appointment database... please wait')
   CREATE tempappt
   APPEND BLANK
   REPLACE field_name WITH 'USERID', field_type WITH 'C', field_len WITH 8
   APPEND BLANK
   REPLACE field_name WITH 'DATE', field_type WITH 'D', field_len WITH 8
   APPEND BLANK
   REPLACE field_name WITH 'TIME', field_type WITH 'C', field_len WITH 5
   APPEND BLANK
   REPLACE field_name WITH 'ENDTIME', field_type WITH 'C', field_len WITH 5
   APPEND BLANK
   REPLACE field_name WITH 'BRIEF', field_type WITH 'C', field_len WITH 50
   APPEND BLANK
   REPLACE field_name WITH 'COMMENTS', field_type WITH 'M', field_len WITH 10
   CREATE (mfile) FROM tempappt
   USE
   ERASE tempappt.dbf
   Waitoff()
ENDIF
IF ! FILE(mfile + 'user.ntx') .OR. ! FILE(mfile + 'date.ntx')
   USE (mfile) EXCLUSIVE
   INDEX ON Userid + DTOS(Date) + Time TO (mfile + 'user')
   INDEX ON DTOS(Date) + Userid + Time TO (mfile + 'date')
   USE
ENDIF
USE (mfile) INDEX (mfile + 'date'), (mfile + 'user')

** initial screen setup
SETCOLOR(maincolor)
shadowbox(0, 39, 12, 72, 4)

** draw menu options box
@ 1,       45 SAY 'move by day'
@ 1,       61 SAY 'by week'
@ ROW()+1, 51 SAY 'move by month'
@ ROW()+1, 51 SAY 'first/last day'
@ ROW()+1, 45 SAY 'add/edit'
@ ROW(),   62 SAY 'exit'
@ ROW()+1, 45 SAY 'Search for text'
@ ROW()+1, 45 SAY 'Print appointments'
@ ROW()+1, 45 SAY 'Delete appointments'
@ ROW()+1, 45 SAY 'Daily time blocks'
@ ROW()+1, 45 SAY 'Weekly appt summary'
*@ ROW()+1, 45 SAY 'Schedule a meeting'
@ ROW()+1, 45 SAY 'Current date: ' + DTOC(mdate)
SETCOLOR('+'+maincolor)
@ 0,46 SAY ' CALENDAR OPTIONS '
SETCOLOR('I')
@ 1,       41 SAY CHR(27) + CHR(26)
@ ROW(),   58 SAY CHR(24) + CHR(25)
@ ROW()+1, 41 SAY 'PgUp/PgDn'
@ ROW()+1, 41 SAY 'Home/End'
@ ROW()+1, 41 SAY CHR(17) + CHR(217)
@ ROW(),   58 SAY 'Esc'
@ ROW()+1, 41 SAY 'S'
@ ROW()+1, 41 SAY 'P'
@ ROW()+1, 41 SAY 'D'
@ ROW()+1, 41 SAY 'T'
@ ROW()+1, 41 SAY 'W'
*@ ROW()+1, 41 SAY 'M'
@ ROW()+1, 41 SAY 'C'

calendbox(.T.)

** commence main keypress loop
DO WHILE .T.
   redraw = .F.
   Appt_Show()
   keypress = INKEY(0)
   tdate = mdate                  && store highlighted date
   newrow = 'trow'
   newcol = 'tcol'
   DO CASE

   CASE keypress = 24 .OR. keypress = 5    && go forward/backward one week
      mdate = mdate + IF(keypress = 24, 7, -7)
      trow = mrow + IF(keypress = 24, 1, -1)
      tcol = mcol

   CASE keypress = 19         && go back one day
      mdate = mdate - 1
      ** did we just go from Sunday to Saturday??
      trow = IF(DOW(mdate) = 7, mrow - 1, mrow)
      tcol = IF(DOW(mdate) = 7, mleft + 19, mcol - 3)

   CASE keypress = 4          && go forward one day
      mdate = mdate + 1
      ** did we just go from Saturday to Sunday??
      trow = IF(DOW(mdate) = 1, mrow + 1, mrow)
      tcol = IF(DOW(mdate) = 1, mleft + 1, mcol + 3)

   CASE keypress = 18         && go back one month
      IF MONTH(mdate) = 1     && going to December of previous year
         mdate = STOD(STR( VAL(SUBSTR(DTOS(mdate), 1, 4)) - 1, 4) + ;
                 '12' + SUBSTR(DTOS(mdate), 7) )
      ELSE
         ** check for validity of current date in previous month
         ** i.e., cannot go from March 31 to February 31, etcetera
         tempdate = CTOD('')
         DO WHILE EMPTY(tempdate)
            tempdate = STOD(SUBSTR(DTOS(mdate), 1, 4) + ;
                       IF(MONTH(mdate) < 11, '0', '') + ;
                       LTRIM(STR(MONTH(mdate) - 1)) + SUBSTR(DTOS(mdate), 7))
            mdate = mdate - 1
         ENDDO
         mdate = tempdate
      ENDIF

   CASE keypress = 3          && go forward one month
      mdate = NextMonth(mdate)

   CASE keypress = 1          && go to first day
      newrow = 'frow'
      newcol = 'fcol'
      mdate = STOD(SUBSTR(DTOS(mdate), 1, 6) + '01')

   CASE keypress = 6          && go to last day
      newrow = 'lrow'
      newcol = 'lcol'
      mdate = STOD(SUBSTR(DTOS(mdate), 1, 6) + STR(lastday, 2))

   CASE keypress = 13             && Enter - view appts
      trow = mrow
      tcol = mcol
      viewappts()
      NewCurs(32,0)            && set cursor off again

   CASE keypress = 115 .OR. keypress = 83      && S - text search
      trow = mrow
      tcol = mcol
      PRIVATE finds_[12]
      mtype = YES_NO2("Search through descriptions or notes?", 12, " Descriptions ", " Notes ")
      IF LASTKEY() != 27
         CLEAR GETS
         xx = 0
         mstring = SPACE(20)
         SETCOLOR("+W/RB")
         buffer = ShadowBox(11, 18, 13, 61, 2)
         @ 12, 20 SAY "Text to search for:" GET mstring
         NewCurs(0, cursbottom)     && functional equivalent of SET CURSOR ON
         READ
         NewCurs(32,0)              && functional equivalent of SET CURSOR OFF
         ByeByeBox(buffer)
         IF LASTKEY() != 27 .AND. ! EMPTY(mstring)
            mstring = TRIM(mstring)
            GO TOP
            LOCATE FOR UPPER(mstring) $ UPPER(IF(mtype, BRIEF, COMMENTS))
            DO WHILE FOUND() .AND. xx < 12   && max of 12 finds for now
               finds_[xx + 1] = DTOC(DATE) + [  ] + TIME + [ - ] + ENDTIME + [ ] + BRIEF + STR(RECNO(), 6)
               xx = xx + 1
               CONTINUE
            ENDDO
            IF xx > 0
               SETCOLOR("+W/R")
               buffer = ShadowBox(06, 2, 19, 78, 2, 'Finds for "' + mstring + '"')
               xx = ACHOICE(07, 3, 18, 77, finds_)
               ByeByeBox(buffer)
               NewCurs(32,0)    && SET CURSOR OFF again 'cause of ACHOICE()!
               IF xx > 0
                  mdate = CTOD(SUBSTR(finds_[xx], 1, 8))
                  redraw = .T.    && gotta redo the calendar screen
                  *** if we searched through notes, ask if they want to view 'em
                  IF ! mtype
                     IF Yes_No("View these notes now")
                        GO VAL(SUBSTR(finds_[xx], 75))
                        Appt_Notes()
                     ENDIF
                  ENDIF
               ENDIF

            ELSE
               Err_Msg("No finds")
            ENDIF
         ENDIF
      ENDIF

   CASE keypress = 67 .OR. keypress = 99      && C - jump to system date
      trow = crow
      tcol = ccol
      mdate = DATE()

   CASE keypress = 80 .OR. keypress = 112     && P - print appts
      trow = mrow
      tcol = mcol
      IF RECCOUNT() = 0
         Err_msg('File is currently empty')
      ELSE
         xx = Yes_No2("Print for week or selected dates", 12, " Week ", " Selected dates ")
         IF LASTKEY() != 27
            IF xx
               ApptPrintW()
            ELSE
               grab_dates('Print')
               IF LASTKEY() != 27
                  ApptPrintD()
               ENDIF
            ENDIF
         ENDIF
      ENDIF

   CASE keypress = 68 .OR. keypress = 100     && D - delete appts
      trow = mrow
      tcol = mcol
      IF RECCOUNT() = 0
         Err_msg('File is currently empty')
      ELSE
         grab_dates('Delete')
         IF LASTKEY() != 27
            SET SOFTSEEK ON
            SEEK DTOS(start)
            IF ! EOF()
               xx = RECNO()
               COUNT WHILE DATE>=start .AND. DATE<=end TO numappts
               IF numappts > 0
                  IF Yes_no('You are about to delete ' + LTRIM(STR(numappts)) + ;
                        ' appointments between '+DTOC(start)+' and '+;
                        DTOC(end),'Do you want to continue')
                     GO xx
                     DELETE WHILE DATE>=start .AND. DATE<=end .AND. RLOCK()
                     IF DATE <= end .AND. ! EOF()
                        Err_msg('Only deleted appointments up to '+DTOC(DATE))
                     ENDIF
                     redraw = .T.    && gotta redo the calendar screen
                  ENDIF
               ELSE
                  Err_msg('No appointments fall within the dates ' + ;
                     DTOC(start) + ' and ' + DTOC(end))
               ENDIF
            ENDIF
            SET SOFTSEEK OFF
         ENDIF
      ENDIF

   CASE keypress = 84 .OR. keypress = 116     && T - show time blocks
      Appt_Day()
      LOOP

   CASE keypress = 87 .OR. keypress = 119     && W - show weekly time blocks
      Appt_Week()
      LOOP

 *  CASE keypress = 77 .OR. keypress = 109     && M - schedule meeting
 *     Appt_Meet()
 *     LOOP

   CASE keypress = 27         && Esc - quit
      EXIT

   OTHERWISE                  && any other keystroke
      LOOP

   ENDCASE
   ** if we changed months or a recurring appointment was added, redraw calendar
   foundone = .F.
   IF MONTH(tdate) != MONTH(mdate) .OR. redraw
      calendbox(.T.)
   ELSE
      SETCOLOR('+' + maincolor)
      ** check to see if previous date had appointments (for display purposes)
      SEEK DTOS(tdate)
      IF FOUND()
         SETCOLOR('I')
      ENDIF
      @ mrow,mcol SAY STR(DAY(tdate),2)
      ** check to see if new date has appointments (for display purposes)
      SEEK DTOS(mdate)
      foundone = (FOUND())
      SETCOLOR(IF(foundone, '*I', '*+' + maincolor))
      @ &newrow., &newcol. SAY STR(DAY(mdate),2)
      * store new row/column coordinates for highlighted date
      mrow = &newrow.
      mcol = &newcol.
   ENDIF
ENDDO

SETCOLOR(oldcolor)
RESTSCREEN(0, 0, 24, 79, oldscrn)
IF hotkey != 0
   SET KEY hotkey TO POPDATE
ENDIF
USE
SELECT(wk_area)
@ moldrow,moldcol SAY ''
NewCurs(mcurstop, mcursbot)         && restore previous cursor
RETURN
*--------- end of procedure POPDATE


*!*********************************************************************
*!
*!      Function:  VIEWAPPTS
*!
*!      Called by: POPDATE.PRG                   
*!
*!*********************************************************************
FUNCTION viewappts
PRIVATE fields[1], heads[1], saycolor, numappts, firstrec, boxtop, newday, ;
        oldboxtop, wk_area, recur_appt, buffer1, buffer2

RESTSCREEN(15, 02, 24, 79, oldscrn2)  && remove daily appt window first

buffer1 = SAVESCREEN(22, 00, 24, 79)
saycolor = IF(ISCOLOR(), '+W/BG', '+W/N')

*** note that in the next line we check for the length of the memofield to
*** be longer than two -- this is because sometimes the dreaded CR/LF duo
*** sneaks in there...
fields[1]="USERID+' '+TIME+' '+ENDTIME+' '+BRIEF + ' ' +IF(LEN(TRIM(comments)) > 2, CHR(251), CHR(32))"
heads[1]="Who"+SPACE(6)+"Start  End  Appointment Description" + ;
         SPACE(20) + CHR(251) + " = Notes"
** must determine # of appointments for highlighted date
SEEK DTOS(tdate)
IF FOUND()
   firstrec = RECNO()
   COUNT WHILE DATE=tdate TO numappts
   boxtop = MAX(17 - numappts, 0)
   newday = .F.       && this is not a new date - flag not needed
ELSE
   boxtop = 16
   firstrec = LASTREC() + 1
   newday = .T.       && flag to force adding first appt for a new date
   KEYBOARD CHR(65)   && go immediately into add mode - chr(65) is 'A'
ENDIF
SETCOLOR(oldcolor)
SCROLL(22, 00, 24, 79, 0)
@ 24,05 SAY "add"
@ 24,13 SAY "edit"
@ 24,22 SAY "delete"
@ 24,33 SAY "move"
@ 24,42 SAY "recurring appt"
@ 24,61 SAY "notes"
@ 24,73 SAY "exit"
SETCOLOR('I')
@ 24, 3 SAY "A"
@ 24,11 SAY "E"
@ 24,20 SAY "D"
@ 24,31 SAY "M"
@ 24,40 SAY "R"
@ 24,59 SAY "N"
@ 24,69 SAY "Esc"
DO WHILE LASTKEY() != 27
   GO firstrec
   IF DELETED()  && in case they just deleted 1st appt for this date
      SKIP
   ENDIF
   SETCOLOR(saycolor)
   *** we need to store the current box top row to a temp variable
   *** so that we can restore the screen properly upon exiting DBEDIT()
   *** even if the box top row is changed because of an add or delete
   oldboxtop = MIN(boxtop, 0)
   buffer2 = shadowbox(boxtop, 02, 20, 76, 2)
   DBEDIT(boxtop+1, 03, 19, 75, fields, 'appt_udf', '', heads, '', '')
   ByeByeBox(buffer2)
ENDDO
RESTSCREEN(22, 00, 24, 79, buffer1)
RETURN('')
*--------- end of function ViewAppts()


*!*********************************************************************
*!
*!      Function: APPT_UDF()
*!
*!          Calls: APPT_MAINT()       (procedure in POPDATE.PRG)
*!
*!*********************************************************************
FUNCTION appt_udf
PARAM browmode, indx
PRIVATE mode, ret_val, cur_col, cur_row, keypress, mcomments, mdate, marker, ;
        mtime, mendtime, mbrief
cur_col = COL()
cur_row = ROW()
keypress = LASTKEY()
ret_val = 1
IF browmode < 4
   ** if user hit PgUp or PgDn, rather than go into an endless loop,
   ** just go back to the first record...
   IF keypress = 3 .OR. keypress = 18
      GO firstrec
   ELSEIF DATE != tdate
      SKIP -1
      IF DATE != tdate
         IF BOF()
            SKIP
         ELSE
            SKIP 2
         ENDIF
      ENDIF
      ret_val = 2
   ENDIF

ELSE

   DO CASE
      CASE keypress = 27             && Esc to exit
         ret_val = 0

      CASE keypress = 68 .OR. keypress = 100
         Appt_Hilit()
         IF Yes_no('This appointment will be deleted', 'Continue')
            Lock_Appt(.T.)
         ENDIF

      CASE keypress = 65 .OR. keypress = 97     && A = Add Record
         mode = 'A'
         recur_appt = .F.
         ret_val = Appt_Maint()

      CASE keypress = 77 .OR. keypress = 109    && M = Move appt
         *** the following two variables are used by NoConflict()
         clear gets
         mtime = time
         mendtime = endtime
         Appt_Hilit()
         PRIVATE buffer
         SETCOLOR('+GR/N')
         mdate = CTOD("")
         buffer = shadowbox(21, 26, 23, 53, 2, 'Move appointment')
         @ 22,28 SAY 'Enter new date:' GET mdate
         NewCurs(0, cursbottom)     && functional equivalent of SET CURSOR ON
         READ
         NewCurs(32,0)              && functional equivalent of SET CURSOR OFF
         CLEAR GETS
         ByeByeBox(buffer)
         IF LASTKEY() != 27
            IF NoConflict("E", mdate, userid)
               IF Lock_Appt()
                  redraw = .T.
               ENDIF
            ENDIF
         ENDIF

      CASE keypress = 69 .OR. keypress = 101    && E = Edit Record
         mode = 'E'
         recur_appt = .F.
         ret_val = Appt_Maint()

      CASE keypress = 82 .OR. keypress = 114    && R = Recurring appt
         mode = 'A'
         recur_appt = .T.
         ret_val = Appt_Maint()

      CASE keypress = 78 .OR. keypress = 110    && view notes
         Appt_Hilit()
         Appt_Notes()

   ENDCASE
ENDIF
SETCOLOR(saycolor + ',I')
RETURN(ret_val)
*--------- end of function Appt_Udf()


*!*********************************************************************
*!
*!       Function: LOCK_APPT()  -- used for moving and deleting appts
*!
*!*********************************************************************
FUNCTION lock_appt
PARAMETER kill_it  && parameter passed only when deleting appointment
IF RLOCK()
   IF PCOUNT() = 1                      && deleting
      DELETE
   ELSE                                 && moving
      REPLACE date WITH mdate
   ENDIF
   UNLOCK
   *** if this is the only appointment for this date
   *** force an exit back to calendar to avoid endless loop
   IF boxtop = 16
      KEYBOARD CHR(27)
      foundone = .F.        && so as not to redraw appt window below
   ELSE
      boxtop = boxtop + 1   && adjust box window
   ENDIF
   ret_val = 0
ELSE
   Err_msg('Could not lock file at this time')
ENDIF
RETURN (ret_val = 0)
*--------- end of function Lock_Appt()


*!*********************************************************************
*!
*!     Function: Appt_Hilit() -- "fake" highlight bar used when
*!               deleting, moving, or adding notes to an appointment
*!               so that the user knows which appointment is current!
*!
*!*********************************************************************
FUNCTION Appt_Hilit
PRIVATE oldcolor
oldcolor = SETCOLOR('I')
@ ROW(), COL() SAY USERID + ' ' + TIME + ' ' + ENDTIME + ' ' + BRIEF + ' ' + ;
                   IF(LEN(TRIM(comments)) > 2, CHR(251), CHR(32))
SETCOLOR(oldcolor)
RETURN('')
*--------- end of function Appt_Hilit()


*!*********************************************************************
*!
*!       Function: APPT_MAINT
*!
*!          Calls: APPT_RECUR()    (function in POPDATE.PRG)
*!
*!*********************************************************************
FUNCTION Appt_Maint
PRIVATE oldscrn, muserid, mtime, mendtime, mbrief, ret_val, marker, mstart
mstart = "  :  "
oldscrn = SAVESCREEN(cur_row-1, 0, 24, 79)
muserid  = IF(mode = 'A', SPACE(8), USERID)
mtime    = IF(mode = 'A', mstart, TIME)
mendtime = IF(mode = 'A', mstart, ENDTIME)
mbrief   = IF(mode = 'A', SPACE(50), BRIEF)
ret_val = 2
SETCOLOR(oldcolor)
SCROLL(22, 00, 24, 79, 0)
@ 23,24 SAY 'save edits          exit without saving'
SETCOLOR('I')
@ 23,17 SAY 'Ctrl-W'
@ 23,40 SAY 'Esc'
SETCOLOR('+W/BG,I')
CLEAR GETS
shadowbox(cur_row-1, 1, cur_row+1, 77, 4)
@ cur_row,  3 GET muserid PICTURE '@!' VALID ! empty(muserid)
@ cur_row,  12 GET mtime PICTURE '##:##' ;
      VALID VAL(SUBSTR(mtime,1,2)) < 24 .AND. VAL(SUBSTR(mtime,4)) < 60
@ cur_row, 18 GET mendtime PICTURE '##:##' VALID mendtime = mstart .OR. ;
      (VAL(SUBSTR(mendtime,1,2)) < 24 .AND. VAL(SUBSTR(mendtime,4)) < 60 .AND. ;
      mendtime >= mtime) .OR. LASTKEY() = 5
@ cur_row, 24 GET mbrief
NewCurs(0, cursbottom)        && functional equivalent of SET CURSOR ON
READ
NewCurs(32,0)                 && functional equivalent of SET CURSOR OFF
IF (LASTKEY() !=27 .AND. (VAL(mtime)>0 .OR. ! EMPTY(mbrief))) .AND. ;
                         NoConflict(mode, tdate, muserid)
   IF recur_appt
      marker = RECNO()
      Appt_Recur()
      boxtop = MAX(boxtop - 1, 0)   && adjust box window for new record
      ret_val = 0          && force exit from DBEDIT() to resize window
   ELSE

      IF mode = 'A'      && adding record
         APPEND BLANK
         app_ok = ! NETERR()
         IF ! newday
            boxtop = MAX(boxtop - 1, 0)   && adjust box window for new record
            ret_val = 0          && force exit from DBEDIT() to resize window
         ENDIF
      ELSE
         app_ok = RLOCK()
      ENDIF
      IF app_ok
         *** note that if user left ending time blank, we will replace it
         *** with the starting time
         REPLACE date WITH tdate, time WITH mtime, brief WITH mbrief, ;
                 endtime WITH IF(mendtime = mstart, mtime, mendtime), ;
                 userid WITH muserid
      ELSE
         Err_msg('Could not lock record, not saved at this time')
      ENDIF
      newday = .F.             && reset this flag to False - no longer needed
      UNLOCK
   ENDIF
ELSEIF newday .OR. redraw
   ** what this means is that either the user escaped without adding an appt
   ** for a new day, or else a recurring appointment was just added
   ** in either instance, we want to force an escape from the DBEDIT() back
   ** to the main calendar window
   KEYBOARD CHR(27)
ENDIF
RESTSCREEN(cur_row-1, 0, 24, 79, oldscrn)
RETURN (ret_val)
*--------- end of function Appt_Maint()


*!*********************************************************************
*!
*!       Function: Appt_Recur()
*!
*!      Called by: APPT_MAINT()   (procedure in POPDATE.PRG)
*!
*!          Calls: Appt_CDate()   (function  in POPDATE.PRG)
*!               : NEXTMONTH()    (function  in POPDATE.PRG)
*!
*!*********************************************************************
FUNCTION Appt_Recur
PRIVATE elements, oldscrn, oldcolor, end, start, curr_area, mdom, mmonth
elements = IF(DOW(tdate)>1 .AND. DOW(tdate)<7, 6, 5)
*** we only need the 5th option if current day is, in fact, a weekday
PRIVATE choices[elements]
oldcolor = SETCOLOR('W/RB')
oldscrn  = shadowbox(16, 48, 17 + elements, 72, 2)
mdom = Appt_Dom(tdate)
SETCOLOR('+W/RB')
choices[1] = 'Every ' + CDOW(tdate)
choices[2] = 'Every other ' + CDOW(tdate)
choices[3] = 'Every ' + Appt_CDate(DAY(tdate)) + ' of the month'
choices[4] = 'Every day'
choices[5] = 'Every ' + Appt_CDate(mdom) + ' ' + CDOW(tdate)
IF elements = 6
   choices[6] = 'Every weekday'
ENDIF
@ 16,50 SAY 'Recurring Appointment'
sel = ACHOICE(17, 49, 16 + elements, 71, choices, '', 'sel_udf')
IF sel > 0
   CLEAR GETS
   SETCOLOR(maincolor)
   shadowbox(18, 49, 20, 71, 2)
   SETCOLOR('+' + maincolor)
   end = tdate
   @ 19,50 SAY 'Ending date:' GET end picture '@D' VALID end >= tdate
   NewCurs(0, cursbottom)        && functional equivalent of SET CURSOR ON
   READ
   CLEAR GETS
   NewCurs(32,0)                 && functional equivalent of SET CURSOR OFF
   Waiton('Adding recurring appointments... please wait')
   start = tdate
   DO WHILE start <= end
      IF NoConflict("A", start, muserid, .T.)
         APPEND BLANK
         app_ok = ! NETERR()
         IF app_ok
            REPLACE date WITH start, time WITH mtime, endtime WITH mendtime, ;
                    brief WITH mbrief, userid WITH muserid
            UNLOCK
         ELSE
            Err_msg('Could not lock ' + IF(mode='A','file','record') + ', record' + ;
            ' not ' + IF(mode='A', 'added','edited') + ' at this time')
         ENDIF
      ENDIF
      *** increment date by appropriate interval
      DO CASE
         CASE sel = 1 .OR. sel = 2
            start = start + sel * 7
         CASE sel = 3
            start = NextMonth(start)
         CASE sel = 4
            start = start + 1
         CASE sel = 5
            start = NextMonth(start)
            DO WHILE start <= end
               *** first determine 1st day of next month
               start = stod(substr(dtos(start), 1, 6) + "01")
               *** go to the correct day of the week
               DO WHILE DOW(start) != DOW(tdate)
                  start = start + 1
               ENDDO
               *** add the appropriate number of weeks
               mmonth = CMONTH(start)
               start = start + 7 * (mdom - 1)
               IF CMONTH(start) != mmonth
                  Err_Msg("There is no " + Appt_CDate(mdom) + " " + ;
                          CDOW(tdate) + " in " + mmonth)
               ELSE
                  EXIT
               ENDIF
            ENDDO

         CASE sel = 6
            start = start + IF(DOW(start) < 6, 1, 3)
      ENDCASE

   ENDDO
   redraw = .T.
   Waitoff()
ENDIF
SETCOLOR(oldcolor)
ByeByeBox(oldscrn)
RETURN('')
*--------- end of function Appt_Recur()


*!*********************************************************************
*!
*!       Function: APPT_NOTES()
*!
*!          Calls: MEMEDIT()      (function  in Grumpfish Library)
*!
*!*********************************************************************
FUNCTION Appt_Notes
PARAMETER editing
PRIVATE oldcolor, oldscrn, mcomments
cur_row = IF(TYPE('cur_row') = 'U', 15, cur_row)
oldcolor = SETCOLOR(IF(ISCOLOR(), '+w/rb', '+w/n'))
oldscrn = SAVESCREEN(cur_row+1, 00, 24, 79)
shadowbox(cur_row+1, 00, 24, 79, 2, 'Ctrl-W to save, Esc to exit')
NewCurs(0, cursbottom)        && functional equivalent of SET CURSOR ON
mcomments = MEMOEDIT(comments, cur_row+2, 01, 23, 78, .T., 'memo_udf')
NewCurs(32,0)                 && functional equivalent of SET CURSOR OFF
SETCOLOR(oldcolor)
RESTSCREEN(cur_row+1, 00, 24, 79, oldscrn)
IF LASTKEY() != 27
   IF RLOCK()
      REPLACE comments WITH mcomments
      UNLOCK
   ELSE
      Err_msg('Could not lock record, notes not saved')
   ENDIF
ENDIF
RETURN []
*--------- end of function Appt_Notes()


*!*********************************************************************
*!
*!       Function: MEMO_UDF()
*!
*!      Called by: APPT_NOTES()   (procedure in POPDATE.PRG)
*!
*!*********************************************************************
FUNCTION memo_udf  && Alters "ABORT Y/N" msg if Esc is hit during Memoedit
PARAMETERS stat, line, col
IF LASTKEY() = 27
   NewCurs(32,0)       && functional equivalent of SET CURSOR OFF
   TONE (500,1)
   TONE (500,1)
   SETCOLOR('+W/R')
   shadowbox(cur_row + 2, 29, cur_row + 4, 50, 2)
   @ cur_row + 3, 31 SAY 'NOTES NOT UPDATED!'
   INKEY(2)
ENDIF
RETURN (0)
*--------- end of function Memo_Udf()


*!*********************************************************************
*!
*!       Function: ApptPrintD()
*!
*!      Called by: POPDATE.PRG                   
*!
*!          Calls: PRINTOK()      (function  in Grumpfish Library)
*!               : APPT_HEAD()
*!
*!*********************************************************************
FUNCTION ApptPrintD
PRIVATE noteprint, page, xx
noteprint = Yes_no('Do you want to include comments')
page = 1
WaitOn()
IF printok()
   Appt_HeadD()
   SET SOFTSEEK ON
   SEEK DTOS(start)
   SET SOFTSEEK OFF
   DO WHILE DATE >= start .AND. DATE <= end
      IF PROW() > 59
         Appt_HeadD()
      ENDIF
      IF ! EMPTY(time) .OR. ! EMPTY(brief)   && weed out blanked records
         @ PROW()+2,01  SAY Userid
         @ PROW(),  10  SAY DTOC(DATE)
         @ PROW(),  20  SAY TIME
         @ PROW(),  27  SAY ENDTIME
         @ PROW(),  34  SAY substr(BRIEF, 1, 45)
         IF noteprint    && print memofield also
            lines = MLCOUNT(comments, 50)
            FOR xx = 1 TO lines
               IF PROW() > 59
                  Appt_HeadD()
               ENDIF
               @ PROW()+1, 25 SAY TRIM(MEMOLINE(comments, 50, xx))
            NEXT
         ENDIF
      ENDIF
      SKIP
   ENDDO
   EJECT
ENDIF
SET DEVICE TO SCREEN
Waitoff()
RETURN []
*--------- end of function ApptPrintD()


*!*********************************************************************
*!
*!       Function: Appt_HeadD()
*!
*!      Called by: ApptPrintD()
*!
*!      Heading for appointment report (range of dates)
*!
*!*********************************************************************
FUNCTION Appt_HeadD
@ 0,1 SAY DTOC(DATE())
Center(0, 'Appointments for ' + DTOC(start) + ' through ' + DTOC(end))
@ 0,71 SAY 'Page ' + LTRIM(STR(page))
page = page + 1
@ PROW()+2,01 SAY 'Who'
@ PROW(),  12 SAY 'Date'
@ PROW(),  20 SAY 'Start'
@ PROW(),  28 SAY 'End'
@ PROW(),  47 SAY 'Appointment Description'
@ PROW()+1,01 SAY '---'
@ PROW(),  10 SAY REPLICATE('-',8)
@ PROW(),  20 SAY REPLICATE('-',5) + '  ' + REPLICATE('-',5)
@ PROW(),  34 SAY REPLICATE('-',45)
RETURN []
*--------- end of function Appt_HeadD()


*!*********************************************************************
*!
*!       Function: Appt_CDate()
*!
*!      Called by: Appt_Recur()
*!
*!*********************************************************************
FUNCTION Appt_CDate && returns day # and 'st, th, nd' (e.g., '2nd, 3rd...')
PARAMETER mday
PRIVATE mtag, mtagstr
mtagstr = 'stndrd'
IF (mday > 3 .AND. mday < 21) .OR. (mday > 23 .AND. mday < 31)
   mtag = 'th'
ELSE
   mtag = SUBSTR(mtagstr, (mday % 10 - 1)*2 + 1, 2)
ENDIF
RETURN (LTRIM(STR(mday)) + mtag)
*--------- end of function Appt_CDate()


*!*********************************************************************
*!
*!       Function: Appt_Show()
*!
*!      Called by: POPDATE.PRG                   
*!
*!       Displays appointments for highlighted date while in calendar
*!
*!*********************************************************************
FUNCTION Appt_Show
PRIVATE appts[5], xx, oldcolor
IF foundone
   oldcolor = SETCOLOR(maincolor)
   Shadowbox(15, 02, 23, 77, 1, 'Appointments for ' + DTOC(mdate))
   SEEK DTOS(mdate)
   xx = 16
   DO WHILE mdate = DATE .AND. xx < 23
      @ xx, 03 SAY USERID + ' ' + TIME + ' - ' + ENDTIME + ' ' + BRIEF
      xx = xx + 1
      SKIP
   ENDDO
   *** there are more than seven appointments for this date
   IF mdate = DATE
      @ 23, 37 SAY ' More '
   ENDIF
   SETCOLOR(oldcolor)
ELSE
   RESTSCREEN(15, 02, 24, 79, oldscrn2)
ENDIF
RETURN('')
*--------- end of function Appt_Show()


*!*********************************************************************
*!
*!       Function: GRAB_DATES()
*!
*!      Called by: POPDATE.PRG                   
*!
*!       Get starting and ending date for printing/deleting appts
*!
*!*********************************************************************
FUNCTION grab_dates
PARAM msg          && will be either 'Delete' or 'Print'
PRIVATE buffer
SETCOLOR('+GR/N')
clear gets
STORE tdate TO start, end
buffer = shadowbox(12, 27, 15, 52, 2, msg + ' appointments')
@ 13,29 SAY 'Starting date: '
@ 14,29 SAY 'Ending date: '
@ 13,44 GET start
@ 14,44 GET end VALID end >= start
NewCurs(0,cursbottom)    && functional equivalent of SET CURSOR ON
READ
NewCurs(32,0)            && functional equivalent of SET CURSOR OFF
CLEAR GETS
ByeByeBox(buffer)
RETURN('')
*--------- end of function Grab_Dates()


*!*********************************************************************
*!
*!       Function: Appt_Day()
*!
*!       Displays used and available time blocks for highlighted date
*!
*!*********************************************************************
FUNCTION Appt_Day
PRIVATE buffer, times[65], minutes, xx, yy, zz, mrec, lastuser, oldcolor
AFILL(times, 176)
SEEK DTOS(tdate)
IF FOUND()
   mREC = recno()
   YY   = 0
   lastuser = '%^@'
   DO WHILE DATE == tdate .AND. ! EOF()
      IF lastuser <> userid             && employees for the array size
         yy = yy + 1
         lastuser = userid
      ENDIF
      SKIP
   ENDDO
   GOTO mREC
   ZZ = 0
   oldcolor = SETCOLOR('+GR/N')
   buffer = ShadowBox(0, 1, 4+yy, 77, 2, 'Appointment summary for ' + ;
            DTOC(tdate) + ' (' + CHR(176) + CHR(177) + ;
           ' = available, ' + CHR(219) + ' = used)')
   @  2, 11 SAY '6   7   8   9   10  11  12  1   2   3   4   5   6   7   8   9   10'
   @  3+yy, 11 SAY '6   7   8   9   10  11  12  1   2   3   4   5   6   7   8   9   10'
   DO WHILE DATE == tdate .and. ! EOF()
      muserid = userid
      zz = zz + 1
      ***
      * we will alternate between two different background characters:
      * (ASCII 176 and 177) so that each employee's line is distinct
      *
      AFILL(times, 176 + (zz % 2))
      *
      ***
      DO WHILE muserid == userid .AND. Date == tdate .and. ! EOF()
         IF VAL(left(Time, 2)) > 5 .AND. VAL(SUBSTR(Time, 1, 2)) < 22
            xx = (VAL(left(Time, 2)) - 6) * 4 + 1 + ;
                 INT(VAL(SUBSTR(Time, 4)) / 15)
            times[xx] = 219
         *** determine time differential (in minutes) between starting
         *** and ending times for this appointment
            minutes = VAL(left(endtime,2)) * 60 + VAL(SUBSTR(endtime, 4)) - ;
                      VAL(left(time,2)) * 60 - VAL(SUBSTR(time, 4))
            DO WHILE minutes > 15 .AND. xx <= 64
               xx = xx + 1
               times[xx] = 219
               minutes = minutes - 15
            ENDDO
         ENDIF
         SKIP
      ENDDO
      @ 2 + zz, 2 SAY muserid
      FOR xx = 1 TO 65
          @ 2 + zz, 10 + xx SAY CHR(TIMES[xx])
      NEXT
   ENDDO
   INKEY(0)
   ByeByeBox(buffer)
   SETCOLOR(oldcolor)
ENDIF
RETURN('')
*--------- end of function Appt_Day()


*!*********************************************************************
*!
*!       Function: NoConflict()
*!
*!      Called by: APPT_MAINT()
*!
*!       Ensure that this appt is not creating a conflict with time
*!
*!*********************************************************************
FUNCTION NoConflict
PARAMETER mode, mdate, muserid, xx
*** note: last parameter will be passed only when adding a recurring
*** appointment, so that if there is a conflict, the error message will
*** also display the date for the user's benefit
PRIVATE ret_val, marker
marker = RECNO()
ret_val = .T.
SET ORDER TO 2   && make USERID primary search field
GO TOP
SEEK muserid + DTOS(mdate)
DO WHILE mdate = date .AND. userid == muserid .AND. ! EOF()
   IF (mode = 'A' .OR. marker != RECNO()) .AND. ( (mtime >= time .AND. ;
          mtime < endtime) .OR. (mendtime > time .AND. mendtime <= endtime) ;
          .OR. (mtime <= time .AND. mendtime >= endtime) )
      ret_val = .F.
      EXIT
   ENDIF
   SKIP
ENDDO
SET ORDER TO 1   && switch back to DATE as primary search field
GO marker
IF ! ret_val
   Err_msg('This appointment creates a conflict' + IF(PCOUNT() = 4, ;
           ' on ' + DTOC(mdate), '') + ' and thus cannot be added')
ENDIF
RETURN ret_val
*--------- end of function NoConflict()


*!*********************************************************************
*!
*!       Function: Appt_Week()
*!
*!       Displays summary of morning/afternoon appts for the next week
*!
*!*********************************************************************
Function Appt_Week
PRIVATE mfile, xx, yy, zz, num_recs, pDATE, nDATE, mcol, mrow, ;
        oldcolor, buffer1, buffer2, wk_area
oldcolor = SETCOLOR("+GR/N")
wk_area = select()
mcol  = 11

*** we only want to look at this week, so reset date to Monday.
*** note: if we are on a weekend day, we want to look at the upcoming week.
pDATE = DOW(tDATE)
DO CASE
   CASE pDATE = 1
      pDATE = tDATE + 1
   CASE pDATE = 7
      pDATE = tDATE + 2
   OTHERWISE
      pDATE = tDATE - (pDATE - 2)
ENDCASE

mfile = RandFile("APPT")
copy to (mfile) for DATE >= pDATE .and. DATE < pDATE + 5
select 0
USE (mfile) EXCLUSIVE
PACK
IF LASTREC() > 0
   SET UNIQUE ON
   INDEX ON userid TO (mfile)
   COUNT TO num_recs
   SET UNIQUE OFF
   INDEX ON userid + DTOS(date) + time TO (mfile)
   GO TOP
   buffer1 = ShadowBox(0, 1, 3+num_recs, 77, 1, 'Weekly appointment summary (' +;
      CHR(176) + CHR(177) + ' = available, ' + CHR(219) + ' = used)')
   nDATE = pDATE
   buffer2 = REPLICATE('',12)

   FOR xx = 0 TO 4
      @ 1, mcol + (xx * 13) + 1 SAY LEFT(CDOW(nDATE), 3) + ' ' + ;
                                    STR(MONTH(nDATE), 2) + '/' + ;
                                    IF(DAY(nDATE) < 10, '0', '') + ;
                                    LTRIM(STR(DAY(nDATE), 2))
      @ 2, mcol + (xx * 13) SAY buffer2
      nDATE = nDATE + 1

      IF DOW(nDATE) = 1
         nDATE = nDATE + 1
         mcol  = mcol + 2
      ELSEIF dow(nDATE) = 7
         nDATE = nDATE + 2
         mcol  = mcol + 2
      ENDIF
   NEXT

   muserid = userid

   FOR mrow = 1 TO num_recs
      IF muserid <> userid
         muserid = userid
      ENDIF

      mcol =  11
      zz   = -1
      @ mrow + 2, 2 SAY muserid

      IF EOF()
         EXIT
      ENDIF

      FOR yy = 0 to 6
         if userid != muserid .OR. EOF()
            muserid = userid
            exit
         endif
         muserid = userid

         *** if we are on a Sunday, find the next weekday record and
         *** leave an add'l space on screen to denote this break
         IF DOW(pDATE + yy) = 1
            DO WHILE DATE = pDATE + yy
               SKIP
            ENDDO
            mcol = mcol + 2
            LOOP
         ENDIF

         *** if we are on a Saturday, find the next weekday record
         IF DOW(pDATE + yy) = 7
            DO WHILE DATE = pDATE + yy
               SKIP
            ENDDO
            LOOP
         ENDIF
         zz = zz + 1
         PRIVATE times[12]
         *** again, alternate between two different background characters
         *** (ASCII 176 and 177) so that each employee's line is distinct
         AFILL(times, 176 + (mrow % 2))
         IF DATE = pDATE + yy
            DO WHILE DATE = pDATE + yy .AND. muserid = userid .AND. ! EOF()
               IF VAL(left(Time, 2)) > 5
                  mstart = VAL(left(Time, 2)) - 5
                  *** the MIN() function ensures that we won't blow up our
                  *** array with a subscript greater than 12
                  mend = MIN(VAL(left(Endtime, 2)) - 5, 12)
               ENDIF
               FOR xx = mstart TO mend
                  times[xx] = 219
               NEXT
               SKIP
            ENDDO
            FOR xx = 1 TO 12
               @ mrow + 2, mcol + (zz*13) + xx - 1 SAY CHR(times[xx])
            NEXT
         ENDIF
      NEXT
   NEXT
   SETCOLOR(oldcolor)
   USE
   INKEY(0)
   ByeByeBox(buffer1)
ELSE
   Err_Msg("No appointments for the week of " + DTOC(pDATE) + " - " + ;
           DTOC(pDATE + 4))
ENDIF
ERASE (mfile + '.dbf')
ERASE (mfile + '.dbt')
ERASE (mfile + '.ntx')
SELECT(wk_area)
RETURN('')
*--------- end of function Appt_Week()


*!*********************************************************************
*!
*!       Function: ApptPrintW()
*!
*!      Called by: POPDATE.PRG                   
*!
*!          Calls: PRINTOK()      (function  in Grumpfish Library)
*!               : APPT_HEAD()
*!
*!*********************************************************************
FUNCTION ApptPrintW
PRIVATE page, xx, buffer, muserid, mstart, mFILE, N, D, num_recs, wk_area, ;
        mDBF, mCOL, mdate
mstart  = tdate - (DOW(tdate) - 2)
muserid = 'ALL     '
SETCOLOR('+GR/N')
buffer = ShadowBox(12, 27, 14, 52, 2)
@ 13, 29 say 'Employee ID:' GET muserid PICTURE "@!" VALID ! EMPTY(muserid)
NewCurs(0,cursbottom)    && functional equivalent of SET CURSOR ON
READ
NewCurs(32,0)            && functional equivalent of SET CURSOR OFF
ByeByeBox(buffer)
IF LASTKEY() != 27
   Waiton()
   mDBF = RandFile("APPT")
   wk_area = select()
   mCOL  = 11
   if muserid = 'ALL'
      copy to (mDBF) for DATE >= mSTART .and. DATE < mSTART + 5
   else
      copy to (mDBF) for DATE >= mSTART .and. DATE < mSTART + 5 .and. ;
                                              userid == muserid
   endif
   select 0
   USE (mDBF) EXCLUSIVE
   pack
   if LASTREC() = 0
      err_msg('No records found!')
   ELSE
      SET UNIQUE ON
      INDEX ON Userid to (mDBF)
      COUNT TO num_recs
      SET UNIQUE OFF
      INDEX ON userid + dtos(Date) + Time to (mDBF)
      go top
      IF PrintOK()
         PAGE = 0
         Appt_HeadW()
         aROW   = 7
         MAXROW = 0
         private aDATA[150]
         for xx = 1 to num_recs
            muserid = userid
            MAXROW = 0
            afill (aDATA,space(13))
            do while muserid = userid .and. ! eof()
               mDATE = DATE
               D     = 1
               aDAY  = dow(DATE) - 1
               aDATA[aDAY] = ' ' + TIME + '-' + ENDTIME
               do while mDATE = DATE .and. muserid = userid .and. !eof()
                  mBRIEF = BRIEF
                  D     = D + 1
                  aDATA[D * 5 - (5-aDAY)] = SUBSTR(mbrief, 1, 13)

                  ** the following block is to spread the description
                  ** over several lines
                  *do while !empty(mBRIEF)
                  *   D = D + 1
                  *   mFILE = alltrim(parse(@mBRIEF,'/'))
                  *   aDATA[D*5-(5-aDAY)] = mFILE + space(13-len(mFILE))
                  *enddo

                  skip
                  if mDATE = DATE .and. muserid = userid
                     D = D + 1
                     aDATA[D * 5 - (5-aDAY)] = replicate('',13)
                     D = D + 1
                     aDATA[D * 5 - (5-aDAY)] = ' ' + TIME + '-' + ENDTIME
                  endif
               enddo
               if D > MAXROW
                  MAXROW = D
               endif
            enddo
            @ aROW,0 say '' + muserid + ''
            for N = 1 to 5
               @ aROW, (N * 14) - 4 say aDATA[N]
               if N < 5
                  @ aROW, (N * 14) + 9 say ''
               endif
            next
            puserid = '        '
            for N = 6 to (maxrow * 5)
               D = N % 5
               IF D = 0
                  D = 5
               endif
               if D = 1
                  @ aROW, 79 say ''
                  aROW = aROW + 1
                  if aROW > 56
                     Appt_HeadW()
                     aROW = 7
                     puserid = '  ' + muserid + '   '
                  endif
                  @ aROW,0 say puserid
                  puserid = '        '
               else
                  @ aROW,(D*14)-5 say ''
               endif
               @ aROW,(D*14)-4 say left(aDATA[N],13)
            next
            @ aROW, 79 say ''
            aROW = aROW + 1
            if xx < num_recs .AND. ! EOF()
               if aROW > 52
                  Appt_HeadW()
                  aROW = 7
               else
                  @ aROW,0  say '' + replicate('',4) + ;
                                '͹'
                  aROW = aROW + 1
               endif
            ENDIF
         NEXT
         @ aROW,0 say '' + replicate('',4) + ;
                      'ͼ'
         EJECT
      ENDIF
      set device to screen
   ENDIF
   Waitoff()
   use
   select(wk_area)
   erase (mDBF + '.dbf')
   erase (mDBF + '.dbt')
   erase (mDBF + '.ntx')
ENDIF
RETURN []
*--------- end of function ApptPrintW()


*!*********************************************************************
*!
*!       Function: Appt_HeadW()
*!
*!      Called by: ApptPrintW()
*!
*!      Heading for appointment report (weekly)
*!
*!*********************************************************************
FUNCTION Appt_HeadW
IF page > 0
   @ aROW,0 say '' + replicate('',4) + 'ͼ'
   EJECT
ENDIF
page = page + 1
@ 1, 1 SAY DTOC(DATE())
Center(1, 'Appointments for ' + dtoc(mstart) + ' through ' + dtoc(mstart + 4))
@ 1,73 SAY 'Page ' + LTRIM(STR(page))
@ 4, 0 SAY '' + replicate('',4) + 'ͻ'
@ 5, 0 SAY ' EMP ID '
for N = 1 to 5
   @ 5, N * 14 - 3 SAY LEFT(CDOW(mstart + N - 1), 3) + '-'
   @ 5, N * 14 + 2 say STR(DAY(mstart + N - 1), 2) + ' ' + ;
                       LEFT(CMONTH(mstart + N - 1), 3)
   IF N < 5
      @ 5,(N*14)+9 say ''
   ENDIF
NEXT
@ 5, 79 say ''
@ 6,  0 say '' + replicate('',4) + '͹'
RETURN []
*--------- end of function Appt_HeadW()


*!*********************************************************************
*!
*!     Function: Appt_Dom()
*!
*!    Called by: Appt_Recur()
*!
*!    Determine which day of month, e.g., 2nd Monday, 3rd Tuesday, etc
*!
*!*********************************************************************
FUNCTION Appt_Dom
PARAMETER mdate
PRIVATE mday, ret_val, mmonth
mday = CDOW(mdate)
ret_val = 0
mmonth = month(mdate)
DO WHILE month(mdate) = mmonth
   mdate = mdate - 7
   ret_val = ret_val + 1
ENDDO
RETURN ret_val

*: end of file: POPDATE.PRG
