*:*********************************************************************
*:
*:        Program: POPPHONE.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-89, Greg Lief
*:  Last Modified: 10/22/89 -- rewrote auto-dialer in Clipper to circumvent
*:                             inconsistent performance of previous ASM module
*:                             NOTE: DOS MODE command will probably have to
*:                             be called prior to using this!
*:                 12/02/89 -- added communications parameters
*:                 12/03/89 -- added redial
*:                 12/03/89 -- added long-distance parameter for
*:                             long-distance numbers in same area code
*:                 12/03/89 -- added search feature
*:                 03/03/90 -- added another modem parameter: "timing delay"
*:                             necessary because some modems like to wait
*:                             longer than others between commands (thanks
*:                             to Dan Johnson for his assistance with this!)
*:                          -- added pocket-size phone directory print option
*:
*:  Procs & Fncts: PHONEMAINT()
*:               : ADDR_MAINT()
*:               : PRINTPHONE()
*:               : PH_HEAD()
*:               : DIALER()
*:               : ESC_MSG()
*:               : MODEMSETUP()
*:               : PHONESRCH()
*:               : PHONEBOOK()
*:               : FRONTPAGE()
*:
*:          Calls: ERR_MSG()     (function in $ERRMSG.PRG)
*:               : YES_NO()      (function in $YESNO.PRG)
*:               : HELPACT()     (function in $HELPBRO.PRG)
*:               : PRINTOK()     (function in $PRINTOK.PRG)
*:               : CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:               : SHADOWBOX()   (function in $SHADOWB.PRG)
*:               : WAITON()      (function in $WAITON.PRG)
*:               : WAITOFF()     (function in $WAITON.PRG)
*:               : DBEDITSRCH()  (function in $HELPACT.PRG)
*:
*:           Uses: TEMPHONE.DBF
*:               : PHONE.DBF
*:               : GFPHONE.DBF
*:               : NEWPHONE.DBF
*:
*:        Indexes: PHONE.NTX
*:
*:*********************************************************************

** F7 is default hot key for this module

PARAM gfproc,line,var
PRIVATE oldscrn, oldcolor, fldnames, fldlens, wk_area, hotkey, ;
        moldrow, moldcol, mcurstop, mcursbot, redialnum, cursbottom

** save previous screen coordinates
moldrow = ROW()
moldcol = COL()

IF PCOUNT() > 0
   hotkey = LASTKEY()
   SET KEY hotkey TO
ELSE
   hotkey = 0
ENDIF
** if phonedir is predefined, use that as the search path for phone.dbf
mfile = IF(TYPE('phonedir')<>'U',phonedir+'\','')+'phone'
CLEAR GETS
SET SCOREBOARD OFF

** either pull in pre-saved configuration or recreate defaults now
IF FILE('gfphone.mem')
   RESTORE FROM gfphone.mem ADDITIVE
ELSE
   gfphonpbx  = .F.                 && PBX flag
   gfphonhang = StrPad('ATH', 20)   && hang-up sequence
   gfphoninit = StrPad('ATZ', 20)   && modem initialization sequence
   gfphontone = 'T'                 && T = Tone, P = Pulse
   gfphonport = '1'                 && COM port to use
   gfphonarea = SPACE(3)            && User's local area code
   gfphontime = 1.0                 && Modem timing delay between commands
   SAVE ALL LIKE gfphon* TO gfphone.mem
ENDIF

oldscrn = SAVESCREEN(0, 0, 24, 79)
wk_area = SELECT()
oldcolor = SETCOLOR()
mcurstop = Curs_Top()         && get the current cursor starting scan line
mcursbot = Curs_Bot()         && get the current cursor ending scan line
cursbottom = IF(ISCOLOR(), 7, 13)   && establish bottom cursor scan line

*** create phone.dbf if it does not already exist
IF ! FILE(mfile+'.dbf')
   Waiton('Initializing phone database... please wait')
   SELE 0
   CREATE temphone
   fldnames='FNAME   LNAME   PHONE_NOEXT     COMPANY ADDRESS CITY    STATE   '+;
            'ZIP     LONGDIST'
   fldlens='15151204202515021001'
   FOR X = 1 TO 10
      APPE BLANK
      REPLACE field_name with SUBSTR(fldnames, (X-1)*8 + 1, 8)
      REPLACE field_type with IF(X < 10, 'C', 'L')
      REPLACE field_len with VAL(SUBSTR(fldlens,(X-1)*2 + 1, 2))
   NEXT
   CREATE (mfile) FROM temphone
   USE
   ERASE temphone.dbf
   Waitoff()
ENDIF

SELE 0
IF ! FILE(mfile + '.ntx')
   USE (mfile) EXCLUSIVE
   INDEX ON UPPER(lname+fname) TO (mfile)
ENDIF
USE (mfile) INDEX (mfile)

PRIVATE fields[1], heads[1], saycolor, editcolor
saycolor = IF(ISCOLOR(), '+W/R', '+W/N')
editcolor = IF(ISCOLOR(), '+W/N', 'I')
fields[1]="StrPad(TRIM(FNAME)+' '+LNAME,32)+COMPANY+' '+PHONE_NO+'  '+EXT+" +;
          "' ' + IF(LONGDIST,'Y','N')"
heads[1]="CONTACT NAME" + SPACE(20) + "COMPANY NAME         PHONE NUMBER  EXT LD"
SCROLL(22, 00, 24, 79, 0)
@ 22,06 SAY 'address'
@ 22,20 SAY 'phoneBook'
@ 22,37 SAY 'list'
@ 22,48 SAY 'dial'
@ 22,60 SAY 'redial'
@ 22,74 SAY 'setup'
SETCOLOR('I')
@ 22,00 SAY 'Alt-A'
@ 22,14 SAY 'Alt-B'
@ 22,31 SAY 'Alt-L'
@ 22,42 SAY 'Alt-P'
@ 22,54 SAY 'Alt-R'
@ 22,68 SAY 'Alt-S'
@ 24,01 SAY "A"
@ 24,12 SAY "E"
@ 24,24 SAY "D"
@ 24,38 SAY "S"
@ 24,52 SAY CHR(24)+CHR(25)
@ 24,68 SAY "Esc"
SETCOLOR(oldcolor)
@ 24,03 SAY "Add"
@ 24,14 SAY "Edit"
@ 24,26 SAY "Delete"
@ 24,40 SAY "Search"
@ 24,55 SAY "to move"
@ 24,72 SAY "to exit"
SETCOLOR(saycolor)
ShadowBox(0, 31, 2, 45, 6)
@ 01,32 SAY 'PHONE NUMBERS'
shadowbox(05, 02, 20, 76, 1)
DBEDIT(06, 03, 19, 75, fields, 'phone_udf', '', heads, '', '')
USE
RESTSCREEN(0, 0, 24, 79, oldscrn)
SETCOLOR(oldcolor)
SELECT(wk_area)
IF hotkey != 0
   SET KEY hotkey TO POPPHONE
ENDIF
@ moldrow,moldcol SAY ''
NewCurs(mcurstop, mcursbot)         && restore previous cursor
RETURN
*-------- end of procedure POPPHONE


*-------------------------------------------------------------
* Phone_UDF(): user-defined function for use w/ DBEDIT() above
*-------------------------------------------------------------
FUNCTION phone_udf
PARAM browmode, indx
PRIVATE mode, keypress, ret_val, cur_col, cur_row, oldscrn
SETCOLOR('I')
cur_col = COL()
cur_row = ROW()
ret_val = 1
keypress = LASTKEY()
IF browmode > 3                       && keystroke exception
   DO CASE
      CASE keypress = 27             && Esc to exit
         ret_val = 0

      CASE keypress = 65 .OR. keypress = 97   && Add Record
         mode = 'A'
         phonemaint()

      CASE keypress = 69 .OR. keypress = 101  && Edit Record
         mode = 'E'
         phonemaint()

      CASE keypress = 68 .OR. keypress = 100  && Delete Record
         IF Yes_no('This record will be deleted from the file',;
                   'Do you want to do this')
            IF RLOCK()
               DELETE
               SKIP -1
            ELSE
               Err_msg('Unable to lock record, record not deleted')
            ENDIF
         ENDIF
         ret_val = 2

      CASE keypress = 83 .OR. keypress = 115  && Search for record
         ret_val = phonesrch()   && which currently returns 2 to refresh screen

      CASE keypress = 286  && Alt-A -- view/edit address
         addr_maint()

      CASE keypress = 294  && Alt-L -- print list
         printphone()

      CASE keypress = 304  && Alt-B -- phone book
         phonebook()

      CASE keypress = 281  && Alt-P -- dial phone #
         Dialer(phone_no)

      CASE keypress = 275  && Alt-R -- redial
         IF TYPE('redialnum') = 'U'
            Err_msg('No redial number has been established yet')
         ELSE
            Dialer(redialnum)
         ENDIF

      CASE keypress = 287  && Alt-S -- modem setup
         modemsetup()

   ENDCASE
ENDIF
SETCOLOR(saycolor+','+editcolor)
RETURN(ret_val)
*-------- end of function Phone_Udf()


*------------------------------------------------------------
* Phone_Maint(): add/edit records in phone.dbf
*------------------------------------------------------------
FUNCTION phonemaint   &&for adding/editing records (if necessary)
PRIVATE mfname, mlname, mcompany, mphone_no, mext, locked_ok, ;
        oldscrn, buffer
oldscrn = SAVESCREEN(cur_row-1, 1, cur_row+2, 79)
Esc_Msg()
mfname    = IF(mode = 'A', SPACE(15), fname)
mlname    = IF(mode = 'A', SPACE(15), lname)
mcompany  = IF(mode = 'A', SPACE(20), company)
mphone_no = IF(mode = 'A', gfphonarea + SPACE(9), phone_no)
mext      = IF(mode = 'A', SPACE(4) , ext)
mlongdist = IF(mode = 'A', .T., longdist)
SETCOLOR(oldcolor)
@ 22,00 SAY SPACE(79)
SETCOLOR(saycolor + ',' + editcolor)
ShadowBox(cur_row-1, 1, cur_row+1, 77, 4)
@ cur_row-1, 3 SAY '(first)'
@ cur_row-1,19 SAY '(last)'
@ cur_row, 3 GET mfname PICTURE "!XXXXXXXXXXXXXX"
@ cur_row,19 GET mlname PICTURE "!XXXXXXXXXXXXXX"
@ cur_row,35 GET mcompany PICTURE "!XXXXXXXXXXXXXXXXXXX"
@ cur_row,56 GET mphone_no PICTURE 'XXX-XXX-XXXX'
@ cur_row,70 GET mext PICTURE 'XXXX'
@ cur_row,75 GET mlongdist PICTURE 'Y'
NewCurs(0, cursbottom)      && functional equivalent of SET CURSOR ON
READ
NewCurs(32,0)               && functional equivalent of SET CURSOR OFF
IF LASTKEY() != 27
   IF mode='A'
      APPEND BLANK
   ENDIF
   locked_ok = IF(mode = 'A', ! NETERR(), RLOCK())
   IF locked_ok
      REPLACE fname WITH mfname,lname WITH mlname,phone_no WITH mphone_no, ;
              ext WITH mext,company WITH mcompany, longdist WITH mlongdist
      UNLOCK
   ELSE
      Err_Msg('Could not lock ' + IF(mode='A','file','record') + ', record' + ;
              ' not ' + IF(mode='A', 'added','edited') + ' at this time')
   ENDIF
ENDIF
SETCOLOR(saycolor+','+editcolor)
RESTSCREEN(22, 00, 24, 79, buffer)
RESTSCREEN(cur_row-1, 1, cur_row+2, 79, oldscrn)
RETURN('')
*-------- end of function Phone_Maint()


*------------------------------------------------------------
* Addr_Maint(): add/edit address information
*------------------------------------------------------------
FUNCTION addr_maint
PRIVATE maddress, mcity, mstate, mzip, mphone_no, mext, oldscrn, buffer
oldscrn = SAVESCREEN(6, 12, 19, 68)
Esc_Msg()
maddress = address
mcity = city
mstate = state
mzip = zip
mphone_no = phone_no
mext = ext
SETCOLOR(IF(ISCOLOR(), '+W/B', '+W/N'))
ShadowBox(06, 12, 18, 66, 2, "ADDRESS INFORMATION")
@ 08,15 SAY 'Name:    '+TRIM(fname)+' '+lname
@ 10,15 SAY "Company: "+company
@ 12,15 SAY "Address:"
@ 14,15 SAY "City:                     St:     Zip:"
@ 16,15 SAY 'Phone:   '+phone_no+'        Ext: '+ext
@ 12,24 GET maddress PICTURE "!XXXXXXXXXXXXXXXXXXXXXXXX"
@ 14,24 GET mcity PICTURE "!XXXXXXXXXXXXXX"
@ 14,45 GET mstate PICTURE "!!"
@ 14,54 GET mzip PICTURE "@!"
NewCurs(0, cursbottom)      && functional equivalent of SET CURSOR ON
READ
CLEAR GETS
NewCurs(32,0)               && functional equivalent of SET CURSOR OFF
IF LASTKEY() != 27
   IF RLOCK()
      REPLACE address WITH maddress, city WITH mcity, state WITH mstate, ;
              zip WITH mzip
   ELSE
      Err_msg('Record in use, edits not saved')
   ENDIF
ENDIF
RESTSCREEN(22, 0, 24, 79, buffer)
RESTSCREEN(6, 12, 19, 68, oldscrn)
RETURN('')
*-------- end of function Addr_Maint()


*------------------------------------------------------------
* PrintPhone(): print phone list
*------------------------------------------------------------
FUNCTION printphone
PRIVATE page, buffer, morder
phonehead = IF(TYPE('phonehead') = 'U', 'Phone List', phonehead)
STORE 1 TO morder, page
SETCOLOR('+W/RB')
buffer = ShadowBox(19, 10, 22, 69, 2, 'List Options')
@ 20,12 SAY 'Sort list by (1) last name, (2) company, (3) phone no'
@ 21,22 SAY '(Press Esc to exit without printing)'
@ 20,68 GET morder PICTURE '#'
NewCurs(0, cursbottom)      && functional equivalent of SET CURSOR ON
READ
NewCurs(32,0)               && functional equivalent of SET CURSOR OFF
CLEAR GETS
ByeByeBox(buffer)
IF LASTKEY() <> 27
   IF morder = 2
      INDEX ON UPPER(company) TO phonetmp
   ELSEIF morder = 3
      INDEX ON phone_no TO phonetmp
   ENDIF
   GO top
   Waiton()
   IF PrintOk()
      ph_head()
      DO WHILE .NOT. EOF()
         IF PROW() > 59
            ph_head()
         ENDIF
         @ PROW()+2,1 SAY company
         @ PROW(),24  SAY TRIM(fname) + ' ' + lname
         @ PROW(),58  SAY phone_no
         @ PROW(),74  SAY ext
         @ PROW()+1,1 SAY address
         @ PROW(),31  SAY TRIM(city) + ' ' + state + '  ' + zip
         SKIP
      ENDDO
      EJECT
   ENDIF
   Waitoff()
   SET DEVICE TO SCREEN
   SET INDEX TO (mfile)
   ERASE phonetmp.ntx
ENDIF
RETURN('')
*-------- end of function PrintPhone()


*------------------------------------------------------------
* Ph_Head(): heading for phone list report
*------------------------------------------------------------
FUNCTION ph_head
@ 0,1 SAY DTOC(DATE())
Center(0, phonehead)
@ 0,71 SAY 'Page ' + LTRIM(STR(page))
page = page + 1
@ PROW()+2,1 SAY 'Company Name'
@ PROW(),24  SAY 'Contact Name'
@ PROW(),58  SAY 'Phone Number'
@ PROW(),74  SAY 'Ext.'
@ PROW()+1,1 SAY 'Address'
@ PROW(),31  SAY 'City  State  Zip'
@ PROW()+1,1 SAY REPLICATE('-',79)
RETURN('')
*-------- end of function Ph_Head()


*------------------------------------------------------------
* Esc_Msg(): general status message for when adding/editing
*------------------------------------------------------------
FUNCTION Esc_Msg
buffer = SAVESCREEN(22, 00, 24, 79)
SETCOLOR(oldcolor)
SCROLL(22, 00, 24, 79, 0)
@ 23,25 SAY 'save edits'
@ 23,43 SAY 'exit without saving'
SETCOLOR('I')
@ 23,18 SAY 'Ctrl-W'
@ 23,39 SAY 'Esc'
RETURN('')
*-------- end of function Esc_Msg()


*------------------------------------------------------------
* Dialer(): self-explanatory
*------------------------------------------------------------
FUNCTION Dialer
PARAMETER numtodial
PRIVATE numbr, xx, yy, oldcolor, handle, buffer, mareacode
redialnum = numtodial   && save this number for redialing
mareacode = SUBSTR(numtodial, 1, 3)
*** see if this area code matches the local area code
IF mareacode = gfphonarea
   numtodial = IF(longdist, '1-', '') + SUBSTR(numtodial, 5)
ENDIF
numtodial = LTRIM(TRIM(numtodial))

*** clean up the phone number
DO CASE
   CASE LEN(numtodial) = 12
      numtodial = '1-' + numtodial
   CASE SUBSTR(numtodial, 1, 1)='-'
      numtodial = SUBSTR(numtodial, 2)
ENDCASE
yy = LEN(numtodial)
numbr = ''
** convert any letters in phone number to their numeric equivalent
FOR xx = 1 TO yy
   DO CASE
      CASE SUBSTR(numtodial, xx, 1) $ '0123456789'
         numbr = numbr + SUBSTR(numtodial, xx, 1)
      CASE SUBSTR(numtodial, xx, 1) $ 'ABC'
         numbr = numbr + '2'
      CASE SUBSTR(numtodial, xx, 1) $ 'DEF'
         numbr = numbr + '3'
      CASE SUBSTR(numtodial, xx, 1) $ 'GHI'
         numbr = numbr + '4'
      CASE SUBSTR(numtodial, xx, 1) $ 'JKL'
         numbr = numbr + '5'
      CASE SUBSTR(numtodial, xx, 1) $ 'MNO'
         numbr = numbr + '6'
      CASE SUBSTR(numtodial, xx, 1) $ 'PRS'
         numbr = numbr + '7'
      CASE SUBSTR(numtodial, xx, 1) $ 'TUV'
         numbr = numbr + '8'
      CASE SUBSTR(numtodial, xx, 1) $ 'WXY'
         numbr = numbr + '9'
      CASE SUBSTR(numtodial, xx, 1) = '-'
         ** nada
      OTHERWISE
         err_msg('Invalid phone number')
         RETURN (.F.)
   ENDCASE
NEXT

** add the requisite half second pause if using a PBX system
numbr = IF(gfphonpbx, "9 ~ ", "") + numbr

SET PRINTER TO ('com' + gfphonport)
oldcolor = SETCOLOR('+GR/N')
buffer = ShadowBox(19, 12, 22, 67, 2)
@ 21,14 SAY "Wait for ring, then pick up phone and press spacebar"
SETCOLOR('+' + SETCOLOR())
Center(20,"Dialing " + numtodial)
SETCOLOR(oldcolor)
SET PRINT ON
SET CONSOLE OFF
?? '+++' + CHR(13)
INKEY(gfphontime)
?? TRIM(gfphoninit) + CHR(13)
INKEY(gfphontime)
?? 'ATD' + gfphontone + numbr + CHR(13)
INKEY(0)
?? TRIM(gfphonhang) + CHR(13)
SET CONSOLE ON
SET PRINT OFF
ByeByeBox(buffer)
SET PRINTER TO
RETURN('')
*-------- end of function Dialer()


*------------------------------------------------------------
* ModemSetup(): establish/edit modem parameters
*------------------------------------------------------------
FUNCTION modemsetup
PRIVATE oldscrn, prevcolor, buffer
Esc_Msg()
prevcolor = SETCOLOR('+W/RB')
oldscrn = ShadowBox(08, 18, 16, 61, 2, 'Communications parameters')
gfphonhang = StrPad(gfphonhang, 20)
gfphoninit = StrPad(gfphoninit, 20)
@ 09, 20 SAY 'PBX (Y/N)'
@ 10, 20 SAY 'Modem set-up string'
@ 11, 20 SAY 'Modem hang-up string'
@ 12, 20 SAY '[T]one / [P]ulse'
@ 13, 20 SAY 'COM port'
@ 14, 20 SAY 'Local area code'
@ 15, 20 SAY 'Timing delay'
@ 09, 41 GET gfphonpbx PICTURE "Y"
@ 10, 41 GET gfphoninit
@ 11, 41 GET gfphonhang
@ 12, 41 GET gfphontone PICTURE '!' VALID gfphontone $ 'PT'
@ 13, 41 GET gfphonport VALID gfphonport $ '12345'
@ 14, 41 GET gfphonarea PICTURE '###'
@ 15, 41 GET gfphontime PICTURE '#.#'
NewCurs(0, cursbottom)      && functional equivalent of SET CURSOR ON
READ
NewCurs(32,0)               && functional equivalent of SET CURSOR OFF
gfphonhang = TRIM(gfphonhang)
gfphoninit = TRIM(gfphoninit)
IF LASTKEY() != 27
   IF yes_no('Save these settings permanently')
      SAVE ALL LIKE gfphon* TO gfphone.mem
   ENDIF
ENDIF
ByeByeBox(oldscrn)
RESTSCREEN(22, 00, 24, 79, buffer)
SETCOLOR(prevcolor)
RETURN ('')
*-------- end of function ModemSetup()


*------------------------------------------------------------
* PhoneSrch(): search for a name in phone.dbf
*------------------------------------------------------------
FUNCTION phonesrch
PRIVATE prevcolor, buffer, oldscrn
prevcolor = SETCOLOR()
oldscrn = SAVESCREEN(05, 00, 05, 79)  && area where search name will be shown
buffer = SAVESCREEN(22, 00, 24, 79)
SETCOLOR(oldcolor)
SCROLL(22, 00, 24, 79, 0)
@ 22, 11 SAY 'Type in last name to search for, press Enter when finished'
SETCOLOR(saycolor)
DBEditSrch(06, 03, 19, 75, fields, heads, 5, 40)
SETCOLOR(prevcolor)
RESTSCREEN(05, 00, 05, 79, oldscrn)
RESTSCREEN(22, 00, 24, 79, buffer)
RETURN (2)
*-------- end of function PhoneSrch()


*------------------------------------------------------------
* PhoneBook(): prints pocket-sized phone directory
*------------------------------------------------------------
FUNCTION PhoneBook
PARAMETER mastername
PRIVATE xx, yy, mletter, newletter, mname, mcompany, ;
        mcity, curr_line, pages, maxrec, mpage, mastername, laserjet

mastername = IF(PCOUNT() = 0, "", mastername)
laserjet = yes_no2("What type of printer are you using?", 12, " HP Laserjet ", " IBM Graphics ")
IF LASTKEY() != 27
   WaitOn("Creating phone directory")
   *** create temporary file to hold formatted output for printing
   SELECT 0
   CREATE dumbo
   APPEND BLANK
   REPLACE field_name WITH 'descrip', field_type WITH 'C', field_len WITH 45
   CREATE printit FROM dumbo
   ERASE dumbo.dbf
   USE printit
   SELECT phone
   GO TOP
   mletter = CHR(32)
   STORE 1 TO curr_line, pages
   DO WHILE ! EOF()
      newletter = .F.   && flag set true if alpha category changes
      *** determine person and company names to use for this entry
      IF LEN(TRIM(lname)) > 0
         mname = LTRIM(TRIM(lname)) + ;
                 IF(LEN(TRIM(lname)) > 0, ', ' + LTRIM(TRIM(fname)), [])
         mcompany = space(5) + LTRIM(TRIM(company))
      ELSE
         mname = LTRIM(TRIM(company))
         mcompany = []
      ENDIF
      mname = mname + SPACE(32 - LEN(mname)) + ;
              IF(VAL(phone_no) = 0, "   No Number", phone_no)
      maddress = space(5) + LTRIM(TRIM(address))
      mcity = space(5) + LTRIM(TRIM(city)) + " " + LTRIM(TRIM(state)) + " " + ;
              LTRIM(TRIM(zip))
      lines = 1
      *** did alphabetical category change?
      IF mletter != SUBSTR(lname, 1, 1)
         mletter = SUBSTR(lname, 1, 1)
         lines = lines + 1
         newletter = .T.
      ENDIF
      ** determine how many lines this entry will need, so that we
      ** can thus determine whether or not they will fit on this page
      lines = lines + IF(LEN(mcompany) > 1, 1, 0) + ;
                      IF(LEN(maddress) > 1, 1, 0) + ;
                      IF(LEN(mcity) > 3, 1, 0)
      SELECT printit
      *** this entry will go on next page - add blank lines to complete this page
      IF lines + curr_line > 22
         pages = pages + 1
         FOR xx = curr_line TO 22
            APPEND BLANK
         NEXT
         curr_line = 1
      ENDIF
      *** if we started a new alpha category, must put the heading in now
      IF newletter
         APPEND BLANK
         curr_line = curr_line + 1
         REPLACE descrip WITH SPACE(20) + '- ' + mletter + ' -'
      ENDIF
      APPEND BLANK
      curr_line = curr_line + 1   && waiting for 5.0's increment operator!
      REPLACE descrip WITH mname
      IF LEN(mcompany) > 1
         APPEND BLANK
         curr_line = curr_line + 1
         REPLACE descrip WITH mcompany
      ENDIF
      IF LEN(maddress) > 1
         APPEND BLANK
         curr_line = curr_line + 1
         REPLACE descrip WITH maddress
      ENDIF
      IF LEN(mcity) > 3
         APPEND BLANK
         curr_line = curr_line + 1
         REPLACE descrip WITH mcity
      ENDIF
      SELECT phone
      SKIP
   ENDDO
   SELECT printit
   *** add enough blank records in temp file to make last page full length
   IF curr_line < 22
      *** also, it will be a hell of a lot easier if we have an even number
      *** of pages, so we'll check on that right now
      mlines = (23 - curr_line) + IF(pages%2 != 0, 22, 0)
      *** increment number of pages if we previously had an odd number
      pages = pages + IF(pages % 2 != 0, 1, 0)
      FOR xx = 1 TO mlines
         APPEND BLANK
      NEXT
   ENDIF
   GO TOP

   *** now begins the arduous task of printing the book
   IF PrintOK()
      @ PROW(),PCOL() say if(laserjet, chr(27) + "&k2S", chr(15))
      maxrec = LASTREC() / 2
      mpage = 0
      FrontPage()
      DO WHILE RECNO() < maxrec
         mpage = mpage + 1
         FOR xx = 1 TO 27
            @ PROW()+1, 0 SAY "."
            IF xx = 1 .OR. xx = 27
               FOR yy = 1 TO 46
                  @PROW(), PCOL()+1 SAY "."
               NEXT
            ELSE
               IF xx = 26
                  @ PROW(), 22 SAY mpage PICTURE '###'
                  @ PROW(), 68 SAY pages + 1 - mpage PICTURE '###'
               ELSE
                  IF xx > 2 .AND. xx < 25
                     @ PROW(), 2 SAY descrip
                  ENDIF
                  IF (xx > 8 .AND. xx < 12) .OR. (xx > 16 .AND. xx < 20)
                     @ PROW(), 47 SAY "|"
                  ENDIF
                  IF xx > 2 .AND. xx < 25
                     marker = RECNO()
                     GO LASTREC() - (22 * (INT(RECNO() / 22) + 1)) + (RECNO()%22)
                     @ PROW(), 49 SAY descrip
                     GO marker
                     SKIP
                  ENDIF
               ENDIF
            ENDIF
            @ PROW(), 94 SAY "."
         NEXT
         ** since we are putting two pages per sheet, only eject every other time
         IF mpage%2 != 0
            EJECT
         ENDIF
      ENDDO
      @ prow(),pcol() say if(laserjet, chr(27) + "&k0S", chr(18))
      EJECT
      SET DEVICE TO SCREEN
   ENDIF
   USE
   ERASE printit.dbf
   SELECT phone
   WaitOff()
ENDIF
RETURN []
*-------- end of function PhoneBook()


*------------------------------------------------------------
* FrontPage(): print phone page of phone directory
*------------------------------------------------------------
FUNCTION FrontPage
FOR xx = 1 TO 27
   @ PROW()+1, 0 SAY "."
   DO CASE
      CASE xx = 1 .OR. xx = 27
         FOR yy = 1 TO 46
            @PROW(), PCOL()+1 SAY "."
         NEXT
      *** draw lines for staples in middle of page
      CASE (xx > 8 .AND. xx < 12) .OR. (xx > 16 .AND. xx < 20)
         @ PROW(), 47 SAY "|"
   ENDCASE
   DO CASE
      CASE xx = 11
         @ PROW(), 61 SAY "Telephone Directory"
      CASE xx = 13
         @ PROW(), 69 SAY "for"
      CASE xx = 15
         @ PROW(), 48 + INT((48 - LEN(mastername)) / 2) SAY mastername
      CASE xx = 17
         mdate = CMONTH(DATE()) + ' ' + LTRIM(STR(day(DATE()))) + ', ' + ;
                 LTRIM(STR(YEAR(DATE())))
         @ PROW(), 48 + INT((48 - LEN(mdate)) / 2) SAY mdate
   ENDCASE
   @ PROW(), 94 SAY "."
NEXT
RETURN []
*-------- end of function FrontPage()

*: End of File: POPPHONE.PRG
