*:*********************************************************************
*:
*:        Program: POPSTOP.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-89, Greg Lief
*:  Last modified: 08/19/89 - cosmetic enhancements and alarm feature
*:                            special thanks to Dennis Levin
*:
*:  Procs & Fncts: DRAWSTOPBX()
*:
*:          Calls: CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:               : SHADOWBOX()   (function in $SHADOWB.PRG)
*:
*:     Pop-up stopwatch
*:
*:*********************************************************************

** To use background operation, variable START_TIME must be declared
** PUBLIC prior to calling this!

PARAM gfproc,line,var
PRIVATE oldscrn, oldcolor, elaptime, disptime, hotkey, moldrow, moldcol, ptr, ;
    mcurstop, mcursbot, already_on, maincolor, mwincol, mtop, mleft, gfsavedefs

** determine whether this was called via hot-key; if so, disable it
IF PCOUNT() > 0
   hotkey = LASTKEY()
   SET KEY hotkey TO
ELSE
   hotkey = 0
ENDIF

** determine screen position and color - check for global variables first
maincolor = IF(TYPE('gfstopcol') = 'U', 'W/B', gfstopcol)
mwincol   = IF(ISCOLOR(),'+W/N','I')
mtop      = IF(TYPE('gfstoptop') = 'U',   4, gfstoptop)
mleft     = IF(TYPE('gfstopleft') = 'U', 26, gfstopleft)

gfsavedefs = .F.
** User-configurable color/screen coordinates: GFSAVEDEFS is a flag that
** is set true (.T.) whenever the user changes either the color or the
** screen position of the calculator window.  Current hot-key to change
** color is Alt-F10, although you can easily change this (see line 236
** below).  Press any arrow key, Home, End, PgUp, PgDn to move window.

** save previous screen coordinates
moldrow = ROW()
moldcol = COL()
oldscrn = SAVESCREEN(mtop, mleft, mtop + 5, mleft + 29)
oldcolor = SETCOLOR()
mcurstop = Curs_Top()         && get the current cursor starting scan line
mcursbot = Curs_Bot()         && get the current cursor ending scan line
NewCurs(32,0)                 && functional equivalent of SET CURSOR OFF
DrawStopBx()
keypress = 0
** if the variable START_TIME is type numeric, it means the clock is running..
already_on = (TYPE('start_time') = 'N')
DO WHILE keypress != 27
   IF ! already_on
      @ mtop+2,  mleft+2  SAY 'Alt-B ='
      @ ROW()+1, mleft+2  SAY 'Bk Grnd'
      @ ROW(),   mleft+22 SAY 'start'
      @ ROW()+1, mleft+6  SAY 'Press Esc to exit'
      keypress = INKEY(0)
      IF keypress = -39         && Alt-F10: change color
         maincolor = ColorPal(maincolor, 16, IF(mleft > 36, 0, 64))
         IF LASTKEY() != 27
            SETCOLOR(maincolor)
            drawstopbx()
            gfsavedefs = .T.
         ENDIF
         LOOP
      ENDIF
      IF keypress < 32
         IF CheckMove(5, 29)         && see $MOVING.PRG for this function
            gfsavedefs = .T.
         ENDIF
         LOOP
      ENDIF
   ENDIF
   IF keypress = 304
      start_time = SECONDS()
      keypress = 27                      && to force exit from the main loop
   ELSEIF keypress != 27 .OR. already_on
      ** use different prompts if the clock is already running...
      @ mtop+2,  mleft+2  SAY IF(already_on, '  IN   ', SPACE(7))
      @ ROW()+1, mleft+2  SAY IF(already_on, 'BK GRND', SPACE(7))
      @ ROW(),   mleft+22 SAY 'stop '
      @ ROW()+1, mleft+6  SAY IF(already_on, 'Press Esc to exit', REPLICATE(CHR(220), 17))
      SETCOLOR(mwincol)
      start_time = IF(! already_on, SECONDS(), start_time)
      DO WHILE INKEY() = 0
         elaptime = SECONDS() - start_time
         ** sound alarm at minute and hour intervals
         IF elaptime = 60.00
            TONE(880,1)
            TONE(880,1)
            TONE(880,1)
         ENDIF
         IF elaptime = 3600.00
            TONE(440,2)
            TONE(440,2)
            start_time = 0
         ENDIF
         disptime = STR(INT(elaptime/60),2) + ':' + ;
            IF(elaptime%60<10,'0'+STR(elaptime%60,4,2),STR(elaptime%60,5,2))
         @ mtop+2, mleft+10 SAY disptime
      ENDDO
      SETCOLOR(maincolor)
      @ mtop+2, mleft+10 SAY disptime
      keypress = LASTKEY()
      IF .NOT. (keypress = 27 .AND. already_on)
         STORE .F. TO start_time, already_on
      ENDIF
   ENDIF
ENDDO

*** check for changes to screen coordinates and/or color
IF gfsavedefs
   IF yes_no('Save changes to position/color for this session')
      PUBLIC gfstopleft, gfstoptop, gfstopcol
      gfstopleft = mleft
      gfstoptop  = mtop
      gfstopcol  = maincolor
   ENDIF
ENDIF

** restore hot-key
IF hotkey != 0
   SET KEY hotkey TO POPSTOP
ENDIF

** restore previous environment
SETCOLOR(oldcolor)
RESTSCREEN(mtop, mleft, mtop + 5, mleft + 29, oldscrn)
@ moldrow,moldcol SAY ''
NewCurs(mcurstop, mcursbot)         && restore previous cursor
RETURN


FUNCTION drawstopbx    && draw the stopwatch box, of course
SETCOLOR(maincolor)
ShadowBox(mtop, mleft, mtop+4, mleft+27, 5)
** determine converse of main color and use it as the title
ptr = AT('/', maincolor)
SETCOLOR(SUBSTR(maincolor, ptr+1) + '/' + SUBSTR(maincolor, 1, ptr-1))
@ mtop,    mleft+ 9 SAY 'STOP WATCH'
SETCOLOR(maincolor)
@ ROW(),   mleft+ 1 SAY REPLICATE(CHR(219), 8)
@ ROW(),   mleft+19 SAY REPLICATE(CHR(219), 8)
@ ROW()+2, mleft+ 9 SAY '0:00.00'
@ ROW(),   mleft+19 SAY 'Any key'
@ ROW()+1, mleft+19 SAY 'to'
@ mtop+1, mleft+9, mtop+3, mleft+18 BOX 'Ŀ '
RETURN('')

*: EOF: POPSTOP.PRG
