*:*********************************************************************
*:
*:        Program: SETFILT.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1989, Greg Lief
*:  Last modified: 09/11/89
*:
*:  The monstrous interactive query builder
*:
*:  Procs & Fncts: QUERYBROW()
*:               : NEWQUERY
*:               : ADDFILT()
*:               : PICTEMP()
*:               : SHOWFILT()
*:
*:          Calls: HELP.PRG
*:               : CURS_TOP()
*:               : CURS_BOT()
*:               : NEWCURS()
*:
*:           Uses: TMPQUERY.DBF
*:               : QUERIES.DBF
*:
*:      Documented 03/03/89 at 13:40                SNAP!  version 3.12a
*:*********************************************************************
EXTERNAL SOUNDEX
PRIVATE mfilter, oldcolor, work_area, prequery, mcurstop, mcursbot, mrow, ;
        mcol, gen_err, run_it_now

*** save previous environment
mrow = ROW()
mcol = COL()
mcurstop = curs_top()         && get the current cursor starting scan line
mcursbot = curs_bot()         && get the current cursor ending scan line
oldcolor=SETCOLOR()
SAVE SCREEN TO prequery
work_area = UPPER(ALIAS())
SET SCOREBOARD OFF
newcurs(32,0)                 && functional equivalent of SET CURSOR OFF
CLEAR
IF ! FILE('queries.dbf') .OR. ! FILE('queries.dbt')
   Waiton('Initializing query file... please wait')
   SELE 0
   CREATE tmpquery
   APPEND BLANK
   REPLACE field_name WITH 'DESCRIP', field_type WITH 'C', field_len WITH 50
   APPEND BLANK
   REPLACE field_name WITH 'MFILTER', field_type WITH 'M', field_len WITH 10
   APPEND BLANK
   REPLACE field_name WITH 'MEFILTER', field_type WITH 'M', field_len WITH 10
   APPEND BLANK
   REPLACE field_name WITH 'QUERY_FILE', field_type WITH 'C', field_len WITH 8
   CREATE queries FROM tmpquery
   USE
   ERASE tmpquery.dbf
   Waitoff()
ENDIF
SELECT 0
USE queries

** give user choice of existing queries first
PRIVATE fields[1], heads[1]
fields[1] = 'DESCRIP'
heads[1] = 'QUERY DESCRIPTION'
gen_err = 'No query to '    && general error message used several times below
run_it_now = .F.            && flag set True if user runs query w/o saving
SETCOLOR('I')
@ 24,00 SAY CHR(24)+CHR(25)
@ 24,11 SAY 'F7'
@ 24,22 SAY 'F8'
@ 24,34 SAY 'F9'
@ 24,44 SAY 'F10'
@ 24,60 SAY CHR(17)+""
@ 24,72 SAY 'Esc'
SETCOLOR(oldcolor)
@ 24,03 SAY 'move'
@ 24,14 SAY 'count'
@ 24,25 SAY 'delete'
@ 24,37 SAY 'view'
@ 24,48 SAY 'new query'
@ 24,63 SAY 'select'
@ 24,76 SAY 'exit'
SETCOLOR('W/B,I')
shadowbox(04, 14, 20, 65, 3)
SET FILTER TO TRIM(queries->query_file) == work_area
GO TOP
DO WHILE .T.
   DBEDIT(5, 15, 19, 64, fields, "querybrow", '', heads, '', '')
   IF LASTKEY() != 27                && did user escape?
      IF EOF() .AND. ! run_it_now    && no query to select
         Err_msg(gen_err + 'select')
      ELSE
         IF ! run_it_now             && grab criteria from the database
            m->mfilter = mfilter
         ENDIF
         SELECT &work_area.
         Waiton('Searching records.. please wait')
         SET FILTER TO &mfilter.
         GO TOP
         IF EOF()
            Err_msg('No records meet those criteria.', 'W/G', '+W/G', .F.)
            SET FILTER TO
            SELE queries
            GO TOP
            Waitoff()
         ELSE
            EXIT
         ENDIF
      ENDIF
   ELSE
      EXIT
   ENDIF
ENDDO
RESTORE SCREEN FROM prequery
SETCOLOR(oldcolor)
SELECT queries
USE
IF LEN(work_area) > 0
   SELECT &work_area.
ENDIF
newcurs(mcurstop, mcursbot)
@ mrow,mcol SAY ''
RETURN


*!*********************************************************************
*!
*!       Function: QUERYBROW()
*!
*!          Calls: NEWQUERY       (procedure in SETFILT.PRG)
*!
*!*********************************************************************
FUNCTION querybrow      &&UDF to be used in conjunction with DBEDIT() above
PARAM browmode,indx
PRIVATE retkey, oldscrn, oldcolor, neterrmsg, mtotal
retkey = 1
IF browmode > 3
   DO CASE
      CASE LASTKEY() = 13             && Enter - select highlighted query
         fakehilite()
         retkey = 0
      CASE LASTKEY() = 27             && Esc - quit
         retkey = 0
      CASE LASTKEY() = -6            && F7 - count records for this query
         IF EOF()
            Err_msg(gen_err + 'count')
         ELSE
            fakehilite()
            m->mfilter = queries->mfilter
            SELECT &work_area.
            Waiton('Counting records.. please wait')
            GO TOP
            ** if the filter condition is empty, then the "filter" is
            ** a moot point -- show 'em total # of records
            IF EMPTY(mfilter)
               mvalue = LASTREC()
            ELSE
               COUNT FOR &mfilter. TO mvalue
            ENDIF
            oldcolor = SETCOLOR('+GR/N')
            @ 12, 25 SAY StrPad(STR(mvalue,4) + ' record' + ;
                         if(mvalue > 1, 's', '') + ' for this criteria', 30)
            INKEY(5)
            Waitoff()
            SETCOLOR(oldcolor)
            SELECT queries
         ENDIF
      CASE LASTKEY() = -7            && F8 - delete highlighted query
         IF EOF()
            Err_msg(gen_err + 'delete')
         ELSE
            fakehilite()
            IF Yes_no('Are you sure you want to delete this query')
               neterr_msg = 'Could not delete query at this time'
               IF RLOCK()
                  DELETE
                  USE queries EXCLUSIVE
                  IF ! NETERR()
                     Waiton('Deleting query.... please wait')
                     PACK
                     COPY STRU TO tempqry
                     USE tempqry EXCLUSIVE
                     APPE FROM queries
                     ERASE queries.dbf
                     ERASE queries.dbt
                     USE
                     RENAME tempqry.dbf TO queries.dbf
                     RENAME tempqry.dbt TO queries.dbt
                     USE queries
                     SET FILTER TO TRIM(queries->query_file) == work_area
                     GO TOP
                     Waitoff()
                  ELSE
                     Err_msg(neterr_msg)
                  ENDIF
               ELSE
                  Err_msg(neterr_msg)
               ENDIF
               UNLOCK
            ENDIF
         ENDIF
         retkey = 2
      CASE LASTKEY() = -8            && F9 - view criteria
         IF EOF()               && no query to select
            Err_msg(gen_err + 'view')
         ELSE
            fakehilite()
            oldcolor = SETCOLOR('W/R')
            oldscrn = shadowbox(07, 25, 17, 54, 4, 'Query Criteria')
            @17,30 SAY ' Press Esc to exit '
            MEMOEDIT(mefilter, 8, 26, 16, 53, .F.)
            SETCOLOR(oldcolor)
            ByeByeBox(oldscrn)
         ENDIF
         retkey = 2
      CASE LASTKEY() = -9             && F10 - create new query
         SAVE SCREEN TO oldscrn
         SELECT &work_area.
         DO newquery
         RESTORE SCREEN FROM oldscrn
         CLEAR TYPEAHEAD
         GO TOP
         IF run_it_now
            retkey = 0
            KEYBOARD CHR(27)
         ELSE
            retkey = 2
         ENDIF
   ENDCASE
ENDIF
RETURN(retkey)

*!*********************************************************************
*!
*!       Function: FAKEHILITE()
*!
*!      Called by: QUERYBROW()
*!
*!*********************************************************************
FUNCTION fakehilite
PRIVATE oldcolor
oldcolor = SETCOLOR('I')
@ ROW(), COL() SAY descrip
SETCOLOR(oldcolor)
RETURN('')

*!*********************************************************************
*!
*!      Procedure: NEWQUERY
*!
*!      Called by: QUERYBROW()    (function  in SETFILT.PRG)
*!
*!          Calls: SHOWFILT()     (function  in SETFILT.PRG)
*!               : ADDFILT()      (function  in SETFILT.PRG)
*!
*!*********************************************************************
PROCEDURE newquery
PRIVATE op_string, mplain, menhanced, flds[fcount()], bott, ;
        mefilter, operators[9], op_choices[9], booleans[8], bl_choices[8], ;
        openparen, mdescrip, oldcolor, a_num, mbool
SELECT &work_area.
op_string = '= < > <=>=<>'          && used below when selecting operator
operators[1] = 'Equal to'
operators[2] = 'Less than'
operators[3] = 'Greater than'
operators[4] = 'Less than or equal to'
operators[5] = 'Greater than or equal to'
operators[6] = 'Not equal to'
operators[7] = 'Contains'
operators[8] = 'Does not contain'
operators[9] = 'Sounds like'
booleans[1] = 'Discard'
booleans[2] = ' .AND. '
booleans[3] = ' .OR. '
booleans[4] = ' .AND. ('
booleans[5] = ' .OR. ('
booleans[6] = ') .AND. '
booleans[7] = ') .OR. '
booleans[8] = '<<done>>'
openparen = 0         && keep track of # of open parentheses in the condition
AFILL(op_choices, .T.)
AFILL(bl_choices, .T.)
mfilter = ''
mefilter = 'ALL RECORDS'
AFIELDS(flds)
SAVE SCREEN TO prefilter
* determine colors based upon type of monitor in use
mplain = IF(ISCOLOR(), 'W/B,I', 'W/N, I')
menhanced = '+' + mplain
oldcolor = SETCOLOR()
SETCOLOR('W/N')
CLEAR
SETCOLOR(menhanced)
@0,33 SAY " QUERY BUILDER "
ExBox(3, 0, 18, 38, 2, 0, '', .F., 'Selection Criteria')
@ 04,01 SAY mefilter
BOTT = IF(FCOUNT() > 19, 23, 4 + FCOUNT())
ASORT(flds)
Exbox(20, 00, 24, 25, 2, 0)
@21,05 SAY 'Move Highlight Bar'
@22,06 SAY 'To Select An Item'
@23,06 SAY 'To End Selection'
Exbox(1, 54, bott, 67, 4, 0, '', .F., 'Fields')
SETCOLOR('I')
@21,02 SAY CHR(24)+CHR(25)
@22,02 SAY CHR(17)+""
@23,02 SAY 'Esc'
firstloop = .T.
DO WHILE .T.
   showfilt()
   SETCOLOR(mplain)
   a_num = ACHOICE(2, 56, BOTT-1, 65, flds)
   IF a_num = 0 .AND. firstloop
      EXIT
   ELSEIF a_num > 0
      IF TYPE(flds[a_num]) != 'M'
         IF ! addfilt()
            EXIT
         ENDIF
         IF firstloop .AND. mbool > 1
            firstloop = .F.
         ENDIF
      ENDIF
   ENDIF
ENDDO
showfilt()
SELE queries
mdescrip = SPACE(50)
SETCOLOR(IF(ISCOLOR(), '+W/BR', '+W/N'))
shadowbox(09, 07, 15, 72, 2)
@ 10,09 SAY 'You may now enter a description of up to 50 characters for'
@ 11,09 SAY 'this query.  If you wish to run this query immediately without'
@ 12,09 SAY 'saving it, leave the description empty and press Enter.  If '
@ 13,09 SAY 'you want to exit without saving this query, press Esc.'
@ 14,15 GET mdescrip
Newcurs(6,7)                    && functional equivalent of set cursor on
READ
Newcurs(32,0)                   && functional equivalent of set cursor off
IF LASTKEY() != 27              && if user did not press Esc
   IF LEN(TRIM(mdescrip)) = 0   && left it blank -- do not save it
      run_it_now = .T.          && to run query immediately - see QUERYBROW()
   ELSE
      ** force this record to be added with the following 'endless loop'
      DO WHILE .T.
         APPEND BLANK
         IF ! Neterr()
            REPLACE descrip WITH mdescrip, query_file WITH work_area
            REPLACE mfilter WITH m->mfilter, mefilter WITH m->mefilter
            EXIT
         ENDIF
      ENDDO
   ENDIF
ENDIF
SETCOLOR(oldcolor)
RETURN

*!*********************************************************************
*!
*!       Function: ADDFILT()
*!
*!      Called by: NEWQUERY       (procedure in SETFILT.PRG)
*!
*!          Calls: PICTEMP()      (function  in SETFILT.PRG)
*!               : NEWCURS()      (function  in ?)
*!
*!*********************************************************************
FUNCTION addfilt
PRIVATE buffer, buff1, mvalue, mfield, mefield, wid, boxbott, op, mop, meop
buffer = SAVESCREEN(02,01,20,74)
fld_name=flds[a_num]
DO CASE
   CASE TYPE(fld_name) = 'D'
      pic_len = 8
      mvalue = CTOD('')
   CASE TYPE(fld_name) = 'C'
      mvalue =  SPACE(LEN(&fld_name.))
      pic_len = MIN(LEN(mvalue), 35)
   CASE TYPE(fld_name) = 'L'
      pic_len = 1
      mvalue = .F.
   OTHERWISE
      pic_len = LEN(pictemp(fld_name))
      mvalue = 0
ENDCASE
mop = ' = '
meop = ' Equal to '
IF TYPE(fld_name) != 'L'   && get operator for non-logical fields
   SETCOLOR(IF(ISCOLOR(), '+W/BG', 'W/N'))
   boxbott = IF(TYPE(fld_name) = 'C', 13, 10)
   buff1 = shadowbox(2, 47, boxbott - 1, 72, 3, TRIM(fld_name))
   op = 0
   ** only permit access to soundex() and substr() choices if this
   ** is a character variable -- logical enough, eh?
   STORE (TYPE(fld_name)='C') TO op_choices[7], op_choices[8], op_choices[9]
   DO WHILE op = 0
      op = ACHOICE(3, 48, boxbott - 2, 71, operators, op_choices)
   ENDDO
   ByeByeBox(buff1)
   meop = ' ' + TRIM(operators[op]) + ' '
   IF op < 7
      mop=' ' + TRIM(SUBSTR(op_string, (op-1)*2 + 1, 2)) + ' '
   ENDIF
ENDIF
wid = MAX(LEN(fld_name) + LEN(meop) + pic_len, 30)
SETCOLOR(IF(ISCOLOR(), 'W/RB', 'W/N'))
shadowbox(05, 68-wid, 08, 72, 4)
SETCOLOR(IF(ISCOLOR(), '+W/RB', '+W/N'))
SET KEY 303 TO view_vals                      && Alt-V
@ 07, 42 SAY '(Alt-V for available values)'
@ 06, 70-wid SAY fld_name + meop
@ 06, COL() GET mvalue PICTURE pictemp(fld_name)
newcurs(6,7)                  && functional equivalent of SET CURSOR ON
READ
newcurs(32,0)                 && functional equivalent of SET CURSOR OFF
CLEAR GETS
SET KEY 303 TO
DO CASE
   CASE TYPE(fld_name)='N'
      mfield = fld_name + mop + LTRIM(STR(mvalue))
      mefield = fld_name + meop + LTRIM(STR(mvalue))
   CASE TYPE(fld_name)='L'
      mfield = IF(mvalue, fld_name, '! ' + fld_name)
      mefield = IF(mvalue, fld_name, 'NOT ' + fld_name)
   CASE TYPE(fld_name)='D'
      mfield = fld_name + mop + "CTOD('" + DTOC(mvalue) + "')"
      mefield = fld_name + meop + DTOC(mvalue)
   OTHERWISE
      ** Criteria 7 and 8 ('contains' and 'sounds like') are special cases
      ** and must be processed a bit differently than the first six
      DO CASE
         CASE op = 7        && 'contains'
            mfield = '[' + TRIM(mvalue) + '] $ ' + fld_name
         CASE op = 8        && 'does not contain'
            mfield = '! [' + TRIM(mvalue) + '] $ ' + fld_name
         CASE op = 9        && 'sounds like'
            mfield = "SOUNDEX(&fld_name.) = SOUNDEX([" + TRIM(mvalue) + "])"
         OTHERWISE
            mfield = fld_name + mop + "[" + TRIM(mvalue) + "]"
      ENDCASE
      mefield = fld_name + meop + "'" + TRIM(mvalue) + "'"
ENDCASE
SETCOLOR(IF(ISCOLOR(), 'W/R', 'W/N'))
shadowbox(10, 56, 19, 65, 3)
SETCOLOR(IF(ISCOLOR(), '+W/R', '+W/N'))
mbool = 0
STORE (openparen > 0) TO bl_choices[6], bl_choices[7]
** force them to make a selection
DO WHILE mbool = 0
   mbool = ACHOICE(11, 57, 18, 64, booleans, bl_choices)
ENDDO
IF mbool = 8
   *** add the required number of closed parenthesis to balance it out
   mfilter = mfilter + mfield + REPLICATE(')', openparen)
   mefilter = STRTRAN(mefilter,'ALL RECORDS','') + mefield + ;
              REPLICATE(')',openparen)
ELSEIF mbool > 1     && selection 1 means they want to discard this condition
   openparen = openparen + IF(mbool=4 .OR. mbool=5, 1, ;
               IF(mbool > 5, -1, 0))
   mfilter = mfilter + mfield + booleans[mbool]
   mefilter = STRTRAN(mefilter,'ALL RECORDS','') + mefield + booleans[mbool]
   SETCOLOR(mplain)
   @23,02 SAY SPACE(23)
ENDIF
RESTSCREEN(02, 01, 20, 74, buffer)
RETURN (mbool < 8)

*!*********************************************************************
*!
*!       Function: PICTEMP()
*!
*!      Called by: ADDFILT()      (function  in SETFILT.PRG)
*!
*!*********************************************************************
FUNCTION pictemp   && returns matching picture string for specified field
PARAM field
PRIVATE pstring, strng
DO CASE
   CASE TYPE(FIELD) = "C"
      pstring = '@S' + LTRIM(STR(MIN(LEN(&field),35)))
   CASE TYPE(FIELD) = "N"
      strng = STR(&field)
      IF "." $ strng
         pstring = REPLICATE('9',AT(".",strng)-1)+"."
         pstring = pstring + REPLICATE('9', LEN(strng) - LEN(pstring))
      ELSE
         pstring = REPLICATE('9', LEN(strng))
      ENDIF
   CASE TYPE(FIELD) = "L"
      pstring = 'Y'
   OTHERWISE
      pstring=''
ENDCASE
RETURN pstring

*!*********************************************************************
*!
*!       Function: SHOWFILT()
*!
*!      Called by: NEWQUERY       (procedure in SETFILT.PRG)
*!
*!*********************************************************************
FUNCTION showfilt
SETCOLOR(menhanced)
MEMOEDIT(mefilter,04,01,17,37,.F.,.F.)
RETURN('')


*!*********************************************************************
*!
*!      Procedure: VIEW_VALS
*!      Called by: NEWFILT()       (function in SETFILT.PRG)
*!      Hot key (Alt-V) to pop up field values for quick reference
*!
*!*********************************************************************
PROCEDURE view_vals
PRIVATE fields[1], heads[1], buffer
SET KEY 303 TO
buffer = shadowbox(10, 68 - pic_len, 20, 71, 2)
STORE fld_name TO fields[1], heads[1]
GO TOP
DBEDIT(11, 70 - pic_len, 19, 70, fields, '', '', heads, '', '')
IF LASTKEY() != 27
   mvalue = &fld_name
ENDIF
ByeByeBox(buffer)
newcurs(6,7)                  && functional equivalent of SET CURSOR ON
SET KEY 303 TO view_vals
RETURN

*: EOF: SETFILT.PRG
