*:*********************************************************************
*:
*:        Program: _APICK.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1989, Greg Lief
*:      Special thanks to Rick Hubbard!
*:
*:  Procs & Fncts: APICK()
*:               : APICK_FUNC()
*:
*:      Documented 04/19/89 at 21:15                SNAP!  version 3.12a
*:*********************************************************************

*!*********************************************************************
*!
*!       Function: APICK()
*!
*!*********************************************************************
FUNCTION apick
PARAMETERS mtop, mleft, mbottom, mright, marray, box_clr, bar_clr, stat_clr, ;
           actual_len, tagging, unsel_clr, hilite_clr, minit, boxtitle
PRIVATE bar_line, xx, last_ele, oldcolor, oldscrn, choice, draw_bar, ;
        searchstr, midpoint

*** establish colors for box, status bar, and indicator
*
* default colors: box        = bright white on blue or bright white on black
*                 status bar = white on black
*                 indicator  = yellow
*                 unselected = white on blue or white on black
*                 highlight  = inverse
*
box_clr   = IF(PCOUNT() < 6, '+' + IF(ISCOLOR(), 'W/B', 'W/N'), ;
            IF(LEN(box_clr) > 0, box_clr, '+' + IF(ISCOLOR(), 'W/B', 'W/N')))
bar_clr   = IF(PCOUNT() < 7, 'W/N, I', ;
            IF(LEN(bar_clr) > 0, bar_clr, 'W/N, I'))
stat_clr  = IF(PCOUNT() < 8, IF(ISCOLOR(), '+GR/N', 'I'), ;
            IF(LEN(stat_clr) > 0, stat_clr, IF(ISCOLOR(), '+GR/N', 'I')))
unsel_clr = IF(PCOUNT() < 11, IF(ISCOLOR(), 'W/B', 'W/N'), ;
            IF(LEN(unsel_clr) > 0, unsel_clr, IF(ISCOLOR(), 'W/B', 'W/N')))
hilite_clr= IF(PCOUNT() < 12, 'I', IF(LEN(hilite_clr)>0, hilite_clr, 'I'))
searchstr = []
midpoint = mleft + INT((mright - mleft) / 2) - 4

** determine # of last element by grabbing length of array,
** unless of course the actual length was passed as parameter #9
last_ele = IF(TYPE('actual_len') != "N", LEN(marray), actual_len)

** if all of the array elements fit in the window, no need for the status bar
draw_bar = (last_ele > mbottom - mtop - 1)

** establish startup row for status bar
bar_line = mtop + 2    && this will force status bar to be drawn on 1st pass

** are we allowing the user to tag elements?  If so, add a space to
** each array element to hold the ubiquitous checkmark 
tagging = IF(PCOUNT() < 10, .F., tagging)
IF tagging
   FOR xx = 1 TO last_ele
      marray[xx] = marray[xx] + [ ]
   NEXT
ENDIF

** determine initial element if not passed as parameter 13
minit = IF(PCOUNT() < 13, 0, IF(TYPE('minit') != "N", 0, minit))
rel_row = minit  && relative row for ACHOICE() window

** build parallel array for available choices by looping through main array
** unavailable selections will begin with a tilde ("~")
** you can also have horizontal lines for aesthetic purposes by having
** array elements of tilde and the character to use for the line.  For
** example, "~" will be expanded into a line such as "" and
** will encompass the width of the achoice() box.
PRIVATE choices[last_ele]
AFILL(choices, .T.)
FOR xx = 1 TO last_ele
   IF SUBSTR(marray[xx], 1, 1) = '~'
      choices[xx] = .F.
      marray[xx] = SUBSTR(marray[xx], 2)
      *** see if they want to draw a horizontal line - if so, the trimmed
      *** length of this array element will now be one.
      IF LEN(TRIM(marray[xx])) = 1
         marray[xx] = REPLICATE(TRIM(marray[xx]), mright - mleft - 1)
      ENDIF
   ENDIF
NEXT

** draw box and status bar
oldcolor = SETCOLOR(box_clr)
oldscrn = ShadowBox(mtop, mleft, mbottom, mright, 1, ;
          if(pcount() < 14, '', boxtitle))  && use title if passed as parameter
IF draw_bar
   SETCOLOR(bar_clr)
   @ mtop + 1, mright, mbottom - 1, mright box chr(176)
ENDIF
SETCOLOR(box_clr + ',' + hilite_clr + ',,,' + unsel_clr)

** stuff weirdo character into buffer to force drawing status indicator
** on 1st pass without actually moving the highlight bar - sneaky sneaky
KEYBOARD CHR(255)
DO WHILE .T.
   choice = ACHOICE(mtop + 1, mleft + 1, mbottom - 1, mright - 1, ;
            marray, choices, 'apick_func', minit, rel_row)
   IF LASTKEY() = 13 .OR. LASTKEY() = 27
      EXIT
   ENDIF
ENDDO
ByeByeBox(oldscrn)
SETCOLOR(oldcolor)
RETURN choice

* end function APick()
*--------------------------------------------------------------------*


*!*********************************************************************
*!
*!       Function: APICK_FUNC()
*!
*!*********************************************************************
FUNCTION apick_func
PARAMETERS mstatus, curr_elem, curr_row
PRIVATE ret_val, oldrow, oldcol, key, oldcolor

** save screen coordinates
oldrow = ROW()
oldcol = COL()

key = LASTKEY()

ret_val = 2    && default return value -- continue selection process

DO CASE
   CASE mstatus = 0 .OR. key = 255    && idle or start-up

      oldcolor = SETCOLOR()
      IF draw_bar
         ** draw arrows if there are elements beyond top or bottom of window
         ** first the bottom
         @ mbottom, mright SAY IF(last_ele - curr_elem >= mbottom - oldrow, ;
                                  CHR(25), CHR(188))
         ** then the top
         @ mtop,mright SAY IF(oldrow - curr_elem < mtop, CHR(24), CHR(187))

         ** if status bar position has changed...
         IF bar_line != mtop + 1 + ;
                        INT((curr_elem / last_ele) * (mbottom - mtop - 2))
            *** first, blank out previous status bar
            SETCOLOR(bar_clr)
            @ bar_line, mright SAY CHR(176)
            *** then recalculate position of status bar
            bar_line = mtop + 1 + INT((curr_elem/last_ele)*(mbottom-mtop-2))
            SETCOLOR(stat_clr)
            *** finally, redraw it
            @ bar_line, mright SAY CHR(219)
         ENDIF
      ENDIF
      SETCOLOR(oldcolor)

   CASE mstatus = 1
      minit = last_ele
      KEYBOARD CHR(255)       && force status bar display
      ret_val = 0             && force ACHOICE() to restart

   CASE mstatus = 2
      minit = 1
      KEYBOARD CHR(255)       && force status bar display
      ret_val = 0             && force ACHOICE() to restart

   CASE mstatus = 3           && key exception

      DO CASE

         CASE key = 27        && Esc to abort
            ret_val = 0

         CASE key = 13        && Enter - made selection
            ret_val = 1

         CASE key = 1         && Home - jump to top
            KEYBOARD CHR(31)  && equivalent to Ctrl-PgUp
            ret_val = 2

         CASE key = 6         && End - jump to bottom
            KEYBOARD CHR(30)  && equivalent to Ctrl-PgDn
            ret_val = 2

         *** Spacebar - tag this item if tagging is available
         CASE key = 32 .AND. tagging
            mArray[curr_elem] = LEFT(mArray[curr_elem], ;
               LEN(mArray[curr_elem]) - 1) + ;
               IF(RIGHT(mArray[curr_elem], 1) = " ", "", " ")
            minit = curr_elem
            rel_row = curr_row
            searchstr = []              && reset search string
            @ mbottom, midpoint SAY REPLICATE(CHR(205), 8)
            ret_val = 0                 && Force ACHOICE redisplay

         *** F8: tag all items
         CASE key = -7 .and. tagging
            FOR xx = 1 TO last_ele
               *** only process available choices
               IF choices[xx]
                  mArray[xx] = LEFT(mArray[xx], LEN(mArray[xx]) - 1) + CHR(251)
               ENDIF
            NEXT
            minit = curr_elem      && save current position
            rel_row = curr_row     && and relative position
            ret_val = 0            && Force ACHOICE redisplay

         *** F9: clear all tags
         CASE key = -8 .and. tagging
            FOR xx = 1 TO last_ele
               *** only process available choices
               IF choices[xx]
                  mArray[xx] = LEFT(mArray[xx], LEN(mArray[xx]) - 1) + [ ]
               ENDIF
            NEXT
            minit = curr_elem       && save current position
            rel_row = curr_row      && and relative position
            ret_val = 0             && Force ACHOICE redisplay

         *** F10: reverse all tags
         CASE key = -9 .and. tagging
            FOR xx = 1 TO last_ele
               *** only process available choices
               IF choices[xx]
                  mArray[xx] = LEFT(mArray[xx], LEN(mArray[xx]) - 1) + ;
                               IF(RIGHT(mArray[xx], 1) = " ", "", " ")
               ENDIF
            NEXT
            minit = curr_elem    && save current position
            rel_row = curr_row   && and relative position
            ret_val = 0          && Force ACHOICE redisplay

         *** letter key
         CASE ISALPHA(CHR(key))
            searchstr = searchstr + CHR(key)
            telem = ASCAN(marray, searchstr)
            minit = IF(telem = 0, curr_elem, telem)
            @ mbottom, midpoint SAY "[" + StrPad(searchstr, 6) + "]"
            ret_val = 0      && Force ACHOICE redisplay

         CASE key = 8 .OR. key = 19
            IF LEN(searchstr) > 0
               searchstr = SUBSTR(searchstr, 1, LEN(searchstr) - 1)
               telem = ASCAN(marray, searchstr)
               minit = IF(telem = 0, curr_elem, telem)
            ENDIF
            @ mbottom, midpoint SAY IF(LEN(searchstr) = 0, ;
                                    REPLICATE(CHR(205), 8), ;
                                    "[" + StrPad(searchstr, 6) + "]")
            ret_val = 0      && Force ACHOICE redisplay

      ENDCASE

ENDCASE
** restore proper screen coordinates
@ oldrow, oldcol SAY ""
RETURN ret_val

* end function APick_Func()
*--------------------------------------------------------------------*

* eof $apick.prg

