*:*********************************************************************
*:
*:       Functions to save and load arrays from/to text files
*:       GSAVEARRAY() and GLOADARRAY()
*:         Author: Greg Lief
*:      Copyright (c) 1990 Grumpfish, Inc. - All Rights Reserved
*:      Clipper Summer '87 version
*:
*:*********************************************************************

*:*********************************************************************
*:
*:       Function: GLoadArray()
*:         Author: Greg Lief
*:      Copyright (c) 1990 Grumpfish, Inc. - All Rights Reserved
*:
*:      Purpose: load an array from a previously saved text file
*:
*:      Syntax: GLoadArray(<array>, <filename>)
*:
*:      Parameters:  <array> is the name of the array to be loaded.
*:                   This must be enclosed in quotes.  However, it
*:                   need not be declared prior to calling this
*:                   function.
*:
*:                   <filename> is the name of the file from which to
*:                   load the array.
*:
*:      Return Value:  A logical True (.T.) if the load was successful;
*:                     False (.F.) if it was not.  Reasons for it
*:                     failing would be related to an inability to
*:                     create the file (probably due to lack of available
*:                     file handles) or inability to read from the file
*:                     (in which case your guess is as good as mine).
*:
*:      Sample usage:  gloadarray('myarray', 'array.txt')
*:
*:          Calls: GFReadLine()   (function contained in $READLIN.PRG)
*:
*:*********************************************************************
Function gloadarray
parameter aArray, cFile
private nLen, nX, nHandle, lRetval, cBuff
cBuff = []
nHandle = fopen(cFile)
lRetval = .F.
if nHandle != -1
   nLen = space(2)
   if fread(nHandle, @nLen, 2) = 2
      nLen = BIN2I(nLen)
      public &aArray.[nLen]
      for nX = 1 to nLen
         gfreadline(@cBuff, nHandle)
         cType = SUBSTR(cBuff, 1, 1)
         cBuff = SUBSTR(cBuff, 2)
         DO CASE
            CASE cType = "C"
               &aArray[nX] = cBuff
            CASE cType = "D"
               &aArray[nX] = CTOD(cBuff)
            CASE cType = "L"
               &aArray[nX] = (cBuff == 'T')
            CASE cType = "N"
               &aArray[nX] = VAL(cBuff)
         ENDCASE
      next
      lRetval = .T.
   endif
endif
fclose(nHandle)
return lRetval

* end function GLoadArray()
*--------------------------------------------------------------------*


*:*********************************************************************
*:
*:       Function: GSaveArray()
*:         Author: Greg Lief
*:      Copyright (c) 1990 Grumpfish, Inc. - All Rights Reserved
*:
*:      Purpose: You can't save an array to a .MEM file, but this is
*:               the next best thing.  GSaveArray() saves a specified
*:               array to a text file.
*:
*:      Syntax: GSaveArray(<array>, <filename>)
*:
*:      Parameters:  <array> is the name of the array to be saved.
*:                   Do not enclose this in quotes!
*:
*:                   <filename> is the name of the file in which to
*:                   save the array.  Note that if this file exists,
*:                   it will be overwritten!
*:
*:      Return Value:  A logical True (.T.) if the save was successful;
*:                     False (.F.) if it was not.  Reasons for it
*:                     failing would be related to an inability to
*:                     create the file (probably due to lack of available
*:                     file handles) or inability to write to the file
*:                     (in which case your guess is as good as mine).
*:
*:      Sample Usage:  gsavearray(marray, 'array.txt')
*:
*:*********************************************************************
FUNCTION gsavearray
parameter aArray, cFile
private nLen, nX, nHandle, lRetval
nHandle = fcreate(cFile)
lRetval = .F.
if nHandle != -1
   nLen = len(aArray)
   fwrite(nHandle, I2BIN(nLen))
   for nX = 1 to nLen
      cBuff = TYPE('aArray[nX]')
      DO CASE
         CASE cBuff = "C"
            cBuff = cBuff + aArray[nX]
         CASE cBuff = "D"
            cBuff = cBuff + DTOC(aArray[nX])
         CASE cBuff = "L"
            cBuff = cBuff + IF(aArray[nX], 'T', 'F')
         CASE cBuff = "N"
            cBuff = cBuff + STR(aArray[nX])
      ENDCASE
      cBuff = cBuff + CHR(13) + CHR(10)
      if fwrite(nHandle, cBuff, LEN(cBuff)) != LEN(cBuff)
         ret_val = .F.
         EXIT
      endif
   next
   lRetval = .T.
endif
fclose(nHandle)
return lRetval

* end function GSaveArray()
*--------------------------------------------------------------------*

* eof $arrays.prg
