*:*********************************************************************
*:
*:        Program: CALC_PIC()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-89, Greg Lief
*:
*:     Enables calculator-style numeric data entry
*:
*:          Calls: CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:
*:*********************************************************************
FUNCTION calc_pic
PARAMETERS mvar, pitcher   && no, I'm not a bad speller-I like baseball!
PRIVATE decplace, tnum, maxdecimal, mrow, mcol, oldcolor, commas, ;
        mcurstop, mcursbot, xx, key

*** save previous environment
oldcolor = SETCOLOR()
mrow = ROW()
mcol = COL()
mcurstop = Curs_Top()         && get the current cursor starting scan line
mcursbot = Curs_Bot()         && get the current cursor ending scan line
NewCurs(32,0)                 && functional equivalent of SET CURSOR OFF
SETCOLOR(SUBSTR(oldcolor, AT(',', oldcolor)+1))   && sneaky sneaky

*** determine maximum # of decimals based on location of decimal point
maxdecimal = IF(AT('.', pitcher) > 0, LEN(pitcher) - AT('.', pitcher), 0)
STORE 0 TO decplace, key

***
* determine if there are commas in the picture clause
* if so, we must account for them
***
commas = 0
IF ',' $ pitcher
   *** loop through PICTURE clause and tally up # of commas
   FOR xx = 1 TO LEN(pitcher)
      commas = commas + IF(SUBSTR(pitcher, xx, 1) = ',', 1, 0)
   NEXT
ENDIF

***
* NOTE: you could also accomplish the comma count with the next
* line of code, courtesy of Mary Matthews.  If there are commas
* in the PICTURE clause, this method is certainly quicker than
* the FOR..NEXT loop.  However, it will make your .EXE file app.
* 900 bytes larger because you will now be linking in the Clipper
* STRTRAN() function.  If you do not want to link in STRTRAN(),
* or you do not anticipate including commas in the picture clause,
* I recommend that you use the FOR..NEXT loop.
***
*commas = LEN(pic) - LEN(STRTRAN(pic,",") + "")

***
* begin main loop
*
* we simulate an actual READ by allowing the following active keys:
* Enter (13),  Esc (27),  Ctrl-W (23),  UpArrow (5),
* DownArrow (2), PgUp (18), and PgDn (3)
*
* If you want to limit escape from this routine, it is a simple
* matter to remove the desired key values from the next statement
***

DO WHILE key != 13 .AND. key != 27 .AND. key != 23 .AND. key != 5 .AND. ;
         key != 2  .AND. key != 18  .AND. key != 3
   @ mrow, mcol SAY mvar PICTURE pitcher
   key = INKEY(0)
   DO CASE

      *** user pressed a numeric key
      CASE key > 47 .AND. key < 58
         tnum = VAL(CHR(key))             && determine numeric value
         DO CASE
            CASE mvar = 0 .AND. decplace = 0
               mvar = tnum
            CASE mvar != 0 .AND. decplace=0 .AND. ;
                          mvar < 10 ^ (LEN(pitcher) - maxdecimal - 2 - commas)
               mvar = mvar * 10 + tnum
            CASE decplace > 0 .AND. decplace <= maxdecimal     && real number
               mvar = mvar + (tnum / (10 ^ decplace))
               decplace = decplace + 1
         ENDCASE

      *** user pressed hyphen to toggle sign of number
      CASE key = 45
         mvar = mvar * -1

      *** user pressed backspace
      CASE key = 8 .AND. mvar != 0
         DO CASE
            *** if number is still an integer, kill least significant digit
            CASE decplace = 0
               mvar = INT(mvar / 10)
            *** if we are at one decimal place, change number to integer
            CASE decplace = 2
               decplace = 0
               mvar = INT(mvar)
            OTHERWISE
               decplace = decplace - 1
               mvar = LTRIM(STR(mvar, 16, decplace))
               mvar = VAL(SUBSTR(mvar, 1, LEN(mvar) - 1))
         ENDCASE

      *** user pressed period to set the decimal point
      CASE key = 46 .and. decplace = 0 .and. maxdecimal > 0
         decplace = 1

   ENDCASE
ENDDO

*** must stuff last keypress into buffer so it will be acted upon properly
KEYBOARD CHR(key)

*** restore environment to previous state
@ mrow, mcol SAY ''
NewCurs(mcurstop, mcursbot)         && restore previous cursor
SETCOLOR(oldcolor)
RETURN pitcher

* end function Calc_Pic()
*--------------------------------------------------------------------*

* eof $calcpic.prg
