*:*********************************************************************
*:
*:        Program: $CALENDA.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-89, Greg Lief
*:
*:  Procs & Fncts: CALENDBOX()
*:               : NEXTMONTH()
*:
*:  Modifications: 06/03/90 - added logic to determine CROW and CCOL -
*:                            coordinates which are used when jumping
*:                            back to system date
*:
*:  NOT FOR USE AS STAND-ALONE!!  CALLED BY GETDATE() and POPDATE
*:
*:*********************************************************************

*!*********************************************************************
*!
*!      Function: CALENDBOX
*!
*!      Called by: POPDATE      (procedure in POPDATE.PRG)
*!                 GETDATE()    (function  in $GETDATE.PRG)
*!
*!*********************************************************************
FUNCTION calendbox
PARAMETER usingappts    && .T. if using appt tracker, .F. if using GETDATE()
PRIVATE hasappts
IF firstloop
   SETCOLOR(maincolor)
   SHADOWBOX(mtop, mleft, mtop+12, mleft+21, 1)
   @ mtop +  3, mleft +  1 SAY "Su Mo Tu We Th Fr Sa"
   @ mtop +  1, mleft + 21 SAY ""
   @ mtop +  2, mleft      SAY "" + REPLICATE("",20) + ""
   @ mtop +  4, mleft      SAY "" + REPLICATE("",20) + ""
   @ mtop + 12, mleft      SAY "" + REPLICATE("",20) + ""
   firstloop = .F.
   calendscrn = SAVESCREEN(mtop, mleft, mtop+13, mleft+23)
ELSE
   RESTSCREEN(mtop, mleft, mtop+13, mleft+23, calendscrn)
ENDIF
SETCOLOR('+' + maincolor)
datehead = CMONTH(mdate) + ' ' + STR(YEAR(mdate),4)
IF LEN(datehead) < 14
   datehead = SPACE((14-LEN(datehead))/2) + datehead + ;
              SPACE((14-LEN(datehead))/2 + 1)
ENDIF
@ mtop+1, mleft+4 SAY datehead
tdate = mdate
tdate = STOD(SUBSTR(DTOS(tdate), 1, 6) + '01')  && first day of month
tmonth = MONTH(tdate)
ROW = 5
gotlastday = .F.   && flag for storing coordinates of last day of mon.
DO WHILE .T.
   hasappts = .F.  && flag for whether a date should be shown in inverse
   * test for new week
   IF (DAY(tdate)>1 .OR. MONTH(tdate) <> tmonth) .AND. DOW(tdate)=1
      ROW = ROW + 1
   ENDIF
   IF ROW > 11
      EXIT
   ENDIF
   *** no need to seek for appointments for current date unless we are
   *** using the appointment tracker
   IF usingappts
      SEEK DTOS(tdate)
      IF FOUND()
         hasappts = .T.
         *** set FOUNDONE flag true if this date is the current date
         *** so that we know to display its appts in DRAW_APPTS
         foundone = IF(tdate = mdate, .T., foundone)
      ENDIF
   ENDIF
   DO CASE
      CASE usingappts .AND. hasappts
         SETCOLOR(IF(tdate = mdate, '*N/W', 'I'))
      CASE MONTH(tdate) = MONTH(mdate)
         SETCOLOR(IF(tdate = mdate, '*', '') + '+' + maincolor)
      OTHERWISE
         SETCOLOR(maincolor)
   ENDCASE
   @ mtop+row, mleft + (DOW(tdate)-1) * 3 + 1 SAY STR(DAY(tdate),2)
   DO CASE
      * first day of month
   CASE DAY(tdate) = 1 .AND. MONTH(tdate) = tmonth
      frow = ROW()
      fcol = COL()-2
      * last day of month
   CASE MONTH(tdate + 1) != tmonth  .AND. ! gotlastday
      lrow = ROW()
      lcol = COL()-2
      lastday = DAY(tdate)
      gotlastday = .T.
   ENDCASE
   * current day
   IF tdate = mdate
      mrow = ROW()
      mcol = COL()-2
   ENDIF
   * system date
   IF tdate = DATE()
      crow = ROW()
      ccol = COL()-2
   ENDIF
   tdate=tdate+1        && increment date counter by 1 day
ENDDO
SETCOLOR(maincolor)
RETURN('')

* end function CalendBox()
*--------------------------------------------------------------------*


*!*********************************************************************
*!
*!       Function: NEXTMONTH()
*!
*!      Called by: GETDATE()
*!               : POPDATE        (procedure in POPDATE.PRG)
*!               : RECURRING      (procedure in POPDATE.PRG)
*!
*!*********************************************************************
FUNCTION nextmonth
** check for validity of date when skipping forward one month
** i.e., cannot go from March 31 to April 31, etcetera
PARAM origdate
PRIVATE ret_val
ret_val = CTOD("")
IF MONTH(origdate) = 12    && going to January of next year
   ret_val = STOD(STR( VAL(SUBSTR(DTOS(origdate), 1, 4 )) + 1, 4) + '01' + ;
             SUBSTR(DTOS(origdate), 7))
ELSE
   DO WHILE EMPTY(ret_val)
      ret_val = STOD(SUBSTR(DTOS(origdate), 1, 4) + ;
                IF(MONTH(origdate) < 9, '0', '') + ;
                LTRIM(STR(MONTH(origdate)+1)) + SUBSTR(DTOS(origdate), 7))
      origdate = origdate - 1
   ENDDO
ENDIF
RETURN (ret_val)

* end function NextMonth()
*--------------------------------------------------------------------*

* eof $calenda.prg
