*:*********************************************************************
*:
*:        Program: COLORPAL()
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-89, Greg Lief
*:  Last Modified: 08/07/90 -- added support for EGA/VGA enhanced
*:                             background colors
*:
*:     Allows user to select color from interactive palette
*:
*:  Procs & Fncts: COLORPAL()
*:               : COLORSAMPLE()
*:
*:          Calls: CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:               : EGAVGA()      (Assembler function in $EGAVGA.ASM)
*:               : BLINKBIT()    (Assembler function in BLINK.ASM)
*:               : COLOR_N2S()   (Clipper   function in $COLORS.PRG)
*:
*:*********************************************************************
FUNCTION colorpal
PARAM curr_color, pal_top, pal_left, redrawfunc, egavga
PRIVATE oldrow, oldcol, oldcolor, mrow, mcol, oldscrn, mfore, mback, ;
        keypress, xx, use_sample, mcurstop, mcursbot, num_colors, ;
        pal_bot, foreground, background

** establish top row and leftmost column for palette if not passed
egavga = IF(PCOUNT() < 5, .F., egavga)
pal_top  = IF(PCOUNT() < 2, IF(egavga, 8, 16), ;
           IF(pal_top < 6, 6, MIN(pal_top, IF(egavga, 8, 16))))
pal_bot =  pal_top + IF(egavga, 16, 8)
pal_left = IF(PCOUNT() < 3, 64, MIN(pal_left, 64))

** only display color sample if custom redraw UDF was not passed as
** fourth parameter, or if user passed a function name that doesn't exist
DO CASE
   CASE PCOUNT() < 4
      use_sample = .T.
   CASE TYPE(redrawfunc) $ 'UE'
      use_sample = .T.
   OTHERWISE
      use_sample = .F.
ENDCASE

** save environment
oldrow = ROW()
oldcol = COL()
oldcolor = SETCOLOR()
mcurstop = Curs_Top()          && get current cursor starting scan line
mcursbot = Curs_Bot()          && get current cursor ending scan line
NewCurs(32,0)                  && functional equivalent of SET CURSOR off
** if custom redraw function was passed, we will save the entire screen
** otherwise, we will only save the section of screen below the palette
IF use_sample
   oldscrn = SAVESCREEN(pal_top-6, pal_left, pal_bot, pal_left+15)
ELSE
   oldscrn = SAVESCREEN(0, 0, 24, 79)
ENDIF

** these string will be used in converting color numbers in the
** range of 0-255 to dBASE color strings (e.g. "W/N" etc)
foreground = 'N  B  G  BG R  RB GR W  N+ B+ G+ BG+R+ RB+GR+W+ '
background = 'N B G BGR RBGRW '

** set start-up color to current color if no parameter passed
curr_color = IF(PCOUNT() = 0, SETCOLOR(), UPPER(curr_color))

curr_color = color_s2n(curr_color)
IF egavga
   blinkbit(.F.)
ENDIF

** create character string of text/color attributes for color palette
** you might want to save THEPALETTE to a .MEM file to avoid this step
IF TYPE('thepalette') != 'C'
   thepalette = ''
   num_colors = IF(egavga, 255, 128)
   FOR xx = 0 TO num_colors
      thepalette = thepalette + chr(4) + chr(xx)
   NEXT
ENDIF
RESTSCREEN(pal_top, pal_left, pal_bot - 1, pal_left+15, thepalette)
SETCOLOR('GR+/N')
SCROLL(pal_bot, pal_left, pal_bot, pal_left+15, 00)
@ pal_bot, pal_left      SAY CHR(24)+CHR(25)+CHR(27)+CHR(26)
@ pal_bot, pal_left + 7  SAY CHR(17) + CHR(217)
@ pal_bot, pal_left + 13 SAY 'Esc'

** determine starting row and column within palette
mrow = pal_top + INT(curr_color/16)
mcol = pal_left + curr_color % 16

** commence main keypress loop
keypress = 0
DO WHILE keypress != 13 .AND. keypress != 27
   SETCOLOR(color_n2s(curr_color))
   IF use_sample
      colorsample()
   ENDIF
   ** draw blinking diamond to mark current color and get keypress
   @ mrow,mcol SAY CHR(219)
   keypress = INKEY(0)

   ** clear blinking diamond
   @ mrow,mcol SAY CHR(4)

   ** process keystroke
   DO CASE
      ** user pressed an arrow key (24=down, 5=up, 4=right, 19=left)
      CASE keypress = 24 .OR. keypress = 5 .OR. keypress = 4 .OR. keypress = 19
         ** adjust row position for up or down arrows
         mrow = IF(keypress = 24, IF(mrow = pal_bot - 1, pal_top, mrow + 1), ;
              IF(keypress = 5, IF(mrow = pal_top, pal_bot - 1, mrow - 1), mrow))
         ** adjust column position for left or right arrows
         mcol = mcol + IF(keypress = 4, IF(mcol = pal_left + 15, -15, 1), ;
                IF(keypress = 19, IF(mcol = pal_left, 15, -1), 0))
         ** change selected color number accordingly
         curr_color = (mrow - pal_top) * 16 + (mcol - pal_left)
       ** PgDn
      CASE keypress = 3
         curr_color = curr_color + (pal_bot - 1 - mrow) * 16
         mrow = pal_bot - 1
      ** PgUp
      CASE keypress = 18
         curr_color = curr_color - (mrow - 6) * 16
         mrow = 6
     ** user pressed Enter or Esc - let me out of here!!
      CASE keypress = 13 .OR. keypress = 27
         ** do nothing
      ** user pressed something else
      OTHERWISE
         TONE(220,1)
         TONE(220,1)
   ENDCASE
   IF ! use_sample
      SETCOLOR(color_n2s(curr_color))
      palbuffer = SAVESCREEN(pal_top - IF(use_sample, 6, 0), pal_left, ;
                             pal_bot, pal_left+15)
      xx = &redrawfunc.
      RESTSCREEN(pal_top - IF(use_sample, 6, 0), pal_left, pal_bot, ;
                             pal_left+15, palbuffer)
   ENDIF
ENDDO

** restore environment
@ oldrow, oldcol SAY ''
SETCOLOR(oldcolor)
IF use_sample
   RESTSCREEN(pal_top-6, pal_left, pal_bot, pal_left+15, oldscrn)
ELSE
   RESTSCREEN(0, 0, 24, 79, oldscrn)
ENDIF
NewCurs(mcurstop, mcursbot)         && restore previous cursor
** if user pressed Esc to abort, return the old color
RETURN (IF(keypress = 27, oldcolor, color_n2s(curr_color)))

* end function ColorPal()
*--------------------------------------------------------------------*

***********************************************************
* ColorSample(): draw sample box showing current selection
***********************************************************
FUNCTION ColorSample
@ pal_top-6, pal_left, pal_top-3, pal_left+15 BOX "ͻȺ "
@ pal_top-5, pal_left+3 SAY "Sample of"
@ pal_top-4, pal_left+1 SAY "current color"
RETURN('')

* end function ColorSample()
*--------------------------------------------------------------------*

* eof $colorpa.prg
