*:*********************************************************************
*:
*:        Program: $COLORS.PRG
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-89, Greg Lief
*:
*:     Conversion functions for colors - numeric to character string
*:                                       and vice versa
*:      Functions: COLOR_S2N()
*:               : COLOR_N2S()
*:
*:*********************************************************************

*****************************************************************
*
* Color_N2S(): convert color number (0-127) to dBASE color string
*
*****************************************************************
FUNCTION color_n2s
PARAM colorno
PRIVATE blinking, foreground, background
IF TYPE('colorno') = "C"
   colorno = BIN2I(colorno)
ENDIF
** these string will be used in converting color numbers in the
** range of 0-255 to dBASE color strings (e.g. "W/N" etc)
foreground = 'N  B  G  BG R  BR GR W  N+ B+ G+ BG+R+ BR+GR+W+ '
background = 'N B G BGR BRGRW '
blinking = (colorno > 127)
colorno = colorno % 128
RETURN (IF(blinking, '*', '') + ;
        TRIM(SUBSTR(foreground, (colorno % 16) * 3 + 1, 3)) + '/' + ;
        TRIM(SUBSTR(background, INT(colorno/16)*2+1, 2)))

* end function Color_N2S()
*--------------------------------------------------------------------*


***************************************************
*
* Color_S2N(): convert dBASE color string to number
*
***************************************************
FUNCTION color_s2n
PARAM colorno
PRIVATE foreground, mslash, mcomma, blinking, mfore, mback
foreground = 'N  B  G  BG R  BR GR W  N+ B+ G+ BG+R+ BR+GR+W+ '

** set start-up color to current color if no parameter passed
colorno = IF(PCOUNT() = 0, SETCOLOR(), UPPER(colorno))

******* parse this string to determine foreground and background colors
*
** first determine how many characters are in the foreground color
** by locating the first slash in the string
mslash = AT('/', colorno)
*
** background color will lie between the slash and the first comma,
mcomma = AT(',', colorno)
** but we also must allow for color parameters passed without a comma
mcomma = IF(mcomma = 0, LEN(colorno) + 1, mcomma)
*
** break out the foreground and background colors
mfore = SUBSTR(colorno, 1, mslash - 1)
blinking = .F.
IF AT('*', mfore) > 0
   mfore = STRTRAN(mfore, '*', '')
   blinking = .T.
ENDIF
mback = SUBSTR(colorno, mslash + 1, mcomma - mslash - 1)
*
** convert the string to a number
RETURN INT(AT(mfore, foreground) / 3) + INT(AT(mback, foreground) / 3) * 16 + ;
       IF(blinking, 128, 0)

* end function Color_S2N()
*--------------------------------------------------------------------*

* eof $colors.prg
