*:*********************************************************************
*:
*:        Program: DROPBOX()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988, Greg Lief
*:  Last modified: 05/07/89 - added coordinate validation for shadow
*:
*:          Calls: BOXTYPE()        (Function in $BOXTYPE.PRG)
*:                 BOX_TITLE()      (Function in $BOXTIT.PRG)
*:
*:     Draws drop-down box
*:
*:     Documented: 01/12/89 at 14:35                 SNAP! version 3.10
*:*********************************************************************
FUNCTION Dropbox
*-----------------------------------------------------------------------
* parameter list:   muprow  = top row of box
*                   mlfcol  = left column of box
*                   mdnrow  = bottom row of box
*                   mrtcol  = right column of box
*                   mbox_no = box style (1-6)
*                             1 = Double Line       2 = Single Line
*                             3 = Double Horizontal, Single Vertical
*                             4 = Single Horizontal, Double Vertical
*                             5 = Thick Border      6 = No Border
*                   delay   = duration of explosion (0-100 recommended)
*                   shadow  = optional flag for drop shadow (default none)
*                   mtitle  = optional box title (default none)
*----------------------------------------------------------------------
PARAM muprow, mlfcol, mdnrow, mrtcol, mbox_no, delay, shadow, mtitle, xbar
PRIVATE ii, jj, bx_string, ret_val
** establish defaults if parameters were not passed
mbox_no = IF(PCOUNT() < 5, 1, mbox_no)
delay  = IF(PCOUNT() < 6, 15, delay)
shadow = IF(PCOUNT() < 7, .F., shadow)
mtitle = IF(PCOUNT() < 8, '', mtitle)
xbar   = (PCOUNT() > 8)
** construct character string that will contain coordinates & contents of screen
** area underneath this box - this can be used with ByeByeBox()
ret_val = CHR(muprow) + CHR(mlfcol) + CHR(mdnrow + IF(shadow, 1, 0)) + ;
          CHR(mrtcol + IF(shadow, 2, 0)) + SAVESCREEN(muprow, mlfcol, ;
          mdnrow + IF(shadow, 1, 0), mrtcol + IF(shadow, 2, 0))
bx_string = Boxtype(mbox_no)
FOR ii = muprow TO mdnrow
   @ muprow, mlfcol, ii, mrtcol BOX bx_string
   FOR jj = 1 TO delay
   NEXT
NEXT

* draw shadow if requested and coordinates permit it
IF shadow .AND. mdnrow < 24 .AND. mrtcol < 78
   gfattr(muprow+1, mrtcol+1, mdnrow+1, mrtcol+2, 8)
   gfattr(MIN(mdnrow+1, 23), mlfcol+2, mdnrow+1, mrtcol+2, 8)
ENDIF

* draw title if requested
IF LEN(mtitle) > 0
   Box_Title(xbar)
ENDIF

RETURN (ret_val)

* end function DropBox()
*--------------------------------------------------------------------*

* eof $dropbox.prg
