*:*********************************************************************
*:
*:        Program: EXBOX()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988, Greg Lief
*:  Last modified: 01/12/89      14:28
*:
*:      Draws exploding box
*:
*:          Calls: BOXTYPE()     (Function in $BOXTYPE.PRG)
*:                 BOX_TITLE()   (Function in $BOXTIT.PRG)
*:
*:     Documented: 01/12/89 at 14:30                 SNAP! version 3.10
*:*********************************************************************
FUNCTION Exbox
************
* parameter list:     muprow  = top row of box
*                     mlfcol  = left column of box
*                     mdnrow  = bottom row of box
*                     mrtcol  = right column of box
*                     mbox_no = box style (1-6)
*                               1 = Double Line       2 = Single Line
*                               3 = Double Horizontal, Single Vertical
*                               4 = Single Horizontal, Double Vertical
*                               5 = Thick Border      6 = No Border
*                     delay   = duration of explosion (0-100 recommended)
*                     fill    = fill character (optional: default is space)
*                     shadow  = flag for drop shadow (optional: default none)
*                     mtitle   = optional title for box (default none)
*                     xbar    = use a cross-bar (optional, default no)

PARAM muprow, mlfcol, mdnrow, mrtcol, mbox_no, delay, fill, shadow, mtitle, xbar
***************
* medver = median between upper and lower rows
* medhor = median between left and right columns
* ii, jj = loop counters
* dom    = dominant (longer) axis
***************
PRIVATE medver, medhor, dom, ii, jj, tuprow, tdnrow, tlfcol, trtcol, ;
        ret_val, bx_string
IF PCOUNT() > 6
   fill = IF(LEN(fill) > 0, SUBSTR(fill, 1, 1), CHR(32))
ELSE
   fill = CHR(32)
ENDIF
shadow = IF(PCOUNT() < 8, .F., shadow)
mtitle = IF(PCOUNT() < 9, '', mtitle)
xbar   = (PCOUNT() > 9)
** construct character string that will contain coordinates & contents of screen
** area underneath this box - this can be used with ByeByeBox()
ret_val = CHR(muprow) + CHR(mlfcol) + CHR(mdnrow + IF(shadow, 1, 0)) + ;
          CHR(mrtcol + IF(shadow, 2, 0)) + SAVESCREEN(muprow, mlfcol, ;
          mdnrow + IF(shadow, 1, 0), mrtcol + IF(shadow, 2, 0))
bx_string = SUBSTR(Boxtype(mbox_no), 1, 8) + fill + ;
            SUBSTR(Boxtype(mbox_no), 10, 4)
IF ISCOLOR() .AND. delay > 0         && color monitor for full effect
   medver=INT(((mdnrow-muprow)+1)/2)   && determine vertical median
   medhor=INT(((mrtcol-mlfcol)+1)/2)   && determine horizontal median
   dom=MAX(medver,medhor)
   * determine dynamic box coordinates
   tuprow = muprow+medver
   tdnrow = mdnrow-medver
   tlfcol = mlfcol+medhor
   trtcol = mrtcol-medhor

   * main loop -- draw box out from center
   FOR ii = 1 TO dom
      @tuprow,tlfcol,tdnrow,trtcol BOX bx_string
      FOR jj=1 TO delay              && slow down explosion as desired
      NEXT

      *  test for border and expand if required
      IF ii < medver
         tuprow=tuprow-1
         tdnrow=tdnrow+1
      ENDIF
      IF ii < medhor
         tlfcol=tlfcol-1
         trtcol=trtcol+1
      ENDIF
   NEXT
ENDIF

* draw shadow if called for and coordinates are valid
IF shadow .AND. ISCOLOR() .AND. mdnrow < 24 .AND. mrtcol < 78
   gfattr(muprow + 1, mrtcol + 1, mdnrow + 1, mrtcol + 2, 8)
   gfattr(MIN(mdnrow+1, 23), mlfcol + 2, mdnrow + 1, mrtcol + 2, 8)
ENDIF

* draw outer box border
@ muprow, mlfcol, mdnrow, mrtcol BOX bx_string

* draw title if requested
IF LEN(mtitle) > 0
   Box_Title(xbar)
ENDIF

RETURN (ret_val)

* end function ExBox()
*--------------------------------------------------------------------*

* eof $exbox.prg
