*:*********************************************************************
*:
*:        Program: EXPBOX()
*:
*:         System: GRUMPFISH LIBRARY
*:        Authors: Mike Clerc and Greg Lief
*:      Copyright (c) 1990, Grumpfish Inc.
*:  Last modified: 05/20/90
*:
*:      Draws exploding box and saves array for TRUE implosion
*:
*:      Syntax: ExpBox(<top>, <left>, <bottom>, <right>,
*:              [<style>, <speed>, <color>, <array>, <title>])
*:
*: Required Parameters: <top> is a numeric representing the top row of
*:                      the box.
*:
*:                      <left> is a numeric representing the left column
*:                      of the box.
*:
*:                      <bottom> is a numeric representing the bottom row
*:                      of the box.
*:
*:                      <right> is a numeric representing the rightmost
*:                      column of the box.
*:
*: Optional Parameters: <style> is a numeric representing the box style:
*:                      1 = double line
*:                      2 = single line
*:                      3 = double horizontal, single vertical
*:                      4 = single horizontal, double vertical
*:                      5 = thick border
*:                      (see $BOXTYPE.PRG for more clarification)
*:                      The default is 1 (double line).
*:
*:                      <speed> kills... except in instances where it may
*:                      be a numeric representing the delay factor to
*:                      introduce into the explosion.  Values between 2
*:                      and 100 are recommended, with 2 being the fastest
*:                      and 100 being the slowest.  The default is 20.
*:
*:                      <color> is a character string that will be used as
*:                      the color in which to draw the box.  If you do not
*:                      pass this parameter, the box will be drawn in the
*:                      current color, so let's hope it is a good one.
*:                      NOTE: the original color will be restored upon exit.
*:
*:                      <array> is a character string that will serve as the
*:                      array in which to save the screen shots for implosion.
*:                      If you do not pass this parameter, the array will not
*:                      be created & you will not be able to implode this box.
*:
*:                      <title> is a character string to be centered across
*:                      the top row of the box.
*:
*:          Calls: BoxType()     (Function in $BOXTYPE.PRG)
*:                 Box_Title()   (Function in $BOXTITL.PRG)
*:                 GfAttr()      (ASM Function in GRUMPATT.ASM)
*:
*:     Documented: 05/21/90 at 11:30                 SNAP! version 3.10
*:*********************************************************************
FUNCTION ExpBox
PARAMETERS muprow, mlfcol, mdnrow, mrtcol, mbox_no, mdelay, mcolor, ;
           marray, mtitle
PRIVATE seed1, seed2, mmuprow, mmlfcol, mmdnrow, mmrtcol, xx, save_it, ;
        kounter, bx_string
IF TYPE("muprow") + TYPE("mlfcol") + TYPE("mdnrow") + TYPE("mrtcol") != "NNNN"
   RETURN (.F.)
ENDIF
bx_string = Boxtype(mbox_no)
mbox_no = IF(PCOUNT() < 5, 1, mbox_no)
mdelay = IF(PCOUNT() < 6, 20, mdelay)
mcolor = IF(PCOUNT() < 7, SETCOLOR(), mcolor)
marray = IF(PCOUNT() < 8, '', marray)
mtitle = IF(PCOUNT() < 9, '', mtitle)
seed1 = Int((mdnrow - muprow) / 2)
seed2 = Int((mrtcol - mlfcol) / 2)
STORE muprow + seed1 TO mmuprow, mmdnrow
STORE mlfcol + seed2 TO mmlfcol, mmrtcol
oldcolor = SETCOLOR(mcolor)
IF LEN(marray) > 0
   Public &marray.[20]
   Afill(&marray., "")
   &marray.[20]= muprow
   &marray.[19]= mlfcol
   &marray.[18]= mdnrow
   &marray.[17]= mrtcol
   &marray.[1] = SaveScreen(muprow, mlfcol, mdnrow + 2, mrtcol + 2)
   kounter = 2
   save_it = .T.
ELSE
   save_it = .F.
ENDIF
IF mdelay > 1
   DO WHILE (mmuprow > muprow .and. mmlfcol > mlfcol) .and. ;
            (mmdnrow < mdnrow .and. mmrtcol < mrtcol)
      @ mmuprow, mmlfcol, mmdnrow, mmrtcol Box bx_string
      If save_it
         &marray.[kounter] = SaveScreen(muprow, mlfcol, mdnrow + 2, mrtcol + 2)
         kounter = kounter + 1
      Endif
      For xx = 1 to mdelay
      Next
      mmuprow = mmuprow -1
      mmlfcol = mmlfcol - (seed2 / seed1)
      mmdnrow = mmdnrow + 1
      mmrtcol = mmrtcol + (seed2 / seed1)
   ENDDO
ENDIF
*** draw shadow if coordinates are valid
IF mdnrow < 24 .AND. mrtcol < 78
   GFAttr(muprow + 1, mrtcol + 1, mdnrow + 1, mrtcol + 2, 8)
   GFAttr(mdnrow + 1, mlfcol + 2, mdnrow + 1, mrtcol + 2, 8)
ENDIF
@muprow, mlfcol, mdnrow, mrtcol BOX bx_string
* draw title if requested
IF LEN(mtitle) > 0
   Box_Title()
ENDIF
SETCOLOR(oldcolor)
RETURN .T.

* end function ExpBox()
*--------------------------------------------------------------------*


*:*********************************************************************
*:
*:        Program: IMPBOX()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Mike Clerc and Greg Lief
*:      Copyright (c) 1990, Grumpfish Inc.
*:  Last modified: 05/20/90      23:28
*:
*:      Implodes a previously-exploded box
*:
*:      Syntax: ImpBox(<array> [, <speed>])
*:
*:  Parameters: <array> is the name of the array created by ExpBox().
*:              If you use a bogus array name, ImpBox() will terminate
*:              immediately.  If you use an array that was not created
*:              with ExpBox(), be prepared for unpredictable results.
*:              In such cases, you are encouraged to send us a print-
*:              screen of your wildest array misadventure.  Prizes will
*:              be awarded for zaniness and originality.
*:
*:              WARNING: be sure to pass the array name by REFERENCE (i.e.,
*:              not surrounded by quotes)!!
*:
*:              <speed> is a drug.  Don't give it to your children.
*:              However, your children would be interested to know that
*:              this is a numeric representing the delay factor to
*:              introduce into the implosion.  Values between 2
*:              and 100 are recommended, with 2 being the fastest
*:              and 100 being the slowest.  The default is 25.
*:
*:
*:     Documented: 05/21/90 at 11:30                 SNAP! version 3.10
*:*********************************************************************
FUNCTION ImpBox
PARAMETERS marray, mdelay
PRIVATE xx, yy
*** ensure that the array exists!
IF TYPE('marray') = "A"
   mdelay = IF(PCOUNT() < 2, 25, mdelay)
   For xx = 16 TO 1 STEP -1
      IF Len(marray[xx]) > 0
         RestScreen(marray[20], marray[19], marray[18]+2, marray[17]+2, ;
                    marray[xx])
         FOR yy = 1 TO mdelay
         NEXT
      ENDIF
   NEXT
   RELEASE marray
ENDIF
RETURN (.T.)

* end function ImpBox()
*--------------------------------------------------------------------*

* eof $explod.prg
