*:*********************************************************************
*:
*:       Function: FOTOCOPY()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-90, Greg Lief
*:
*:  Procs & Funcs: FOTOCOPY()
*:               : LOOPDLOOP()
*:
*:        Purpose: To 'photocopy' the currently selected record into
*:                 between one and four other work areas
*:
*:         Syntax: FOTOCOPY(<alias1> [, <alias2> [, <alias3> [, <alias4>]]])
*:
*:     Parameters: <alias1>..<alias4> are character strings representing
*:                 the work areas that you wish to copy the selected
*:                 record to.  Anything past <alias1> is optional.  You
*:                 could easily modify this to accept more than four
*:                 aliases, but four seemed like a good number to me.
*:
*:   Return Value: An integer numeric representing the number of aliases
*:                 into which the record was successfully copied.
*:
*:          Notes: FOTOCOPY() loops through all of the fields in the target
*:                 database to ensure that they exist in the source database.
*:                 If a field does exist, the type of the source and target
*:                 fields are also verified.  A blank record is appended to
*:                 the target database on the first loop through, and then
*:                 each field is replaced as found.
*:
*:         Inspired by Gary Van Slyke, Minneapolis, MN
*:
*:*********************************************************************
FUNCTION FotoCopy
PARAMETER alias1, alias2, alias3, alias4
PRIVATE num_flds, mfld, num_files, ret_val
ret_val = 0
num_files = PCOUNT()
num_flds = FCOUNT()
PRIVATE mfldnames_[num_flds], mfldtypes_[num_flds]
AFIELDS(mfldnames_, mfldtypes_)
FOR xx = 1 TO num_files
   ret_val = ret_val + ;
   LoopDLoop(IF(xx = 1, alias1, IF(xx = 2, alias2, IF(xx = 3, alias3, alias4))))
NEXT
RETURN ret_val

* end function FotoCopy()
*--------------------------------------------------------------------*


*:*********************************************************************
*:
*:      Function: LoopDLoop()
*:
*:       Purpose: Main loop to copy record into each database
*:
*:*********************************************************************
FUNCTION LoopDLoop
PARAMETER malias
PRIVATE num_flds, firstloop, xx, ret_val, wk_area, had2open
wk_area = ALIAS()
ret_val = 1
had2open = .F.
SELECT(SELECT(malias))
IF FCOUNT() = 0
   IF ! FILE(malias + '.dbf')
      ret_val = 0
   ELSE
      USE (malias)
      had2open = .T.
   ENDIF
ENDIF
IF ret_val = 1
   num_flds = FCOUNT()
   PRIVATE fldnames_[num_flds], fldtypes_[num_flds], mfield
   AFIELDS(fldnames_, fldtypes_)
   firstloop = .T.
   FOR xx = 1 TO num_flds
      ele = ASCAN(mfldnames_, fldnames_[xx])
      IF ele > 0
         IF fldtypes_[xx] = mfldtypes_[ele]
            IF firstloop
               APPEND BLANK
               firstloop = .F.
            ENDIF
            mfield = fldnames_[xx]
            REPLACE &mfield WITH &wk_area.->&mfield
         ENDIF
      ENDIF
   NEXT
   IF had2open
      USE
   ENDIF
ENDIF
SELECT &wk_area
RETURN ret_val

* end function LoopDLoop()
*--------------------------------------------------------------------*

* eof $fotocop.prg
