*:*********************************************************************
*:
*:       Function: GETDATE()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-9, Greg Lief
*:
*:  Procs & Funcs: GETDATE()
*:
*:          Calls: CALENDBOX()    (function  in $CALENDA.PRG)
*:               : NEXTMONTH()    (function  in $CALENDA.PRG)
*:               : CURS_TOP()     (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()     (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()      (Assembler function in CURSNEW.ASM)
*:
*:  Modifications: 10/05/89 - calendar box made smaller
*:                 11/16/89 - added parameter MDATE, which allows
*:                            you to pass the start-up date
*:                            SYNTAX: GetDate([<date>]
*:                            If parameter <date> is not passed, or
*:                            <date> represents a null date (such as
*:                            CTOD("  /  /  "), system date is used
*:                 11/28/89 - added parameters MTOP, MLEFT, and MCOLOR
*:                            to change position/color of calendar
*:*********************************************************************
FUNCTION GetDate
PARAMETER mdate, mtop, mleft, maincolor
PRIVATE oldcolor, oldscrn, mrow, mcol, row, tdate, tmonth, ;
 trow, tcol, frow, fcol, lrow, lcol, lastday, newrow, newcol, tempdate, ;
 moldrow, moldcol, xx, mcurstop, mcursbot, firstloop, redraw, calendscrn

*
* MROW & MCOL are the coordinates of the current date
* FROW & FCOL are the coordinates of the first day of the month
* LROW & LCOL are the coordinates of the last day of the month
* NEWROW & NEWCOL are used to macro substitute name of appropriate set
*    of coordinate variables to use from the three pairs listed above
* lastday is numeric value of last day of month (e.g., 29-31)
*

** use system date as starting date if not passed as parameter
** determine screen position and color if not passed as parameters
mdate = IF(TYPE('mdate') = 'D', IF(EMPTY(mdate), DATE(), mdate), DATE())
mtop      = IF(TYPE('mtop')     != 'N',     6, MIN(mtop, 11))
mleft     = IF(TYPE('mleft')    != 'N',    28, MIN(mleft, 56))
maincolor = IF(TYPE('maincolor') = 'U', 'W/B', maincolor)

* save screen coordinates
moldrow = ROW()
moldcol = COL()
oldscrn = SAVESCREEN(mtop, mleft, mtop + 13, mleft + 23)
oldcolor = SETCOLOR(maincolor)
mcurstop = Curs_Top()         && get the current cursor starting scan line
mcursbot = Curs_Bot()         && get the current cursor ending scan line
NewCurs(32,0)                 && functional equivalent of SET CURSOR OFF
firstloop = .T.   && flag so as to only draw calendar frame one time


IF TYPE('datekey')='U'   && if not defined in calling routine, use default
   datekey = -7
ENDIF
hotkey = (PCOUNT() > 0)        && set true if called by hot key
IF hotkey
   SET KEY datekey TO             && disable hot key to prevent recursion
ENDIF
SET DELETED ON

CalendBox(.F.)
** commence main keypress loop
DO WHILE .T.
   redraw = .F.
   keypress = INKEY(0)
   tdate = mdate                  && store highlighted date
   newrow = 'trow'
   newcol = 'tcol'
   DO CASE

   CASE keypress = 24 .OR. keypress = 5    && go forward/backward one week
      mdate = mdate + IF(keypress = 24, 7, -7)
      trow  = mrow  + IF(keypress = 24, 1, -1)
      tcol  = mcol

   CASE keypress = 19         && go back one day
      mdate = mdate - 1
      ** did we just go from Sunday to Saturday??
      trow = IF(DOW(mdate) = 7, mrow - 1, mrow)
      tcol = IF(DOW(mdate) = 7, mleft + 19, mcol - 3)

   CASE keypress = 4          && go forward one day
      mdate = mdate + 1
      ** did we just go from Saturday to Sunday??
      trow = IF(DOW(mdate) = 1, mrow + 1, mrow)
      tcol = IF(DOW(mdate) = 1, mleft + 1, mcol + 3)

   CASE keypress = 18         && go back one month
      IF MONTH(mdate) = 1     && going to December of previous year
         mdate = STOD(STR( VAL(SUBSTR(DTOS(mdate), 1, 4)) - 1, 4) + ;
                 '12' + SUBSTR(DTOS(mdate), 7) )
      ELSE
         ** check for validity of current date in previous month
         ** i.e., cannot go from March 31 to February 31, etcetera
         tempdate = CTOD('')
         DO WHILE EMPTY(tempdate)
            tempdate = STOD(SUBSTR(DTOS(mdate), 1, 4) + ;
                       IF(MONTH(mdate) < 11, '0', '') + ;
                       LTRIM(STR(MONTH(mdate) - 1)) + SUBSTR(DTOS(mdate), 7))
            mdate = mdate - 1
         ENDDO
         mdate = tempdate
      ENDIF

   CASE keypress = 3          && go forward one month
      mdate = NextMonth(mdate)

   CASE keypress = 1          && go to first day
      newrow = 'frow'
      newcol = 'fcol'
      mdate = STOD(SUBSTR(DTOS(mdate), 1, 6) + '01')

   CASE keypress = 6          && go to last day
      newrow = 'lrow'
      newcol = 'lcol'
      mdate = STOD(SUBSTR(DTOS(mdate), 1, 6) + STR(lastday, 2))

   CASE keypress = 13 .OR. keypress = 27   && Enter or Esc - quit
      EXIT

   OTHERWISE                  && any other keystroke
      LOOP

   ENDCASE
   ** if we changed months or a recurring appointment was added, redraw calendar
   IF MONTH(tdate) != MONTH(mdate) .OR. redraw
      CalendBox(.F.)
   ELSE
      SETCOLOR('+' + maincolor)
      @ mrow,mcol SAY STR(DAY(tdate),2)
      SETCOLOR('*+' + maincolor)
      @ &newrow,&newcol SAY STR(DAY(mdate),2)
      * store new row/column coordinates for highlighted date
      mrow = &newrow.
      mcol = &newcol.
   ENDIF
ENDDO

SETCOLOR(oldcolor)
RESTSCREEN(mtop, mleft, mtop + 13, mleft + 23, oldscrn)
@ moldrow,moldcol SAY ''
NewCurs(mcurstop, mcursbot)         && restore previous cursor
RETURN IF(keypress = 13, mdate, CTOD(""))

* end function GetDate()
*--------------------------------------------------------------------*

* eof $getdate.prg
