*:*********************************************************************
*:
*:        Program: $HBLINDS.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-89, Greg Lief
*:  Last modified: 01/04/90 - changed logic to write screen buffers
*:                            to disk file rather than using macros
*:
*:  Procs & Fncts: HBLINDOPEN()
*:               : HBLINDCLOS()
*:               : HBLINDSAVE()
*:
*:*********************************************************************

*!*********************************************************************
*!
*!       Function: HBLINDOPEN()
*!
*!       Purpose:  Draw title screen using horizontal blinds effect
*!
*!       NOTE:     Title screen must be pre-saved using HBLINDSAVE()
*!
*!       Syntax:   Hblindopen(<name of screen file>, [<delay>])
*!                 Default delay is 15 - change it to suit your hardware
*!
*!*********************************************************************
FUNCTION HBlindOpen
PARAM mfile, delay
PRIVATE handle, screen_[25], buffer, xx, yy
delay = IF(PCOUNT() = 1, 15, delay)
IF FILE(mfile)
   buffer = SPACE(160)
   handle = FOPEN(mfile)
   FOR xx = 1 TO 25
      FREAD(handle, @buffer, 160)
      screen_[xx] = SUBSTR(buffer, 1, 160)
   NEXT
   FCLOSE(handle)
   FOR xx = 4 TO 0 STEP -1
      FOR yy = 4 TO 0 STEP -1
         RESTSCREEN(yy * 5 + xx, 0, yy * 5 + xx, 79, screen_[yy*5 + xx + 1])
      NEXT
      FOR yy = 1 TO delay
      NEXT
   NEXT
ENDIF
RETURN('')

* end function HBlindOpen()
*--------------------------------------------------------------------*

*!*********************************************************************
*!
*!       Function: HBLINDCLOS()
*!
*!       Purpose:  Clear screen using horizontal blinds effect
*!
*!       Syntax:   Hblindclos([<delay>])
*!                 Default delay is 30 - change it to suit your hardware
*!
*!*********************************************************************
FUNCTION hblindclos
PARAM delay
PRIVATE xx, yy
delay = IF(PCOUNT() = 0, 30, delay)
FOR xx = 0 TO 4
   FOR yy = 0 TO 4
      SCROLL(yy * 5 + xx, 0, yy * 5 + xx, 79, 0)
   NEXT
   FOR yy = 1 TO delay
   NEXT
NEXT
RETURN('')

* end function HBlindClos()
*--------------------------------------------------------------------*


*!*********************************************************************
*!
*!       Function: HBLINDSAVE()
*!
*!       Purpose:  Save title screen to specified memory file
*!
*!       Syntax:   HBlindSave(<name of screen file>)
*!
*!*********************************************************************
FUNCTION hblindsave
PARAM mfile
PRIVATE buffer, handle, xx, ret_val
ret_val = .F.
handle = FCREATE(mfile)
IF FERROR() = 0
   ret_val = .T.
   FOR xx = 0 TO 24
      buffer = SAVESCREEN(xx, 0, xx, 79)
      IF FWRITE(handle, buffer) != 160
         ret_val = .F.
         EXIT
      ENDIF
   NEXT
   FCLOSE(handle)
ENDIF
RETURN ret_val

* end function HBlindSave()
*--------------------------------------------------------------------*

* eof $hblinds.prg
