*:*********************************************************************
*:
*:        Program: HELPBROW()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-90, Greg Lief
*:
*:          Calls: EXBOX()      (function in EXPBOX.PRG)
*:                 ERR_MSG()    (function in ERRORMSG.PRG)
*:                 HELPACT()    (function in $HELPACT.PRG)
*:
*:      Interactive help for data validation
*:
*:*********************************************************************

FUNCTION helpbrow
*
* PARAMETERS:
*              S_AREA = search area
*              FIELD1 = search field
*              HEAD1  = heading for first column
*   Optional:
*              FIELD2 = additional field to be displayed
*              HEAD2  = heading for second column
*              BXCOLOR= color to use for browse box (default: +w/b)
*              SAYROW = row coordinate for displaying second field
*              SAYCOL = column coordinate for displaying second field
*              TOPROW = top row for browse box
*              LT_COL = left column for browse box
*              BOTROW = bottom row for browse box
*              RT_COL = right column for browse box
*              MSG_ROW= row to display message
*

PARAM s_area, field1, head1, field2, head2, bxcolor, sayrow, saycol, ;
      toprow, lt_col, botrow, rt_col, msg_row
PRIVATE numflds, oldcolor, wk_area, curr_var, buff1, buff2
curr_var = READVAR()
wk_area = SELECT()
SELE &s_area
* check for existence of index file - if there isn't one, use LOCATE (ugh!)
IF INDEXORD() = 0
   LOCATE FOR &field1 == &curr_var   && have to change this for 5.0
ELSE
   ** determine whether to do case-insensitive search
   IF "UPPER(" $ UPPER( INDEXKEY(0) )
      SEEK UPPER(&curr_var)
   ELSE
      SEEK &curr_var
   ENDIF
ENDIF
IF FOUND()
   IF PCOUNT() > 6
      @ sayrow,saycol SAY &field2
   ENDIF
   SELE (wk_area)
   RETURN(.T.)
ENDIF
bxcolor = IF(PCOUNT() < 6, '+w/b', IF(EMPTY(bxcolor), '+w/b', bxcolor))
numflds = IF(PCOUNT() < 4, 1, IF(LEN(field2) > 0, 2, 1))
PRIVATE fields[numflds], heads[numflds], xx, mfield, mhead, ;
        a_name[fcount()], a_wid[fcount()], width, ele, sp
AFIELDS(a_name, '', a_wid)
IF numflds = 1
   fields[1] = field1
   heads[1] = head1
ELSE
   ** must determine required number of spaces between the fields
   ele = ASCAN(a_name,UPPER(field1))  && locate field in dbf structure array
   sp = MAX(a_wid[ele],LEN(head1)) - IF(TYPE(field1) = 'N', ;
            LEN(STR(&field1)), LEN(&field1)) + 1
   fields[1] = IF(TYPE(field1) = 'N', 'STR(', '') + field1 + ;
               IF(TYPE(field1) = 'N', ')', '') + '+"' + SPACE(sp) + ;
               '"+' + field2
   heads[1] = head1 + ' '  + head2
ENDIF

IF PCOUNT() < 9    && box coordinates not passed - must calculate 'em
   width = 6
   FOR xx = 1 TO numflds
      mfield = 'FIELD' + STR(xx, 1)
      mhead = 'HEAD' + STR(xx, 1)
      *** must set exact on to find a precise match!
      SET EXACT ON
      ele = ASCAN(a_name,UPPER(&mfield))
      SET EXACT OFF
      IF ele = 0
         Err_msg('Error in call to HELPBROW()')
         RETURN (.T.)
      ELSE
         ** increment width by the longer of column heading or field
         width = width + MAX(a_wid[ele], LEN(&mhead))
      ENDIF
   NEXT
   width = MAX(INT(80-width)/2, 0)
   lt_col = width
   rt_col = 79 - width
   toprow = 6
   botrow = 18
ENDIF

** establish row at which to display message
IF PCOUNT() < 13                   && not passed as parameter
   msg_row = 21
ELSE                               && cannot be lower than row 21
   msg_row = MIN(msg_row, 21)
ENDIF
GO TOP
oldcolor = SETCOLOR(bxcolor)
buff1 = Exbox(toprow, lt_col, botrow, rt_col, 3, 0, '', .T.)
buff2 = Exbox(msg_row-1, 12, msg_row+1, 66, 3, 0, '', .T.)
@ msg_row,14 SAY 'Move highlight bar to desired value and press Enter'
dbeditsrch(toprow+1, lt_col+1, botrow-1, rt_col-1, fields, heads, ;
           botrow, lt_col + INT((rt_col - lt_col)/2) - 4)
SETCOLOR(oldcolor)
ByeByeBox(buff1)
ByeByeBox(buff2)
if lastkey() != 27
   STORE &field1 TO &curr_var
   KEYBOARD CHR(13)
ENDIF
SELE (wk_area)
RETURN (.F.)

* eof _HELPBRO.PRG
