*:*********************************************************************
*:
*:        Program: $INDEXBA.PRG
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-90, Greg Lief
*:      With special thanks to John Stolte @ Omicron Software
*:
*:  Procs & Fncts: IndexBar()
*:               : GFShowBar()
*:               : GFKillBar()
*:
*:          Calls: CURS_TOP()    (Assembler function in CURSTOP.ASM)
*:               : CURS_BOT()    (Assembler function in CURSBOT.ASM)
*:               : NEWCURS()     (Assembler function in CURSNEW.ASM)
*:               : SHADOWBOX()   (function in $SHADOWB.PRG)
*:
*:   IndexBar() displays a graph showing percentage complete when
*:   indexing files.  When indexing is complete, IndexBar() will
*:   then strip the graph UDF() call from the .NTX file so that you
*:   don't have to look at the graph when the index is updated.
*:
*:*********************************************************************

***************************************************************************
*
*   Function: IndexBar()
*   Syntax:   IndexBar(<filename> [, <index key>] [, <row>] [, <showcount>])
*             <filename> = name of index file... to be displayed FYI
*             <index key> = self-explanatory -- if not passed, will
*             be assumed to be the same as the index filename
*             <row> = row at which to place window (default: 9)
*             <showcount> = if you pass this parameter, indexbar will
*                           display a record counter along with the graph
*   Example:  IndexBar('customer', 'lname')
*   Returns:  Logical: .T. if index is built, .F. if not
*   Notes:    will not process empty database files
*
***************************************************************************
FUNCTION IndexBar
PARAM mfile, mkey, mrow, showcount
PRIVATE ret_val, oldcolor, xx, gfcurstop, gfcursbot, maincolor, hicolor
ret_val = .F.
mkey = IF(PCOUNT() < 2, mfile, mkey)
*** make sure that the index key expression is valid to preclude a crash!
IF ! TYPE(mkey) $ "UE"
   mrow = IF(PCOUNT() < 3, 9, mrow)
   showcount = ( PCOUNT() > 3 )
   SET INDEX TO
   *** no point in going thru this rigmarole on an empty database (which would
   *** give us zero divide, a/k/a "the Great Divide")
   IF LASTREC() > 0
      mfile = UPPER(mfile) + IF(! "NTX" $ UPPER(mfile), ".NTX", "")
      gfcurstop = Curs_Top()
      gfcursbot = Curs_Bot()
      NewCurs(32, 0)
      maincolor = "W/" + IF(ISCOLOR(), "RB", "N")
      hicolor = "+GR/" + IF(ISCOLOR(), "RB", "N")
      oldcolor = SETCOLOR(maincolor)
      gfgbuffer = ShadowBox(mrow, 08, mrow + 6, 71, 2)
      SETCOLOR('+' + maincolor)
      @ mrow + 3, 10 SAY '' + REPLICATE(CHR(177), 58) + ''
      @ mrow + 4, 10 SAY '' + REPLICATE("", 9) + ';'
      @ mrow + 5, 10 SAY '0'
      FOR xx = 1 TO 10
         @ mrow + 5, 14 + (xx - 1) * 6 SAY STR(10 * xx, 3)
      NEXT
      @ mrow + 1, 22 SAY "Creating index file " + mfile + "..."
      SETCOLOR(hicolor)
      GO TOP
      INDEX ON GFShowBar(mkey) TO (mfile)
      SET INDEX TO
      GFKillBar(mfile, mkey)
      SET INDEX TO (mfile)
      ByeByeBox(gfgbuffer)
      NewCurs(gfcurstop, gfcursbot)
      SETCOLOR(oldcolor)
      ret_val = .T.
   ELSE
      INDEX ON &mkey TO (mfile)
      ret_val = .T.
   ENDIF
ENDIF
RETURN ret_val

* end function IndexBar()
*--------------------------------------------------------------------*

***************************************************************************
*
*   Function: GFShowBar()
*   Syntax:   GFShowBar(<index key>)
*             <index key> = self-explanatory
*   Purpose:  Display status graph whilst indexing
*   Example:  See above -- NOT MEANT FOR USE AS A STANDALONE
*   Returns:  the index key for each record
*
***************************************************************************
FUNCTION GFShowBar
PARAMETER mkey
IF RECNO() <= RECCOUNT()
   IF showcount
      IF RECNO() <> RECCOUNT()
         CENTER(mrow + 2,"INDEXING RECORD " + LTRIM(STR(RECNO())) + ;
                " OF " + LTRIM(STR(RECCOUNT())))
      ELSE
         CENTER(mrow + 2, SPACE(18) + "FINISHED !" + SPACE(18))
      ENDIF
   ENDIF
   @ mrow + 3, 11 SAY REPLICATE(CHR(219), RECNO() / RECCOUNT() * 58)
ENDIF
RETURN &mkey

* end function GFShowBar()
*--------------------------------------------------------------------*

***************************************************************************
*
*   Function: GFKillBar()
*   Syntax:   GFKillBar(<filename>)
*             <filename> = name of index file to be modified - enclose
*             in quotes - .NTX extension assumed.
*   Purpose:  Strip out the call to SHOWGRAPH()
*   Example:  GFKillBar('name.ntx')
*   Returns:  Logical: True (.T.) if successful, False (.F.) if not
*
***************************************************************************
FUNCTION GFKillBar
PARAM mfile, mkey
PRIVATE handle, buffer, pos, ret_val
ret_val = .F.
*** only attempt to open the .NTX low-level if we are guaranteed that no
*** indexes are currently open!  trying to do a low-level write on an open
*** .NTX file will cause lost clusters, general confusion, and a major
*** headache for you the developer, so try to avoid it like the plague
IF INDEXORD() = 0
   mfile = UPPER(mfile) + IF(! "NTX" $ UPPER(mfile), ".NTX", "")
   handle = FOPEN(mfile, 2)
   IF FERROR() = 0
      FSEEK(handle, 22)
      buffer = mkey + replicate(chr(0), 254 - len(mkey))
      IF FWRITE(handle, buffer) = 254
         ret_val = .T.
      ENDIF
      FCLOSE(handle)
   ENDIF
ENDIF
RETURN(ret_val)

* end function GFKillBar()
*--------------------------------------------------------------------*

* eof $indexba.prg
