*------------------------------------------------------------------------
*  Function: LITE_MENU()
*  System:   Grumpfish Library
*  Author:   Greg Lief
*  Copyright (c) 1988-90 Greg Lief
*  Purpose:  Provide colorful replacement to MENU TO... and ACHOICE()
*  Syntax:   LITE_MENU(<top>, <left>, <array>, [<confirmation>] ;
*            [, <unselected>, <selected>, <initial>, <title>, <trigger>])
*
*  Calls:    ShowDisOne()
*------------------------------------------------------------------------
FUNCTION lite_menu
PARAMETERS mtop, mleft, marray, confirmit, plaincolor, hilitcolor, ;
           selection, boxtitle, trigcolor
PRIVATE num_elem, xx, keypress, luvletters, oldcolor, max_len, fallout, ;
        oldscrn, ptr
*** set colors if they were not passed as parameters
plaincolor = IF(PCOUNT() < 5, "W/B", ;
             IF(LEN(plaincolor) = 0, "W/B", plaincolor))
hilitcolor = IF(PCOUNT() < 6, "+GR/R", ;
             IF(LEN(hilitcolor) = 0, "+GR/R", hilitcolor))
trigcolor  = IF(PCOUNT() < 9, "+W/B", ;
             IF(LEN(trigcolor) = 0, "+W/B", trigcolor))
** determine initially highlighted option
selection = IF(PCOUNT() < 7, 1, if(type('selection') != "N", 1, selection))
** determine box title
boxtitle = IF(PCOUNT() < 8, '', boxtitle)
confirmit = IF(PCOUNT() < 4, .T., confirmit)
oldcolor = SETCOLOR(plaincolor)
num_elem = LEN(marray)
luvletters = []
max_len = 0  && this will hold the length of the longest menu option
FOR xx = 1 TO num_elem
   max_len = MAX(LEN(STRTRAN(marray[xx], "~", "")), max_len)
NEXT
*** if the box coordinates were not passed (or were passed as null
*** strings), use alternates
if type("mtop") + type("mleft") != "NN"
   mtop = int(12 - (num_elem / 2)) - 1
   mleft = int((80 - max_len) / 2)
endif
** adjust maximum width based on length of box title
max_len = max(len(boxtitle) + 4, max_len)
oldscrn = ShadowBox(mtop, mleft, mtop + num_elem + 1, mleft + max_len + 1, ;
          2, boxtitle)
*** build the string containing available letters for selection
FOR xx = 1 TO num_elem
   *** the default is to add the first non-space character.  However,
   *** if there is a tilde embedded in this menu option, use the letter
   *** directly following it.
   ptr = AT("~", marray[xx])
   IF ptr > 0
      luvletters = luvletters + UPPER(SUBSTR(marray[xx], ptr + 1, 1))
   ELSE
      luvletters = luvletters + UPPER(SUBSTR(marray[xx], 1, 1))
   ENDIF
   ShowDisOne(xx)
NEXT
keypress = 0
fallout = .F.   && will be set True if confirmation is off
*** commence main key-grabbing loop
DO WHILE keypress != 13 .AND. keypress != 27
   *** first display current option in highlight color
   SETCOLOR(hilitcolor)
   @ mtop + selection, mleft + 1 SAY StrPad(STRTRAN(marray[selection], "~", ""), ;
                                     max_len)
   IF fallout
      EXIT
   ELSE
      keypress = INKEY(0)
      DO CASE

         *** down arrow - go down one option, observing wrap-around conventions
         CASE keypress = 24
            ShowDisOne(selection)
            selection = IF(selection = num_elem, 1, selection + 1)

         *** up arrow - go up one arrow, observing wrap-around conventions
         CASE keypress = 5
            ShowDisOne(selection)
            selection = IF(selection = 1, num_elem, selection - 1)

         *** Home - jump to top option
         CASE keypress = 1
            *** no point in going thru color rigmarole if we're already there
            IF selection != 1
               ShowDisOne(selection)
               selection = 1
            ENDIF

         *** End - jump to bottom option
         CASE keypress = 6
            *** no point in going thru color rigmarole if we're already there
            IF selection != num_elem
               ShowDisOne(selection)
               selection = num_elem
            ENDIF

         *** First Letter - jump to appropriate option
         CASE UPPER(CHR(keypress)) $ luvletters
            ShowDisOne(selection)
            selection = AT(UPPER(CHR(keypress)), luvletters)
            *** if we do not need confirmation, we will set the FALLOUT flag
            *** to True so that we will fall out immediately after redisplaying
            *** this option.  We could just as easily fall out right here, but
            *** it is more aesthetically pleasing to redisplay the selected
            *** option so that the user knows what they just selected
            fallout = ! confirmit

      ENDCASE
   ENDIF
ENDDO
SETCOLOR(oldcolor)
ByeByeBox(oldscrn)
RETURN (IF(LASTKEY() = 27, 0, selection))

* end function Lite_Menu()
*--------------------------------------------------------------------*


*------------------------------------------------------------------------
*  Function: ShowDisOne()
*  Purpose:  Display current prompt in mixed colors
*  Syntax:   ShowDisOne(<current option>)
*------------------------------------------------------------------------
FUNCTION ShowDisOne
PARAMETER WhereAmI
ptr = AT("~", marray[WhereAmI])
IF ptr > 0
   SETCOLOR(plaincolor)
   @ mtop + WhereAmI, mleft + 1 SAY StrPad(STRTRAN(marray[WhereAmI], "~", ""), ;
                                    max_len)
   SETCOLOR(trigcolor)
   @ mtop + WhereAmI, mleft + ptr SAY SUBSTR(marray[WhereAmI], ptr + 1, 1)
ELSE
   SETCOLOR(trigcolor)
   @ mtop + WhereAmI, mleft + 1 SAY SUBSTR(marray[WhereAmI], 1, 1)
   SETCOLOR(plaincolor)
   @ mtop + WhereAmI, mleft + 2 SAY ;
            StrPad(SUBSTR(marray[WhereAmI], 2), max_len-1)
ENDIF
SETCOLOR(plaincolor)
RETURN []

* end function ShowDisOne()
*--------------------------------------------------------------------*

* eof _litemen.prg
