*:*********************************************************************
*:
*:        Program: MENUV()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988, Greg Lief
*:
*:  Procs & Fncts: CENT_PROM()
*:
*:      Creates vertical bounce-bar menu which can be used in a loop
*:      to call various procedures
*:
*:     Documented: 01/15/89 at 20:05                SNAP! version 3.10
*:*********************************************************************
FUNCTION Menuv
*
* PARAMETERS:
*
* REQUIRED: A_PROM = array containing menu selections (& applicable messages)
*                    - can also contain applicable messages
*                    - can also contain procedure names to perform if selected
*
* OPTIONAL: MNAME     = menu title
*           BX_TYPE   = 1->double  2->single  3 & 4->mixture of both
*                       5->thick border (default is double line border)
*                       (add 5 for drop shadow effect, 10 for superduper 3-D)
*           BX_COLOR  = color for box and highlight bar (default: w/n,n/w)
*           TIT_COLOR = color for menu title (default: w/n)
********************************************************************************
PARAMETERS a_prom, mname, bx_type, bx_color, tit_color
PRIVATE oldcolor, choice, use_proc, ele, max_len, t, b, leftcol, rightcol, ;
        gfmenuscrn, oldrow, oldcol, ptr

** save environment
oldrow = ROW()
oldcol = COL()
oldcolor = SETCOLOR()

** establish defaults if parameters were not passed
mname = IF(PCOUNT() = 1, 'Menu', mname)
bx_type = IF(PCOUNT() < 3, 1, bx_type)
bx_color =  IF(PCOUNT() < 4 .OR. ! ISCOLOR(), 'W/N,I', bx_color)
tit_color = IF(PCOUNT() < 5 .OR. ! ISCOLOR(), 'W/N', tit_color)

** determine maximum length for menu selections and draw box accordingly
max_len = LEN(mname) + 4    && must be at least as wide as the menu title!
ele = LEN(a_prom)           && number of elements in array A_PROM
use_proc = .F.              && set true if user passed procedures to DO
FOR X = 1 TO ele
   ** do not include messages or options as part of length
   ptr = AT('$', a_prom[x])
   use_proc  = IF(! use_proc, (AT('^', a_prom[x]) > 0), .T.)
   max_len = MAX(IF(ptr > 0, ptr - 1, LEN(TRIM(a_prom[x]))), max_len)
NEXT
leftcol = INT((80-max_len) / 2)
rightcol = 80 - leftcol - max_len%2
** calculate top and bottom rows for box based on # of selections (max = 20)
IF ele < 21
   t = 12 - INT((ele + 3) / 2)
   b = t + ele + 2
ELSE
   RETURN(0)
ENDIF

** draw the box and any necessary shadows
IF bx_type > 5                         && 3-D effect
   IF bx_type > 10                     && super-duper 3-D
      SETCOLOR('W/W')
      SCROLL(t - 2, leftcol + 3, b - 2, rightcol + 4, 0)
   ENDIF
   SETCOLOR('W/N')
   SCROLL(t - 1, leftcol + 1, b - 1, rightcol + 2, 0)
ENDIF
SETCOLOR(bx_color)
Exbox(t, leftcol - 1, b, rightcol, IF(bx_type%5 = 0, 5, bx_type%5), 0)
SETCOLOR(tit_color)
@ t, INT((78 - LEN(mname)) / 2) SAY ' ' + mname + ' '

** main loop
DO WHILE .T.
   SETCOLOR(bx_color)
   FOR X = 1 TO ele
      cent_prom(t + x + 1, a_prom[x])
   NEXT
   MENU TO choice
   ** exit the loop if they selected the last choice, Escaped, or no
   ** procedures were passed to be done
   IF choice = LEN(a_prom) .OR. choice = 0 .OR. ! use_proc
      EXIT
   ELSE
      ptr = AT('^',a_prom[choice])
      IF ptr > 0
         mproc = TRIM(SUBSTR(a_prom[choice], ptr+1))
         SAVE SCREEN TO gfmenuscrn
         DO &mproc
         RESTORE SCREEN FROM gfmenuscrn
      ENDIF
   ENDIF
ENDDO
SETCOLOR(oldcolor)
@ oldrow, oldcol SAY ''
RETURN(choice)

* end function MenuV()
*--------------------------------------------------------------------*


*************************************************************************
*
* CENT_PROM(): Displays menu options centered, along with messages if any
*
*************************************************************************
FUNCTION cent_prom
PARAM row, strng
PRIVATE prom, msg, ptr
** is there a message corresponding to this option??  if so, display it
ptr = AT('$', strng)
IF ptr > 0
   ** parse string to get the menu prompt
   prom = SUBSTR(strng, 1, ptr - 1)
   ** parse string to get MESSAGE -- be sure to ignore any procedure names!
   IF AT('^', strng) > 0
      msg = SUBSTR(strng, ptr + 1, AT('^', strng) - ptr - 1)
   ELSE
      msg = SUBSTR(strng, ptr + 1, LEN(strng) - ptr)
   ENDIF
   @ row, INT((80 - LEN(prom)) / 2) PROMPT prom MESSAGE msg
ELSE
   @ row, INT((80 - LEN(TRIM(strng))) / 2) PROMPT TRIM(strng)
ENDIF
RETURN(0)

* end function Cent_Prom()
*--------------------------------------------------------------------*

* eof $menuv.prg
