*:*********************************************************************
*:
*:        Program: IsPostCode()
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-90, Greg Lief
*:
*:  Provide basic validation for Canadian postal codes
*:     1) verify that length is seven
*:     2) verify that format is X#X #X#
*:     3) check first character against array of territory codes
*:
*:  Syntax: IsPostCode(@<state>, @<postal code>)
*:
*:          <state> and <postal code> are both character expressions.
*:          They must both be passed by reference (i.e., preceded by
*:          "@").
*:
*:  Return Value: Logical: True if postal code is valid, False if not
*:
*:  Example:
*:
*:          mstate = SPACE(2)
*:          mpostcode = SPACE(10)
*:          @ 19, 0 GET mstate PICTURE "@!"
*:          @ 20, 0 GET mpostcode PICTURE "@!" VALID ;
*:                                IsPostCode(@mstate, @mpostcode)
*:          READ
*:
*:  Calls: Err_Msg()      (function in $ERRMSG.PRG)
*:
*:*********************************************************************
FUNCTION IsPostCode
PARAMETER mstate, mpostalcode
PRIVATE ret_val, mpostcode, postcodes_[18], mnumbers, ele, ptr, improv, ;
        badletters
** these letters cannot appear in the postal code except for the beginning
badletters = 'DFIOQU'
mnumbers = '0123456789'
postcodes_[1] = "NFANewfoundland"
postcodes_[2] = "NSBNova Scotia"
postcodes_[3] = "PECPrince Edward Island"
postcodes_[4] = "NBENew Brunswick"
postcodes_[5] = "PQGQuebec (Eastern)"
postcodes_[6] = "PQHMontreal"
postcodes_[7] = "PQJQuebec (Western)"
postcodes_[8] = "ONKOntario (Eastern)"
postcodes_[9] = "ONLOntario (Central)"
postcodes_[10] = "ONMToronto"
postcodes_[11] = "ONNOntario (Southwestern)"
postcodes_[12] = "ONPOntario (Northern)"
postcodes_[13] = "MBRManitoba"
postcodes_[14] = "SKSSaskatchewan"
postcodes_[15] = "ABTAlberta"
postcodes_[16] = "BCVBritish Columbia"
postcodes_[17] = "NTXNorthwest Territories"
postcodes_[18] = "YTYYukon"
ret_val = .F.
DO CASE
   ** first check length - must be 7
   CASE LEN(TRIM(mpostalcode)) != 7
      Err_Msg("Canadian postal codes must be 7 characters long!")
   CASE ! (ISALPHA(SUBSTR(mpostalcode, 1, 1)) .AND. ;
         SUBSTR(mpostalcode, 2, 1) $ mnumbers .AND. ;
         ISALPHA(SUBSTR(mpostalcode, 3, 1)) .AND. ;
         SUBSTR(mpostalcode, 4, 1) = [ ] .AND. ;
         SUBSTR(mpostalcode, 5, 1) $ mnumbers .AND. ;
         ISALPHA(SUBSTR(mpostalcode, 6, 1)) .AND. ;
         SUBSTR(mpostalcode, 7, 1) $ mnumbers)
      Err_Msg("Canadian postal codes must be of the format 'X#X #X#'")
   OTHERWISE
      ** the letters D/F/I/O/Q/U cannot appear in the postal code -
      ** check that first
      IF UPPER(SUBSTR(mpostalcode, 3, 1)) $ badletters .OR. ;
                UPPER(SUBSTR(mpostalcode, 6, 1)) $ badletters
         Err_Msg("Invalid postal code")
      ELSE
         ** up to this point everything checks out okay... now make
         ** sure that the first character matches the geographic area
         ele = ASCAN(postcodes_, UPPER(mstate))
         IF ele > 0
            ** only use first word of the province -- i.e., "Ontario" rather
            ** than "Ontario (Eastern)" -- because if they entered the wrong
            ** first digit, we don't really know which section they wanted
            ptr = AT('(', postcodes_[ele])
            IF ptr > 0
               improv = SUBSTR(postcodes_[ele], 4, ptr - 5)
            ELSE
               improv = SUBSTR(postcodes_[ele], 4)
            ENDIF
            DO WHILE mstate == SUBSTR(postcodes_[ele], 1, 2) .AND. SUBSTR(mpostalcode, 1, 1) != SUBSTR(postcodes_[ele], 3, 1)
               ele = ele + 1
            ENDDO
            IF SUBSTR(mpostalcode, 1, 1) != SUBSTR(postcodes_[ele], 3, 1)
               Err_Msg("Invalid postal code for " + improv)
            ELSE
               ret_val = .T.
            ENDIF
         ELSE
            Err_Msg("Invalid province code")
         ENDIF
      ENDIF
ENDCASE
RETURN ret_val

* end function IsPostCode()
*--------------------------------------------------------------------*

* eof $ocanada.prg
