*:*********************************************************************
*:
*:        Program: SHADOWBOX()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1989, Greg Lief
*:  Last modified: 05/07/89
*:
*:      Draws shadowed box
*:
*:          Calls: BOXTYPE()     (Function in $BOXTYPE.PRG)
*:                 BOX_TITLE()   (Function in $BOXTIT.PRG)
*:
*:     Documented: 01/12/89 at 14:30                 SNAP! version 3.10
*:*********************************************************************
FUNCTION Shadowbox
************
* Required Parameters:  muprow = top row of box
*                       mlfcol  = left column of box
*                       mdnrow  = bottom row of box
*                       mrtcol  = right column of box
*
* Optional Parameters:  mbox_no = box style (1-6)
*                                 1 = Double Line       2 = Single Line
*                                 3 = Double Horizontal, Single Vertical
*                                 4 = Single Horizontal, Double Vertical
*                                 5 = Thick Border      6 = No Border
*                      mtitle   = optional title
*                      xbar     = use a cross-bar (optional, default no)

PARAMETERS muprow, mlfcol, mdnrow, mrtcol, mbox_no, mtitle, xbar
PRIVATE bx_string, ret_val
mbox_no = IF(PCOUNT() < 5, 1, mbox_no)
mtitle = IF(PCOUNT() < 6, '', mtitle)
** construct character string that will contain coordinates & contents of screen
** area underneath this box - this can be used with ByeByeBox()
ret_val = CHR(muprow) + CHR(mlfcol) + CHR(mdnrow + 1) + CHR(mrtcol + 2) + ;
          SAVESCREEN(muprow, mlfcol, mdnrow + 1, mrtcol + 2)
bx_string = Boxtype(mbox_no)
xbar =   (PCOUNT() > 6)

* draw shadow if coordinates are valid
IF mdnrow < 24 .AND. mrtcol < 78
   gfattr(muprow+1, mrtcol+1, mdnrow+1, mrtcol+2, 8)
   gfattr(MIN(mdnrow+1, 23), mlfcol+2, mdnrow+1, mrtcol+2, 8)
ENDIF

* draw box
@ muprow, mlfcol, mdnrow, mrtcol BOX bx_string

** draw title if requested
IF LEN(mtitle) > 0
   Box_Title(xbar)
ENDIF

RETURN (ret_val)

* end function ShadowBox()
*--------------------------------------------------------------------*

* eof $shadowb.prg
