*:*********************************************************************
*:
*:        Program: SHRBOX()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988, Greg Lief
*:  Last modified: 10/02/88     17:13
*:
*:  Draws the incredible shrinking box
*:
*:     Documented: 12/26/88 at 18:06                SNAP! version 3.10
*:*********************************************************************
* parameter list:     uprow   = top row of box
*                     lfcol   = left column of box
*                     dnrow   = bottom row of box
*                     rtcol   = right column of box
*                     delay   = duration of explosion (0-100 recommended)
*                     bk_col  = background color (default: current color)
*
*  Each axis must contain at least one line or an error will occur.
*

FUNCTION Shrbox
PARAM uprow, lfcol, dnrow, rtcol, delay, bk_col

*
* medver = median between upper and lower rows
* medhor = median between left and right columns
* ii, jj = loop counters
* dom = dominant (longer) axis
*
***************
PRIVATE medver,medhor,dom,ii,jj
* determine color to use -or- color to switch back to (if color parm passed)
IF PCOUNT() < 6
   bk_col = SETCOLOR()
ELSE
   oldcolor = SETCOLOR()
ENDIF
SETCOLOR(bk_col)
IF ISCOLOR()    && one needs a color monitor to get the full effect
   medver=INT(((dnrow-uprow)+1)/2)   && determine vertical median
   medhor=INT(((rtcol-lfcol)+1)/2)   && determine horizontal median
   dom=MAX(medver,medhor)
   FOR ii = dom TO 1 step -1
      FOR jj=1 TO delay              && slow down explosion as desired
      NEXT
      
      *  blank out previous box outline
      @uprow,lfcol,dnrow,rtcol BOX '        '
      IF ii < medver
         uprow=uprow+1
         dnrow=dnrow-1
      ENDIF
      IF ii < medhor
         lfcol=lfcol+1
         rtcol=rtcol-1
      ENDIF
   NEXT
ENDIF
* blank out remaining box area
SCROLL(uprow,lfcol,dnrow,rtcol,0)
* change color back to previous only if color parameter was passed
IF PCOUNT() = 6
   SETCOLOR(oldcolor)
ENDIF
RETURN []

* end function ShrBox()
*--------------------------------------------------------------------*

* eof $shrbox.prg
