*****************************************************************
*
* Function: TimeStamp()
* System:   Grumpfish Library
* Author:   Greg Lief
* Copyright (c) 1988-90 Greg Lief
* Purpose:  to implant a date/time stamp into a memo field. Handy
*           for customer contact records and so forth
*
* Syntax: TimeStamp(<memofield> [, <value>] [, <edit_it>])
*
* Parameters: <memofield> is the name of the memofield to append
*             into (enclosed in quotes).
*
*             Optional parameter <value> is a character string,
*             containing additional info to be written to the memo.
*
*             If you pass the third parameter (<edit_it>), you will
*             be placed in the memo field, just past the timestamp,
*             for editing.
*
* Returns: nada
*
* Examples:   TimeStamp('comments')
*             TimeStamp('comments', mNAME + mADDRESS + mCITY)
*             TimeStamp('comments', '', .T.)  && edit the memo also
*
*    Calls:   MEMEDIT()  (function contained in $MEMEDIT.PRG)
*
FUNCTION TimeStamp
PARAMETER mfield, mvalue, edit_it
PRIVATE oldinsert, mstring
mvalue = IF(PCOUNT() < 2, '', mvalue)
IF TYPE(mfield) = 'M'
   *** if third parameter was passed, put them into memoedit()
   mstring = DTOC(DATE()) + [ ] + TIME() + [ ] + ;
             IF(! EMPTY(mvalue), mvalue, []) + CHR(13) + CHR(10)
   IF PCOUNT() > 2
      oldinsert = READINSERT(.T.)
      KEYBOARD mstring + CHR(13) + CHR(10) + CHR(5) + CHR(5) + REPLICATE(CHR(4),18)
      MEMEDIT(mfield)
      READINSERT(oldinsert)
   ELSEIF RLOCK()
      REPLACE &mfield WITH mstring + &mfield
   ELSE
      Err_Msg("Could not write timestamp to " + mfield)
   ENDIF
ENDIF
RETURN []

* end function TimeStamp()
*--------------------------------------------------------------------*

* eof $timesta.prg
