*:*********************************************************************
*:
*:       Function: UP_N_DOWN()
*:         Author: Greg Lief
*:      Copyright (c) 1990 Greg Lief
*:         Special thanks to Greg Smith for his suggestions!
*:
*:      Flexible upper and lower case data entry -- for example, if
*:      you have one field for the entire name (first and last),
*:      this will automatically make each name proper (first letter
*:      upper, rest lower).
*:
*:*********************************************************************
FUNCTION up_n_down
PARAMETERS mvar
PRIVATE nextupper, mrow, mcol, oldcolor, key, mkey, ret_val, mlength, origvar
*** save previous environment
oldcolor = SETCOLOR()
mrow = ROW()
mcol = COL()
SETCOLOR(SUBSTR(oldcolor, AT(',', oldcolor)+1))  && more trickery

@ mrow, mcol SAY mvar                  && for initial display
nextupper = .T.                        && 1st character must be uppercase
ret_val = REPLICATE('X', LEN(mvar))    && the PICTURE clause
mlength = LEN(mvar)                    && set maximum length for data entry
origvar = mvar                         && save original value in case of ESC
mvar = []
key = 0

***
* begin main loop
*
* we simulate an actual READ by allowing the following active keys:
* Enter (13),  Esc (27),  Ctrl-W (23),  UpArrow (5),
* DownArrow (2), PgUp (18), and PgDn (3)
*
* If you want to limit escape from this routine, it is a simple
* matter to remove the desired key values from the next statement
*
* Note also that we fall out of the loop when MLENGTH reaches 0.  This
* effectively simulates SET CONFIRM OFF.
***
DO WHILE key != 13 .AND. key != 27 .AND. key != 23 .AND. key != 5 .AND. ;
         key != 2  .AND. key != 18  .AND. key != 3 .AND. mlength > 0
   @ mrow, mcol SAY mvar PICTURE ret_val
   key = INKEY(0)
   DO CASE

      *** user pressed an alphanumeric or punctuation key
      CASE (key > 32 .AND. key < 91) .OR. (key > 96 .AND. key < 123)
         *** if this is an alpha, we must do additional testing
         mkey = chr(key)
         IF key > 64
            IF nextupper
               *** if the next character should be upper but they entered
               *** an upper, switch case to lower: huh?  This is so that
               *** someone could enter, for example, "dBase"
               IF key < 91
                  mkey = CHR(key + 32)
               *** force upper-case if called for
               ELSE
                  mkey = CHR(key - 32)
               ENDIF
            ENDIF
         ENDIF
         mvar = mvar + mkey
         nextupper = .F.
         mlength = mlength - 1       && decrease the length counter

      *** user pressed space bar
      CASE key = 32
         nextupper = .T.      && next character will be uppercase
         mvar = mvar + [ ]
         mlength = mlength - 1       && decrease the length counter

      *** user pressed backspace
      CASE key = 8 .and. len(mvar) > 0
         mvar = SUBSTR(mvar, 1, LEN(mvar) - 1)
         * clear last character the easy way
         SCROLL(mrow, COL()-1, mrow, COL()-1, 0)
         *** if we are at a space or back at the beginning of the
         *** string, next character must be uppercase
         nextupper = (SUBSTR(mvar, LEN(mvar)) = CHR(32) .OR. LEN(mvar) = 0)
         mlength = mlength + 1       && increase the length counter

   ENDCASE
ENDDO

DO CASE
   *** user pressed Esc to exit -- reset MVAR to its original value
   CASE key = 27
      mvar = origvar
      KEYBOARD CHR(13)
   *** entered max number of characters - stuff Enter to continue
   CASE mlength = 0
      KEYBOARD CHR(13)
   *** any other key -- stuff it into buffer to be acted upon properly
   OTHERWISE
      KEYBOARD CHR(key)
ENDCASE

*** restore environment to previous state
@ mrow, mcol SAY ''
SETCOLOR(oldcolor)
*** pad the string back to its original length
mvar = mvar + SPACE(LEN(ret_val) - LEN(mvar))
RETURN ret_val

* end function Up_N_Down()
*--------------------------------------------------------------------*

* eof $upndown.prg
