*:*********************************************************************
*:
*:        Program: $WAITON.PRG
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-90, Greg Lief
*:  Last modified: 08/07/90 -- added many new parameters
*:
*:  Procs & Fncts: WAITON()
*:               : WAITOFF()
*:
*:*********************************************************************

*----------------------------------------------------------------------
*
*  Function: WaitOn()
*
*   Purpose: Display a message on the screen -- ideal for printing reports,
*            reindexing, or any other process that takes a while to do
*
*    Syntax: Waiton([<msg>] [, <sound>] [, <color>] [, <top>] [, <left>])
*
*            All parameters are optional.
*
*            <msg> is a character string representing the message to be
*            displayed.  If you do not pass this parameter, WaitOn() will
*            assume that you are printing and display the message
*            "Now printing... please wait".
*
*            <sound> is a logical value.  If you pass True (.T.), WaitOn()
*            and emit two tones when displaying the message and WaitOff()
*            will emit two tones when restoring the screen.  Pass False
*            (.F.) if you do not want to use sound.
*
*            <color> is a character string representing the color in
*            which to display the box.  The message will be displayed
*            in the blinking version of this color.  For example, if you
*            pass "W/B", the message will be displayed with "*W/B".  If
*            you do not pass this parameter or pass a null string, the
*            default will be yellow on black ("+GR/N").
*
*            <top> is a numeric representing the top row at which to draw
*            the message box.  If you do not pass this parameter, WaitOn()
*            will draw the box from rows 11 to 13.
*
*            <left> is a numeric representing the left column at which to
*            draw the message box.  If you do not pass this parameter,
*            WaitOn() will center the box horizontally on the screen.
*
*   Returns: Nothing
*
*     Calls: ShadowBox()
*
*----------------------------------------------------------------------
FUNCTION Waiton
PARAMETER msg, sound, color, top, left
PRIVATE oldcolor
PUBLIC waitscreen, waitsound          && used in WaitOff()
*** establish all parameters -- use defaults if not passed
msg = IF(PCOUNT() = 0, [], msg)
IF msg == []     && if no message was passed, assume "printing" message
   msg='Now printing... please wait'
ENDIF
waitsound = IF(PCOUNT() < 2, .T., sound)
color = IF(PCOUNT() < 3, "+GR/N", IF(color == [], "+GR/N", color))
top = IF(PCOUNT() < 4, 11, top)
left = IF(PCOUNT() < 5, INT(76 - LEN(msg)) / 2, left)
oldcolor = SETCOLOR(color)
waitscreen = ShadowBox(top, left, top + 2, left + 4 + LEN(msg), 1)
SETCOLOR('*' + color)
@ top + 1, left + 2 SAY msg
IF waitsound
   TONE(800, 1)
   TONE(1600,1)
ENDIF
SETCOLOR(oldcolor)
RETURN []

* end function WaitOn()
*--------------------------------------------------------------------*


*----------------------------------------------------------------------
*
*  Function: WaitOff()
*
*   Purpose: Restore the screen after displaying a message with WaitOn()
*
*    Syntax: Waitoff()
*
*   Returns: Nothing
*
*     Calls: BYEBYEBOX()
*
*----------------------------------------------------------------------
FUNCTION Waitoff
*** if the variable WAITSCREEN is undefined, you did not call WaitOn()
*** first, and thus have no business calling WaitOff()!
IF TYPE('waitscreen') != 'U'
   IF waitsound
      TONE(1600, 1)
      TONE(800,  1)
   ENDIF
   ByeByeBox(waitscreen)
ENDIF
RELEASE waitscreen
RELEASE waitsound
RETURN []

* end function WaitOff()
*--------------------------------------------------------------------*

* eof $waiton.prg
