*:*********************************************************************
*:
*:        Program: YES_NO2()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1989, Greg Lief
*:  Last modified: 05/07/89     12:25
*:
*:  Syntax:     Yes_No(<question>, [<top row>, <yes_msg>, <no_msg>,
*:              <init_val>, <box_color>, <opt_color>])
*:
*:  Parameters: <question> -- message to display (character string)
*:              <top row> -- top row of box (numeric, default 20)
*:              <yes_msg> -- yes prompt (character, default " YES ")
*:              <no_msg>  -- no prompt (character, default " NO ")
*:              <init_val>-- which value to highlight first
*:                           (.T. = Yes, .F. - No, default Yes)
*:              <box_color>  color to draw box and question (character,
*:                           string, default +W/RB)
*:              <opt_color>  color to draw prompts (character string,
*:                           default W/RB for unselected, I for selected)
*:
*:  Returns:  True if user selected YES, False if user selected NO or Escaped
*:
*:      Documented 05/07/89 at 12:26                SNAP!  version 3.12a
*:*********************************************************************

FUNCTION Yes_no2
PARAMETERS question, toprow, yes_msg, no_msg, init_val, box_color, opt_color
PRIVATE _boxleft, _optslen, oldcolor, oldscrn, optionstr, key, ;
        mcurstop, mcursbot

** establish variables if not passed as parameters
toprow    = IF(PCOUNT() = 1, 20, MIN(toprow, 20))
yes_msg   = IF(PCOUNT() < 3, " YES ", IF(LEN(yes_msg) > 0, yes_msg, " YES "))
no_msg    = IF(PCOUNT() < 4, " NO ", IF(LEN(no_msg) > 0, no_msg, " NO "))
box_color = IF(PCOUNT() < 6, "W+/RB", box_color)
opt_color = IF(PCOUNT() < 7, "W/RB, I", opt_color)
** do type checking on INIT_VAL to ensure that it is logical type
init_val  = IF(PCOUNT() < 5, .T., IF(TYPE('init_val') != 'L', .T., init_val))

** save previous cursor and then turn it off for now
mcurstop = curs_top()          && get current cursor starting scan line
mcursbot = curs_bot()          && get current cursor ending scan line
newcurs(32,0)                  && functional equivalent of SET CURSOR off

** determine width of box, and accordingly, left column of box
_optslen = LEN(yes_msg) + LEN(no_msg) + 3    && allow 3 spaces btwn YES & NO
_boxleft = INT((74 - MAX(LEN(question), _optslen)) / 2)

** now that that's over with, let's get on with it...
oldcolor  = SETCOLOR(box_color)
oldscrn   = Shadowbox(toprow, _boxleft, toprow+3, 79 - _boxleft, 2)
Center(toprow + 1, question)
optionstr = FirstChar(yes_msg) + FirstChar(no_msg)

** main keypress loop
DO WHILE .T.
   ** draw yes and no options in appropriate colors
   SETCOLOR(IF(init_val, 'I', opt_color))
   @ toprow + 2, INT((80 - _optslen) / 2) SAY yes_msg
   SETCOLOR(IF(! init_val, 'I', opt_color))
   @ toprow + 2, col() + 3 SAY no_msg

   key = INKEY(0)
   DO CASE
      CASE key = 27
         init_val = .F.
         EXIT
      CASE key = 13
         EXIT
      CASE key = 4 .OR. key = 19     && left or right arrow
         init_val = ! init_val
      CASE UPPER(CHR(key)) $ optionstr
         init_val = (AT(UPPER(CHR(key)), optionstr) = 1)
         EXIT
   ENDCASE
ENDDO

SETCOLOR(oldcolor)
ByeByeBox(oldscrn)
newcurs(mcurstop, mcursbot)         && restore previous cursor
RETURN (init_val)

* end function Yes_No2()
*--------------------------------------------------------------------*


*************************************************************
* firstchar(): find first non-space character in each option
*              to be used as trigger key
*************************************************************
FUNCTION FirstChar
PARAMETER the_msg
PRIVATE xx
FOR xx = 1 TO LEN(the_msg)
   IF SUBSTR(the_msg, xx, 1) != CHR(32)
      EXIT
   ENDIF
NEXT
RETURN (UPPER(SUBSTR(the_msg, xx, 1)))

* end function FirstChar()
*--------------------------------------------------------------------*

* eof $yesno2.prg
