/*
   Program: STATBAR.PRG
   System: GRUMPFISH LIBRARY
   Author: Greg Lief
   Copyright (c) 1988-94, Greg Lief
   CA-Clipper 5.x version
   Compile instructions: clipper gfstatba /n /w

   Procs & Fncts: GFStatInit()
                : GFStatBar()

           Calls: SHADOWBOX()   (function in SHADOWB.PRG)

   GFStatBar() displays a graph showing percentage complete in
   conjunction with the COPY TO or APPEND FROM commands.  GFStatInit()
   *must* be called prior to GFStatBar() in order to properly initialize
   the counters used by GFStatBar().

   The easiest way to use these functions is by utilizing the directives
   in GRUMP.CH.  See the Norton Guide for examples.
*/

#include "grump.ch"

static nNumRecs     // total number of records in current database
static nRecs        // number of records processed
static nGraphLen    // length of graph
static aBuffer      // screen beneath the status window (save / restore)


/*
    Function:   GFStatInit()
    Purpose:    Draws status bar window and initializes counters
    Syntax:     INITIALIZE STATUS BAR ;
                [MESSAGE <msg>] [ROW <row>] [FILE <file>]
    Parameters: <cMessage> == message to be displayed in the window
                              (default: "Copy Progress")
                <nRow> == top row at which to display status window
                <cFile> == name of file to append from (this is
                           REQUIRED if you are planning to append)
                <nCount>== number of records/iterations.  By default,
                           GFStatBar() assumes that you are processing
                           all records in the database.  If you want
                           to process only a subset of the database,
                           you should determine in advance the number
                           of records in that subset and then pass it
                           as this parameter.  Also, if you are planning
                           to append from an ASCII file, you should
                           determine the number of lines in the file
                           and then pass that as <nCount>.

                All parameters are optional.

    Returns:    NIL
*/
function GFStatInit(cMessage, nRow, cFile, nCount)
local nLeft := ( maxcol() / 2 ) - 30
local xx
local nWorkArea
nGraphLen := nRecs := 0
if cFile <> NIL
   nWorkArea := select()
   use (cFile) shared new alias stat9876
   nNumRecs := lastrec()
   dbCloseArea()
   select (nWorkArea)
elseif nCount <> NIL
   nNumRecs := nCount
else
   nNumRecs := lastrec()
endif
default nRow to ( maxrow() / 2 ) - 3
default cMessage to "Copy Progress"
GFSaveEnv()
ColorSet(C_MESSAGE)
dispbegin()
aBuffer := shadowbox(nRow, nLeft, nRow + 6, nLeft + 63, 2)
@ nRow + 3, nLeft + 2  ssay ''
@ nRow + 3, nLeft + 61 ssay ''
@ nRow + 4, nLeft + 2  ssay '' + replicate("", 9) + ';'
@ nRow + 5, nLeft + 2  ssay '0'
for xx := 1 to 10
   @ nRow + 5, nLeft + xx * 6 ssay str(10 * xx, 3)
next
SCRNCENTER(nRow + 1, cMessage)
ColorSet(C_APICK_STATUSBAR)
@ nRow + 3, nLeft + 3 ssay replicate(chr(177), 58)
setpos(nRow + 3, nLeft + 3)
dispend()
ColorSet(C_APICK_INDICATOR)
return NIL

* end function GFStatInit()
*--------------------------------------------------------------------*


/*
    Function: GFStatBar()
    Purpose:  Display in-progress status
    Returns:  True (.T.)
*/
function gfstatbar
if ++nRecs < nNumRecs
   // draw more of the graph if the relative position has changed
   do while nGraphLen < int(nRecs / nNumRecs * 58)
      dispout(chr(219))
      nGraphLen++
   enddo
else
   dispout(chr(219))
   inkey(.5)
   byebyebox(aBuffer)
   GFSaveEnv()
endif
return .t.

* end function GFStatBar()
*--------------------------------------------------------------------*

* eof statbar.prg
